/*  File Version: 4.0.4
 *	Copyright, David Book, buzztouch.com
 *  Updates made by Chris Ruddell, 2/4/2017
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are 
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the 
 *	following disclaimer. 
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list 
 *	of conditions and the following disclaimer in the documentation and/or other materials 
 *	provided with the distribution. 
 *
 *	Neither the name of David Book, or buzztouch.com nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without specific 
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *	OF SUCH DAMAGE. 
 */
package com.buzztouch.core

import androidx.fragment.app.Fragment
import com.buzztouch.database.BT_repository.getScreenById
import com.buzztouch.database.BT_repository.getCachedScreen
import com.buzztouch.ui.bt_screens.BT_fragment_missing.Companion.newInstance
import com.buzztouch.data_models.BT_item
import com.buzztouch.data_models.BT_user
import com.buzztouch.utils.BT_debugger
import org.json.JSONObject
import org.json.JSONArray
import com.buzztouch.data_models.BT_item.onScreenDataReadyListener
import com.buzztouch.data_models.BT_item.onAllScreenDataReadyListener
import com.buzztouch.buzztouch_appDelegate
import com.buzztouch.extensions.*
import com.buzztouch.utils.BT_utils
import com.buzztouch.ui.bt_screens.BT_fragment
import org.json.JSONException
import java.lang.Exception
import java.lang.RuntimeException
import java.util.ArrayList

class BT_application {
    // getters and setters
    // properties...
    var themes: ArrayList<BT_item> = ArrayList()
    var tabs: ArrayList<BT_item> = ArrayList()
    var menus: ArrayList<BT_item> = ArrayList()
    var screens: ArrayList<BT_item> = ArrayList()
    var selectedTab = -1
    var itemId = ""
    var itemType = ""
    var buzztouchAppId = ""
    var buzztouchAPIKey = ""
    var dataURL = ""
    var reportToCloudURL = ""
    var registerForPushURL = ""
    var pushRegistrationId = ""
    var name = ""
    private var versionString = ""
    var configVersion = 0.1f
    var currentMode = ""
    var startLocationUpdates = false
    var promptForPushNotifications = "0"
    var allowRotation = ""
    var jsonVars = ""
    var homeScreenItemId = ""

    // objects that don't change during app's lifecycle...
    var rootTheme: BT_item = BT_item()
    var rootDevice: BT_device = BT_device()
    var rootUser: BT_user = BT_user()

    // objects that change during navigation...
    var currentScreenData: BT_item = BT_item()
    var previousScreenData: BT_item = BT_item()
    var currentMenuItemData: BT_item = BT_item()
    var previousMenuItemData: BT_item = BT_item()
    var currentItemUpload: BT_item = BT_item()

    fun getScreenDataByItemId(theScreenItemId: String, listener: onScreenDataReadyListener) {
        BT_debugger.logInfo(TAG, "getScreenDataByItemId:$theScreenItemId")
        Thread {
            val item = getScreenById(theScreenItemId)
            runOnUi(Runnable { listener.onDataReady(item) })
        }.start()
    }

    // getScreenDataByItemId...
    fun getScreenDataByItemId(theScreenItemId: String?): BT_item? {
        return screens.firstOrNull { it.itemId == theScreenItemId }
    }

    fun getCachedScreenByItemId(theScreenItemId: String?): BT_item? {
        return getCachedScreen(theScreenItemId!!)
    }

    fun getScreenDataByItemNickname(theScreenItemNickname: String?, listener: onScreenDataReadyListener) {
        listener.onDataReady(screens.findByNickname(theScreenItemNickname ?: ""))
    }

    // getScreenDataByItemNickname...
    fun getScreenDataByItemNickname(theScreenNickname: String?): BT_item? {
        return screens.findByNickname(theScreenNickname ?: "")
    }

    // getThemeDataByItemId...
    fun getThemeDataByItemId(theThemeItemId: String): BT_item? {
        BtLog("$TAG:getThemeDataByItemId $theThemeItemId")
        return themes.firstOrNull { it.itemId == theThemeItemId }
    }

    // getMenuItemsById...
    fun getMenuItemsById(theMenuId: String): JSONArray? {
        return menus.firstOrNull { it.itemId == theMenuId }?.childItems?.toJson()?.jsonArray()
    }

    // Gets the first screen in the JSON file.
    val firstScreen: BT_item
        get() {
            BtLog("getFirstScreen() from " + screens.size + " screens")
            return screens.firstItem() ?: errorScreen
        }

    fun getAllScreens(listener: onAllScreenDataReadyListener) {
        listener.onDataReady(screens)
    }

    // gets screen data for the home screen.  This runs on a background thread and returns via the injected listener.
    fun getHomeScreenData(listener: onScreenDataReadyListener) {
        BT_debugger.logInfo(TAG, "getHomeScreenData(listener)")
        Thread {
            var homeScreenItemId = buzztouch_appDelegate.rootApp.homeScreenItemId
            if (homeScreenItemId.isNotEmpty()) {
                BtLog("$TAG: homeScreenItemId property found on root object:$homeScreenItemId")
                val homeScreen = getScreenById(homeScreenItemId)
                if (homeScreen != null) homeScreen.isHomeScreen = true
                BT_debugger.logInfo(TAG, "homeScreen:" + homeScreen!!.jsonObject.toString())
                BT_utils.runOnUiThread(buzztouch_appDelegate.getContext()) {
                    listener.onDataReady(homeScreen)
                }
            } else {
                if (tabs.size > 0) {
                    BT_debugger.logInfo(TAG, "homeScreenItemId not found - using first tab")
                    homeScreenItemId = tabs.firstOrNull()?.homeScreenItemId ?: ""
                    val homeScreen = getScreenById(homeScreenItemId)
                    BT_utils.runOnUiThread(buzztouch_appDelegate.getContext()) {
                        listener.onDataReady(homeScreen)
                    }
                } else {
                    BT_debugger.logInfo(
                        TAG,
                        "homeScreenItemId not found - using first screen in database"
                    )
                    val homeScreen = firstScreen

                    BT_debugger.logInfo(TAG, "found screen:" + homeScreen.jsonObject.toString())
                    BT_utils.runOnUiThread(buzztouch_appDelegate.getContext()) {
                        listener.onDataReady(homeScreen)
                    }
                }
            }
        }.start()
    }// look at this tab...

    // did we find it?

    val homeScreenData: BT_item
        get() {
            BtLog(TAG + ": getHomeScreenData()")
            val homeScreenItemId = tabs.firstOrNull()?.homeScreenItemId ?: this.homeScreenItemId
            val homeScreen = homeScreenItemId.getIfSet()?.let { getScreenDataByItemId(it) } ?: screens.firstOrNull()

            return homeScreen ?: errorScreen
        }

    // initPluginWithScreenData. Returns a Fragment for a plugins JSON...
    fun initPluginWithScreenData(theScreenData: BT_item?): Fragment? {
        var screenData = theScreenData
        if (screenData == null) {
            RuntimeException("initPluginWithScreenData(): theScreenData is null!").printStackTrace()
            screenData = errorScreen
        }
        BtLog(TAG + ":initPluginWithScreenData. Creating plugin with JSON itemId: \"" + screenData.itemId + "\" itemType: \"" + screenData.itemType + "\" itemNickname: \"" + screenData.itemNickname + "\"")

        // get the item type for this screen so we can instantiate the proper fragment (plugin)...
        val itemType = screenData.itemType

        // get screen data for this itemId...
        var theFragmentClass = "com.buzztouch.ui.bt_screens.$itemType"
        var theFrag: Fragment? = null
        var foundFrag = false
        try {
            theFrag = Class.forName(theFragmentClass).newInstance() as Fragment
            foundFrag = true
        } catch (e: Exception) {
            BtLog("Could not find screen $itemType in ui.bt_screens, trying ui.bt_plugins package")
            theFragmentClass = "com.buzztouch.ui.bt_plugins.$itemType"
            foundFrag = true
            try {
                theFrag = Class.forName(theFragmentClass).newInstance() as Fragment
            } catch (e1: Exception) {
                e1.printStackTrace()
                foundFrag = false
                BtLog(TAG + ":initPluginWithScreenData EXCEPTION creating fragment: " + e.toString())
            }
        }

        //return the "missing plugin" fragment if we could not find the plugin for this itemType...
        if (!foundFrag) {
            try {
                theFrag = newInstance(itemType)
            } catch (e: Exception) {
                // ignore...
            }
        }

        // call setScreenData in BT_fragment
        if (theFrag is BT_fragment) theFrag.setScreenData(screenData)

        //return...
        return theFrag
    }

    // getErrorScreen...
    val errorScreen: BT_item
        get() = getErrorScreen("")

    fun getErrorScreen(errorMessage: String?): BT_item {
        BtLog("$TAG: getErrorScreen")

        // create JSON and values for BT_plugin_missing screen...
        val tmpScreen = BT_item()
        tmpScreen.itemId = "n/a"
        tmpScreen.itemNickname = "Plugin Missing"
        tmpScreen.itemType = "BT_fragment_missing"
        var tmpJson: JSONObject? = null
        try {
            tmpJson = JSONObject()
            tmpJson.put("itemId", "n/a")
            tmpJson.put("itemType", "BT_fragment_missing")
            tmpJson.put("itemNickname", "Plugin Missing")
            tmpJson.put("navBarTitleText", "Plugin Missing")
            tmpJson.put("errorMessage", errorMessage)
        } catch (e: JSONException) {
            // ignore...
        }
        tmpScreen.jsonObject = tmpJson ?: JSONObject()

        //return...
        return tmpScreen
    }

    fun setVersion(version: Float) {
        versionString = "" + version
        configVersion = version
    }

    companion object {
        private const val TAG = "BT_application"
    }
}