/*  File Version: 4.0.1
 *	Copyright, David Book, buzztouch.com
 *  Updates made by Chris Ruddell, 1/6/17
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are 
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the 
 *	following disclaimer. 
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list 
 *	of conditions and the following disclaimer in the documentation and/or other materials 
 *	provided with the distribution. 
 *
 *	Neither the name of David Book, or buzztouch.com nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without specific 
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *	OF SUCH DAMAGE. 
 */
package com.buzztouch.data_models

import android.graphics.Bitmap
import androidx.room.Entity
import androidx.room.Ignore
import androidx.room.PrimaryKey
import com.buzztouch.extensions.BtLog
import org.json.JSONObject
import java.util.ArrayList

@Entity
data class BT_item(
    @PrimaryKey(autoGenerate = true) var _id: Int? = null,
    var itemIndex: Int = -1,
    var itemId: String = "",
    var itemNickname: String = "",
    var itemType: String = "",
    var imageName: String = "",
    var imageURL: String = "",
    var sortName: String = "",
    var isHomeScreen: Boolean = false,
    var jsonVars: String? = null,
    var hasChildItems: Boolean = false,
    var splashScreenItemId: String? = null,
    var navBarStyle: String? = null,
    var statusBarStyle: String? = null,
    var backgroundColor: String? = null,
    var parentItemGuid: String? = null,
    var homeScreenItemId: String? = null,
    @Ignore var jsonObject: JSONObject = JSONObject(),
    @Ignore var image: Bitmap? = null,
    @Ignore var childItems: List<BT_item>? = null,
    var type: Type? = null
) {

    fun initFromDb(allChildItems: List<BT_item>): BT_item {
        BtLog("initFromDb(): $jsonVars")
        val initialPass = this.copy(_id = null, jsonObject = JSONObject(jsonVars ?: "{}"), childItems = allChildItems.filter { it.parentItemGuid == this.itemId })
        return initialPass.copy(hasChildItems = !initialPass.childItems.isNullOrEmpty())
    }

    interface onScreenDataReadyListener {
        fun onDataReady(item: BT_item?)
    }

    interface onAllScreenDataReadyListener {
        fun onDataReady(items: ArrayList<BT_item>?)
    }

    enum class Type {
        Theme,
        Tab,
        Menu,
        Screen,
        ChildItem
    }
}