package com.buzztouch.database

import android.content.Context
import androidx.room.Database
import androidx.room.Room
import androidx.room.RoomDatabase
import androidx.room.TypeConverters
import androidx.room.migration.Migration
import androidx.sqlite.db.SupportSQLiteDatabase
import com.buzztouch.data_models.BT_item
import com.buzztouch.data_models.db_models.BtRootItem
import com.buzztouch.data_models.db_models.BtScreen

@Database(entities = [BT_item::class, BtRootItem::class], version = BT_roomDatabase.DB_VERSION)
@TypeConverters(BT_dbTypeConverters::class)
abstract class BT_roomDatabase : RoomDatabase() {

    abstract fun dao(): BT_dao
    abstract fun configDao(): AppConfigDao

    companion object {
        const val DB_VERSION = 5005 // Buzztouch Core v.5; DB Schema v.005 ==> 5005

        private const val DB_NAME = "buzztouch.db"
        @Volatile private var INSTANCE: BT_roomDatabase? = null

        @Synchronized
        operator fun get(context: Context): BT_roomDatabase {
            val db = INSTANCE ?: create(context)
            if (INSTANCE == null) INSTANCE = db
            return db
        }

        // creates an instance of the database.  Will destroy old databases whenever a new version is set
        // alternatively, use an explicit migration strategy
        private fun create(context: Context): BT_roomDatabase =
            Room.databaseBuilder(context.applicationContext, BT_roomDatabase::class.java, DB_NAME).fallbackToDestructiveMigration().build()

        fun getScreenById(context: Context, itemId: String): BT_item? =
            BT_roomDatabase[context].dao().getById(itemId)

        fun insert(context: Context, item: BT_item) = BT_roomDatabase[context].dao().insert(item)
    }
}
