package com.buzztouch.extensions

import android.content.Context
import androidx.annotation.DrawableRes
import androidx.annotation.StringRes
import androidx.compose.runtime.Composable
import androidx.compose.ui.platform.ComposeView
import com.buzztouch.R
import com.buzztouch.utils.BT_alertUtils
import com.google.android.gms.common.ConnectionResult
import com.google.android.gms.common.GoogleApiAvailability

suspend fun Context.showConfirm(@StringRes title: Int? = null, @StringRes message: Int? = null, @DrawableRes icon: Int = R.drawable.bt_blank) =
    BT_alertUtils.showConfirm(
        context = this,
        title = title?.let { getString(it) } ?: "",
        message = message?.let { getString(it) } ?: "",
        icon = icon
    )

fun Context.googlePlayStatus() = try {
    GoogleApiAvailability.getInstance().isGooglePlayServicesAvailable(this)
} catch (e: java.lang.Exception) {
    BtLog("Context.googlePlayStatus(): EXCEPTION: $e")
    null
}

fun Context?.playServicesAvailable(): Boolean = this?.googlePlayStatus() == ConnectionResult.SUCCESS

/**
 * Helper method to create a Jetpack Compose view from a valid context
 */
fun Context.composeView(width: Int = MATCH_PARENT, height: Int = WRAP_CONTENT, block: @Composable () -> Unit): ComposeView {
    val composeView = ComposeView(this).apply {
        setSize(width, height)
        setContent(block)
    }

    return composeView
}
