package com.buzztouch.extensions

import android.content.Context
import android.content.ContextWrapper
import android.content.res.AssetManager
import com.buzztouch.buzztouch_appDelegate
import com.buzztouch.utils.BT_fileManager
import java.io.File
import java.io.InputStream
import java.io.OutputStream
import java.lang.Exception
import java.net.URI

fun String.copyCachedFileToSdCard(): File? =
    if (!BT_fileManager.copyFileFromCacheToSDCard(this)) null
    else FileKt.cacheFile(this).let { if (it.exists()) it else null }

fun String.copyAssetToCache(folder: String = "BT_Docs") = BT_fileManager.copyAssetToCache(folder, this)

fun String.projectAssetExists(folder: String): Boolean = BT_fileManager.doesProjectAssetExist(folder, this)

class FileKt {
    companion object {
        fun cacheFile(filename: String) = File(buzztouch_appDelegate.getApplication().externalCacheDir, filename)
    }
}

fun AssetManager.readText(filename: String): String? = try {
    open(filename).readText()
} catch (e: Exception) {
    e.printStackTrace()
    null
}

fun InputStream.readText(): String? = try {
    reader().let { reader -> reader.readText().also { reader.close() } }
} catch (e: Exception) {
    e.printStackTrace()
    null
}

fun OutputStream.writeText(txt: String): Boolean = try {
    writer().apply {
        write(txt)
        flush()
        close()
    }
    true
} catch (e: Exception) {
    e.printStackTrace()
    false
}

fun ContextWrapper.writeTextToCache(filename: String, txt: String): Boolean =
    openFileOutput(filename, Context.MODE_PRIVATE).writeText(txt)
