package com.buzztouch.extensions

import android.content.Intent
import android.content.pm.ApplicationInfo
import android.content.pm.PackageManager
import android.graphics.drawable.Drawable
import android.net.Uri
import androidx.annotation.DrawableRes
import androidx.annotation.StringRes
import androidx.core.content.ContextCompat
import androidx.core.content.FileProvider
import androidx.fragment.app.Fragment
import com.buzztouch.buzztouch_appDelegate
import com.buzztouch.BuildConfig
import com.buzztouch.R
import com.buzztouch.data_models.BT_item
import com.buzztouch.ui.bt_screens.BT_fragment
import com.buzztouch.utils.BT_strings
import java.io.File

suspend fun BT_fragment.downloadScreenData(dataUrl: String, cacheFileName: String? = null): String {
    showProgress(null, null)
    val useURL = BT_strings.mergeBTVariablesInString(dataUrl)
    val response = useURL.readTextFromUrl(cacheFileName)
    hideProgress()

    return response
}

fun Fragment.getDrawable(@DrawableRes id: Int): Drawable? = context?.let { ContextCompat.getDrawable(it, id) }

suspend fun BT_fragment.loadScreen(tappedItem: BT_item) {
    val itemId = tappedItem.propertyValue("loadScreenWithItemId")
    val itemNickname = tappedItem.propertyValue("loadScreenWithNickname")
    val loadScreenObject = tappedItem.loadScreenObject()
    val screenToLoad: BT_item? = when {
        itemId.isSet() -> loadScreenByItemId(itemId)
        itemNickname.isSet() -> loadScreenByNickname(itemNickname)
        loadScreenObject != null -> loadScreenObject
        else -> null
    }

    screenToLoad?.let {
        loadScreenObject(tappedItem, it)
    } ?: showAlert(getString(R.string.errorTitle), getString(R.string.errorNoScreenConnected))
}

fun BT_fragment.showProgress() = showProgress(null, null)

fun BT_fragment.showAlert(@StringRes title: Int?, @StringRes body: Int?) = showAlert(title?.let { getString(it) } ?: "", body?.let { getString(it) } ?: "")

fun BT_fragment.showErrorAlert(@StringRes body: Int?) = showAlert(R.string.errorTitle, body)

fun BT_fragment.showToast(@StringRes message: Int, duration: String = "short") = showToast(getString(message), duration)

fun Fragment.shareUrlViaEmail(url: Url) {
    // tell Android launch the native email application...
    val emailIntent = Intent(Intent.ACTION_SEND)
    emailIntent.type = "plain/text"
    emailIntent.putExtra(Intent.EXTRA_SUBJECT, getString(R.string.sharingWithYou))
    emailIntent.putExtra(Intent.EXTRA_TEXT, url)

    startActivity(Intent.createChooser(emailIntent, getString(R.string.openWithWhatApp)))
}

fun Fragment.shareFileViaEmail(file: File) {
    val context = context ?: return
    val path = "content://${file.absolutePath}"
    val uri = FileProvider.getUriForFile(context, BuildConfig.APPLICATION_ID + ".provider", file)
    BtLog("shareFileViaEmail(): $path")
    // tell Android launch the native email application...
    val emailIntent = Intent(Intent.ACTION_SEND)
    emailIntent.type = "text/plain"
    emailIntent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
    emailIntent.putExtra(Intent.EXTRA_SUBJECT, getString(R.string.sharingWithYou))
    emailIntent.putExtra(Intent.EXTRA_TEXT, getString(R.string.attachedFile))
    emailIntent.putExtra(Intent.EXTRA_STREAM, uri)

    startActivity(Intent.createChooser(emailIntent, getString(R.string.openWithWhatApp)))
}

fun BT_fragment.propertyValue(key: String, default: String = ""): String = screenData.propertyValue(key, default)

fun BT_fragment.booleanValue(key: String, default: Boolean = false): Boolean = screenData.booleanValue(key, default)

fun BT_fragment.intValue(key: String, default: Int = 0) = propertyValue(key, "$default").toIntOrNull() ?: default

val BT_fragment.packageName get() = buzztouch_appDelegate.getApplication().packageName

val BT_fragment.applicationInfo: ApplicationInfo? get() =
    try {
        buzztouch_appDelegate.getApplication().packageManager.getApplicationInfo(packageName, PackageManager.GET_META_DATA)
    } catch (e: Exception) {
        BtLog("$fragmentName: Unable to get application info for $packageName")
        null
    }

fun BT_fragment.getMetaInfo(key: String): String? = applicationInfo?.metaData?.get(key) as? String