package com.buzztouch.extensions

import android.content.Context
import android.graphics.Bitmap
import android.graphics.drawable.BitmapDrawable
import android.widget.ImageView
import coil.imageLoader
import coil.load
import coil.request.ImageRequest
import java.io.File

suspend fun Context.downloadImage(url: String): BitmapDrawable? {
    val imageLoader = this.imageLoader
    val request = ImageRequest.Builder(this)
        .data(url)
        .allowHardware(false)
        .build()

    return imageLoader.execute(request).drawable as? BitmapDrawable
}

suspend fun ImageView.downloadImage(url: String) {
    this.load(url)
}

suspend fun ImageView.loadImageResource(res: Int) {
    this.load(res)
}

suspend fun ImageView.loadImageFromFile(file: File) {
    this.load(file)
}

fun Bitmap.writeToFile(file: File, format: Bitmap.CompressFormat = Bitmap.CompressFormat.JPEG, quality: Int = 90) {
    file.writeBitmap(this, format, quality)
}

fun File.writeBitmap(bitmap: Bitmap, format: Bitmap.CompressFormat, quality: Int) {
    outputStream().use { out ->
        bitmap.compress(format, quality, out)
        out.flush()
    }
}
