package com.buzztouch.extensions

import com.google.gson.Gson
import org.json.JSONArray
import org.json.JSONException
import org.json.JSONObject
import java.lang.Exception

fun JSONObject.getJsonObjectOrNull(key: String): JSONObject? = if (this.has(key))
    try { this.getJSONObject(key) } catch (e: JSONException) { null }
else null

fun JSONObject.getJsonArrayOrNull(key: String): JSONArray? = if (this.has(key))
    try { this.getJSONArray(key) } catch (e: JSONException) { null }
else null

fun JSONArray.getJsonObjectOrNull(index: Int): JSONObject? =
    try { this.getJSONObject(index) } catch (e: JSONException) { null }

fun JSONObject?.getStringOrNull(key: String): String? = if (this?.has(key) == true)
    try { this.getString(key) } catch (e: JSONException) { null }
else null

fun JSONObject.getBooleanOrNull(key: String): Boolean? = if (this.has(key))
    try { this.getBoolean(key) } catch (e: JSONException) { null }
else null

fun JSONObject.getArrayOrNull(key: String): JSONArray? = if (this.has(key))
    try { this.getJSONArray(key) } catch (e: JSONException) { null } else null

fun String.jsonObject(): JSONObject? = try { JSONObject(this) } catch (e: JSONException) { null }

fun String.jsonArray(): JSONArray? = try { JSONArray(this) } catch (e: Exception) { null }

fun JSONArray.listObjects(): List<JSONObject> = (0 until this.length()).mapNotNull { getJsonObjectOrNull(it) }

fun JSONObject.listObjects(key: String): List<JSONObject> = getArrayOrNull(key)?.listObjects() ?: listOf()

fun <R> JSONArray.mapObj(transform: (JSONObject) -> R?): List<R> = listObjects().mapNotNull { transform(it) }

fun JSONArray.firstObject(): JSONObject? = this.getJSONObject(0)

inline fun <reified T> T.toJson(): String = Gson().toJson(this)
inline fun <reified T> String.fromJson(): T? = try { Gson().fromJson(this, T::class.java) } catch (e: Exception) { null }

fun JSONObject.getIfSet(): JSONObject? = if (this.length() == 0) null else this
