package com.buzztouch.extensions

import com.buzztouch.data_models.BT_item
import com.buzztouch.database.BT_repository
import com.buzztouch.ui.bt_screens.BT_fragment
import com.buzztouch.utils.BT_strings
import org.json.JSONObject

fun BT_item?.propertyValue(key: String, defaultValue: String = ""): String =
    this?.jsonObject?.getStringOrNull(key)
        ?: this?.jsonVars?.jsonObject()?.getStringOrNull(key)
        ?: defaultValue

fun BT_item?.booleanValue(key: String): Boolean? = this?.propertyValue(key)?.let { it == "1" }
fun BT_item?.booleanValue(key: String, default: Boolean): Boolean = this?.propertyValue(key)?.let { it == "1" } ?: default

fun BT_item?.styleValue(key: String, defaultValue: String = ""): String =
    BT_strings.getStyleValueForScreen(this, key, defaultValue)

fun BT_item?.styleValueInt(key: String, defaultValue: Int): Int =
    BT_strings.getStyleValueForScreen(this, key, defaultValue.toString())?.toIntOrNull() ?: defaultValue

fun BT_item.loadScreenObject(): BT_item? {
    return if (jsonObject?.has("loadScreenObject") == true) jsonObject?.getJsonObjectOrNull("loadScreenObject")?.let { obj ->
        BT_item().also {
            itemId = obj.getStringOrNull("itemId") ?: ""
            itemNickname = obj.getStringOrNull("itemNickname") ?: ""
            itemType = obj.getStringOrNull("itemType") ?: ""
            jsonObject = obj.getJsonObjectOrNull("loadScreenObject") ?: JSONObject()
        }
    } else null
}

fun JSONObject.basicBtItem(): BT_item = BT_item().apply {
    itemId = this@basicBtItem.getStringOrNull("itemId") ?: ""
    itemType = this@basicBtItem.getStringOrNull("itemType") ?: ""
    itemNickname = this@basicBtItem.getStringOrNull("itemNickname") ?: ""
}

fun BT_item.hasChildItems(): Boolean = jsonObject?.getBooleanOrNull(BT_repository.KEY_CHILD_ITEMS) == true

fun List<BT_item>.firstItem(): BT_item? = minByOrNull { it.itemIndex }

fun List<BT_item>.findByNickname(nickname: String): BT_item? = firstOrNull { it.itemNickname == nickname }
