package com.buzztouch.extensions

import com.buzztouch.utils.BT_fileManager
import io.ktor.client.*
import io.ktor.client.call.body
import io.ktor.client.engine.cio.*
import io.ktor.client.request.*
import io.ktor.client.request.forms.*
import io.ktor.client.statement.*
import io.ktor.http.*
import io.ktor.utils.io.*

suspend fun String.readTextFromUrl(saveAsFileName: String? = null): String {
    val client = HttpClient(CIO)

    val response: HttpResponse = client.get(this) {}
    client.close()

    saveAsFileName?.getIfSet()?.let {
        BT_fileManager.saveTextFileToCache(response.toString(), it)
    }
    return response.toString()
}

suspend fun readTextFromUrl(url: String): Pair<Int, String> {
    val client = HttpClient(CIO)

    //val response = client.get<String>(url) {}
    val response: HttpResponse = client.request(url)
    val status = response.status.value
    val body = response.bodyAsText()
    client.close()

    return Pair(status, body)
}

suspend inline fun <reified T> httpGet(url: String): String {
    val client = HttpClient(CIO)
    val response: HttpResponse = client.request(url)
    client.close()

    return response.bodyAsText()
}

suspend inline fun <reified T> Any.postToUrl(url: String): String {
    val client = HttpClient(CIO)

    val response: HttpResponse = client.post(url) {
        setBody(this@postToUrl)
    }
//    val response: T = client.post(url) {
//        this.body = this@postToUrl
//    }
    client.close()

    return response.bodyAsText()
}

suspend inline fun <reified T> Any.putToUrl(url: String): String {
    val client = HttpClient(CIO)

    val response: HttpResponse = client.put(url) {
        setBody(this@putToUrl)
    }
    client.close()

    return response.bodyAsText()
}

suspend inline fun <reified T>httpDelete(url: String): String {
    val client = HttpClient(CIO)
    val response: HttpResponse = client.delete(url)
    client.close()

    return response.bodyAsText()
}

suspend inline fun <reified T> HashMap<String, String>.postForm(url: String): String {
    val client = HttpClient(CIO)
    val response: HttpResponse = client.post(url) {
        setBody(FormDataContent(
            Parameters.build {
                this@postForm.keys.forEach { key ->
                    append(key, this@postForm[key] ?: "")
                }
            }
        ))
    }
    client.close()

    return response.body()
}
