package com.buzztouch.extensions

import android.graphics.Color
import com.buzztouch.utils.BT_fileManager
import com.buzztouch.utils.BT_preferencesKt
import com.buzztouch.utils.BT_strings
import org.json.JSONException
import org.json.JSONObject

typealias Url = String

fun String.hexColor(): Int = when {
    this.equals("clear", ignoreCase = true) -> Color.TRANSPARENT
    this.equals("stripes", ignoreCase = true) -> Color.TRANSPARENT
    else -> try { Color.parseColor(this) } catch (e: Exception) { e.printStackTrace(); Color.BLACK }
}

fun JSONObject?.propertyValue(key: String, defaultValue: String = ""): String =
    if (this == null || !this.has(key)) defaultValue
    else try { this.getString(key) } catch (e: JSONException) { e.printStackTrace(); defaultValue }

fun String.doesCachedFileExist(): Boolean = BT_fileManager.doesCachedFileExist(this)

fun String.readCachedFile(): String = BT_fileManager.readTextFileFromCache(this)

fun String?.isSet(): Boolean = !this.isNullOrEmpty()

fun String.mergeBtVariables() = BT_strings.mergeBTVariablesInString(this)

fun String.equalsIgnoreCase(comparison: String) = this.equals(comparison, true)

fun String.getAsStringPref() = BT_preferencesKt.getString(this)
fun String.getAsIntPref() = BT_preferencesKt.getInt(this)

fun String?.getIfSet(): String? = this?.let { if (isNullOrEmpty()) null else this }
