package com.buzztouch.extensions

import android.content.Context
import android.view.ViewGroup
import com.buzztouch.buzztouch_appDelegate
import com.buzztouch.core.BT_device
import com.buzztouch.data_models.BT_item
import com.buzztouch.data_models.Orientation
import com.buzztouch.utils.BT_debugger
import com.buzztouch.utils.BT_utils
import org.json.JSONObject
import kotlin.coroutines.resume
import kotlin.coroutines.suspendCoroutine

const val MATCH_PARENT = ViewGroup.LayoutParams.MATCH_PARENT
const val WRAP_CONTENT = ViewGroup.LayoutParams.WRAP_CONTENT

val isLargeDevice: Boolean get() = buzztouch_appDelegate.rootApp.rootDevice.isLargeDevice

val orientation: Orientation get() = if (buzztouch_appDelegate.rootApp.rootDevice.deviceOrientation.equals("landscape", ignoreCase = true))
    Orientation.Landscape
else Orientation.Portrait

val rootDevice: BT_device? = buzztouch_appDelegate.rootApp.rootDevice

// if we are in landscape orientation the sizes are backwards (intentionally, see BT_device class)...
val deviceWidth: Int get() = when (orientation) {
    Orientation.Portrait -> buzztouch_appDelegate.rootApp.rootDevice.deviceWidth
    Orientation.Landscape -> buzztouch_appDelegate.rootApp.rootDevice.deviceHeight
}

val deviceHeight: Int get() = when (orientation) {
    Orientation.Portrait -> buzztouch_appDelegate.rootApp.rootDevice.deviceHeight
    Orientation.Landscape -> buzztouch_appDelegate.rootApp.rootDevice.deviceWidth
}

val deviceLatitude: String get() = buzztouch_appDelegate.rootApp.rootDevice.deviceLatitude
val deviceLongitude: String get() = buzztouch_appDelegate.rootApp.rootDevice.deviceLongitude

val canSendEmail: Boolean get() = rootDevice?.canSendEmail() ?: false
val canUseGps: Boolean get() = rootDevice?.canUseGPS() ?: false

val appContext: Context get() = buzztouch_appDelegate.theApplication.applicationContext

suspend fun loadScreenByItemId(itemId: String?): BT_item? = buzztouch_appDelegate.rootApp.getCachedScreenByItemId(itemId)

suspend fun loadScreenByNickname(itemNickname: String?): BT_item? =
    buzztouch_appDelegate.rootApp.getScreenDataByItemNickname(itemNickname)

fun loadScreenByObject(obj: JSONObject?, objName: String = "loadScreenObject"): BT_item? {
    obj ?: return null
    if (!obj.has(objName)) return null
    val tmpLoadScreen = obj.getJsonObjectOrNull("loadScreenObject") ?: JSONObject()
    return BT_item().apply {
        itemId = tmpLoadScreen.getStringOrNull("itemId") ?: ""
        itemNickname = tmpLoadScreen.getStringOrNull("itemNickname") ?: ""
        itemType = tmpLoadScreen.getStringOrNull("itemType") ?: ""
        jsonObject = tmpLoadScreen
    }
}

fun BtLog(message: String) = BT_debugger.showIt(message)

fun runSafe(log: String? = null, runnable: () -> Unit) = try { log?.let { BtLog(it) }; runnable() } catch (e: Exception) { BtLog("Caught Error: ${e.localizedMessage}"); e.printStackTrace() }

fun runOnUi(runnable: Runnable) = buzztouch_appDelegate.getContext()?.let { BT_utils.runOnUiThread(it, runnable) }

fun runOnUi(runnable: () -> Unit) = buzztouch_appDelegate.getContext()?.let { BT_utils.runOnUiThread(it, runnable) }
