/*  File Version: 4.0.1
 *	Copyright, Chris Ruddell, buzztouch.com
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are 
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the 
 *	following disclaimer. 
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list 
 *	of conditions and the following disclaimer in the documentation and/or other materials 
 *	provided with the distribution. 
 *
 *	Neither the name of David Book, or buzztouch.com nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without specific 
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *	OF SUCH DAMAGE. 
 */

package com.buzztouch.push_notifications;

import android.content.Context;
import android.net.Uri;
import android.os.AsyncTask;

import com.buzztouch.buzztouch_appDelegate;
import com.buzztouch.utils.BT_debugger;
import com.buzztouch.utils.BT_preferences;
import com.buzztouch.utils.BT_strings;

import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.HashMap;

@Deprecated
public class BT_gcmServerUtils{

    public static String serverURL = buzztouch_appDelegate.rootApp.getRegisterForPushURL();
    public static HashMap<String, String> encodedFormValues;
    public static String postCommand;
    public static String tmpDeviceToken;

    // setupPostValues...
    public static void setupPostValues(){

        // build URL...
        if(serverURL.length() > 1) {
            serverURL = BT_strings.mergeBTVariablesInString(serverURL);
            serverURL = BT_strings.encodeCompleteURL(serverURL);

            try{

                // pull apiKey and apiSecret from URL params...
                Uri uri = Uri.parse(serverURL);
                String useApiKey = uri.getQueryParameter("apiKey");
                String useApiSecret = uri.getQueryParameter("apiSecret");
                if (useApiKey == null) useApiKey = "";
                if (useApiSecret == null) useApiSecret = "";

                // create encoded form values
                encodedFormValues = new HashMap<>();
                encodedFormValues.put("command", "registerForPush");
                encodedFormValues.put("gcmCommand", "");
                encodedFormValues.put("appGuid", buzztouch_appDelegate.rootApp.getBuzztouchAppId());
                encodedFormValues.put("apiKey", useApiKey);
                encodedFormValues.put("apiSecret", useApiSecret);
                encodedFormValues.put("deviceId", buzztouch_appDelegate.rootApp.getRootDevice().getDeviceId());
                encodedFormValues.put("deviceLatitude", buzztouch_appDelegate.rootApp.getRootDevice().getDeviceLatitude());
                encodedFormValues.put("deviceLongitude", buzztouch_appDelegate.rootApp.getRootDevice().getDeviceLongitude());
                encodedFormValues.put("deviceModel", BT_strings.urlEncode(buzztouch_appDelegate.rootApp.getRootDevice().getDeviceModel()));
                encodedFormValues.put("userId", buzztouch_appDelegate.rootApp.getRootUser().getUserId());
                encodedFormValues.put("deviceType", "android");
                encodedFormValues.put("deviceToken", tmpDeviceToken);
                encodedFormValues.put("currentMode", buzztouch_appDelegate.rootApp.getCurrentMode());

            }catch(Exception e){}

        }
    }

    // gcmRegisterOnServer...
    public static void gcmRegisterOnServer(final Context context, final String regId) {
        BT_debugger.showIt("BT_gcmServerUtils:gcmRegisterOnServer");

        // set tmpDevice token to the value passed in...
        tmpDeviceToken = regId;

		// setup post values...
        setupPostValues();

        if(serverURL.length() > 1){
            postCommand = "registerDevice";
            encodedFormValues.put("gcmCommand", postCommand);
            new ServerAsyncTask().execute("post");
        }

    }
    
    // gcmUnregisterOnServer...
    public static void gcmUnregisterOnServer(final Context context, final String regId){
        BT_debugger.showIt("BT_gcmServerUtils:gcmUnregisterOnServer");

        // set tmpDevice token to the value passed in...
        tmpDeviceToken = regId;

        // setup post values...
        setupPostValues();

        // must have URL...
        if(serverURL.length() > 1){
            postCommand = "unregisterDevice";
            encodedFormValues.put("gcmCommand", postCommand);
            new ServerAsyncTask().execute("post");
        }

     }


    // //////////////////////////////////////////////////////////////////
	// send a push related HTTP request...
    public static class ServerAsyncTask extends AsyncTask<String, Integer, String> {

        @Override
        protected String doInBackground(String... params){

            // holds results from server...
            String results = "";

            // build body to post with request...
            java.net.HttpURLConnection urlc = null;
            java.io.OutputStreamWriter out = null;
            java.io.DataOutputStream dataout = null;
            java.io.BufferedReader in = null;
            try {
                URL url = new URL(BT_gcmServerUtils.serverURL);
                urlc = (java.net.HttpURLConnection) url.openConnection();
                urlc.setRequestMethod("POST");
                urlc.setDoOutput(true);
                urlc.setDoInput(true);
                urlc.setUseCaches(false);
                urlc.setAllowUserInteraction(false);
                urlc.setRequestProperty("Content-Type","application/x-www-form-urlencoded");
                dataout = new java.io.DataOutputStream(urlc.getOutputStream());
                // perform POST operation
                String encodedValues = BT_strings.convertMapToEncodedString(encodedFormValues);
                dataout.writeBytes(encodedValues);
                in = new java.io.BufferedReader(new java.io.InputStreamReader(urlc.getInputStream()),8096);

                BT_debugger.showIt("sending data to url:" + url);
                BT_debugger.showIt(encodedValues);

                // write html to System.out for debug
                StringBuilder sb = new StringBuilder();
                String line = null;
                while ((line = in.readLine()) != null) {
                    sb.append(line + "\n");
                    //return response;
                }
                results = sb.toString();
                in.close();
                //return response;
            } catch (java.net.ProtocolException e) {
                results = e.toString();
                //return e.toString();
            } catch (IOException e) {
                results = e.toString();
                //return e.toString();
            } finally {
                if (out != null) {
                    try {
                        out.close();
                        //return  response;
                    } catch (IOException e) {
                        results = e.toString();
                        //return e.toString();
                    }
                }
                if (in != null) {
                    try {
                        in.close();
                    } catch (IOException e) {
                        // e.printStackTrace();
                        results = e.toString();
                    }
                }
            }

            BT_debugger.showIt(results);
            BT_debugger.showIt("parsing results...");

            // parse JSON data...
            try{
                if(results.toString().length() > 1){

                    //raw JSON results (entire string returned by server)...
                    JSONObject objRaw = new JSONObject(results.toString());
                    if(objRaw.has("result")){

                        // get "failed" or "valid" from JSON...
                        JSONObject result = objRaw.getJSONObject("result");
                        if(result.has("status")){
                            if(result.getString("status").equalsIgnoreCase("valid")){

                                String deviceToken = result.getString("deviceToken");
                                BT_debugger.showIt("BT_gcmServerUtils:ServerAsyncTask Device Token: " + deviceToken);

                                // if registering...
                                if(postCommand.equals("registerDevice")){
                                    BT_debugger.showIt("BT_gcmServerUtils:ServerAsyncTask DEVICE REGISTERED");

                                    // save deviceToken (GCM Registration Id) in prefs and delegate...
                                    BT_preferences.setPrefString("googleCloudMessagingToken", deviceToken);
                                    buzztouch_appDelegate.rootApp.setPushRegistrationId(deviceToken);


                                }

                                // if unregistering...
                                if(postCommand.equals("unregisterDevice")){
                                    BT_debugger.showIt("BT_gcmServerUtils:ServerAsyncTask DEVICE UNREGISTERED");

                                    //remove deviceToken (GCM Registration Id) from prefs and delegate...
                                    BT_preferences.setPrefString("googleCloudMessagingToken", "");
                                    buzztouch_appDelegate.rootApp.setPushRegistrationId("");

                                }

                            }else{

                                // GCMRegistrar.setRegisteredOnServer(buzztouch_appDelegate.getContext(), false);
                                BT_debugger.showIt("BT_gcmServerUtils:ServerAsyncTask JSON results returned \"failed\" status");

                            }// valid
                        }// status

                    }// objRaw
                }else{

                    BT_debugger.showIt("BT_gcmServerUtils:ServerAsyncTask The server did not return a result?");

                }//results are empty...
            }catch (Exception e) {
                BT_debugger.showIt("BT_gcmServerUtils:ServerAsyncTask EXCEPTION parsing JSON returned from server: " + e.toString());
            }

            // bail...
            return "done";

        }

        @Override
        protected void onPostExecute(String result){
            BT_debugger.showIt("BT_gcmServerUtils:ServerAsyncTask onPostExecute");
        }

    }
    // //////////////////////////////////////////////////////////////////

	
	//returns string from input stream...
	public static StringBuilder convertStreamToText(InputStream is) {
	    String line = "";
	    StringBuilder total = new StringBuilder();
	    
	    // wrap a BufferedReader around the input stream...
	    BufferedReader rd = new BufferedReader(new InputStreamReader(is));

	    //read response until the end...
	    try{
			while ((line = rd.readLine()) != null) { 
			    total.append(line); 
			}
		}catch(IOException e){
		}
	    
	    //return full string
	    return total;
	}

	
	
}





