/*  File Version: 4.0.3
 *	Copyright, David Book, buzztouch.com
 *  Updates made by Chris Ruddell, 1/31/2017
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are 
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the 
 *	following disclaimer. 
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list 
 *	of conditions and the following disclaimer in the documentation and/or other materials 
 *	provided with the distribution. 
 *
 *	Neither the name of David Book, or buzztouch.com nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without specific 
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *	OF SUCH DAMAGE. 
 */
package com.buzztouch.ui.bt_screens;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.content.res.Configuration;
import android.content.res.Resources;
import android.graphics.drawable.ColorDrawable;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.os.Handler;

import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.widget.TextView;
import android.widget.Toast;

import com.buzztouch.buzztouch_appDelegate;
import com.buzztouch.R;
import com.buzztouch.core.BT_configuration;
import com.buzztouch.data_models.BT_item;
import com.buzztouch.location.BT_locationManager;
import com.buzztouch.push_notifications.BT_FirebaseMessagingService;
import com.buzztouch.push_notifications.BT_pushManager;
import com.buzztouch.ui.bt_screens.BT_fragment.BTFragmentResumedListener;
import com.buzztouch.utils.BT_color;
import com.buzztouch.utils.BT_debugger;
import com.buzztouch.utils.BT_permissions;
import com.buzztouch.utils.BT_preferences;
import com.buzztouch.utils.BT_strings;
import com.buzztouch.utils.BT_utils;
import com.buzztouch.utils.BT_viewUtilities;


public class BT_activity_host extends AppCompatActivity implements SensorEventListener, BTFragmentResumedListener {

	public String activityName = "BT_activity_host";
	public String appLastModifiedOnServer = "";

    // spinner and loadingMessage dialogues...
    public ProgressDialog progressSpinner = null;
    public ProgressDialog loadingMessage = null;
    public Handler refreshActionHandler = new Handler();

    // action bar, fragment...
	public ActionBar actionBar;
	public Fragment currentFragment;

//	// location listener...
//	public LocationManager locationManager;
//	public int locationUpdateCount = 0;
//	public String locationListenerType = "";

	// accelerometer...decrease threshold to increase sensitivity...
	public final float SHAKE_THRESHOLD = (float) 7.0;
	public boolean sensorInitialized;
	public SensorManager sensorManager;
	public Sensor accelerometer;
	public float shakeSpeedX, shakeSpeedY;

	private BroadcastReceiver mFirebaseReceiver;

	// task to register with Firebase Messaging...
	public Context thisContext = null;

	//////////////////////////////////////////////////////////////////////////
	// Activity life cycle events.

    // onCreate
	@Override
    public void onCreate(Bundle savedInstanceState){
        super.onCreate(savedInstanceState);
		BT_debugger.showIt(activityName + ":onCreate");

        // remember this context to be used in child classes...
        thisContext = this;

        requestWindowFeatures();

        // save the app's current version to the prefs so we can determine when the app is updated...
        try{
            PackageInfo packageInfo = getApplicationContext().getPackageManager().getPackageInfo(getApplicationContext().getPackageName(), 0);
            String lastAppVersion = BT_preferences.getPrefString("manifestAppVersion");
            if(!lastAppVersion.equalsIgnoreCase(String.valueOf(packageInfo.versionCode))){

                // remember version in the prefs...
				BT_preferences.setPrefString("manifestAppVersion", String.valueOf(packageInfo.versionCode));

                // clear a possible remembered GCM Push Notification token, we need a new one...
				BT_preferences.setPrefString("googleCloudMessagingToken", "");

            }
        }catch(NameNotFoundException e){}

        // inflate the bt_activity_host.xml for the content view...
        setContentView(getLayoutFile());

        // configure toolbar/actionbar
        setupActionBar();

        // configure the app's environment...
        configureEnvironment();

        // create broadcast receiver to monitor push notifications...
		mFirebaseReceiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                BT_debugger.showIt("onReceive");
                if (intent.getAction().equals(BT_FirebaseMessagingService.ACTION_PUSH_NOTIFICATION)) {
                    String message = intent.getStringExtra(BT_FirebaseMessagingService.ARG_NOTIFICATION_MESSAGE);
                    AlertDialog.Builder adb = new AlertDialog.Builder(BT_activity_host.this);
                    adb.setPositiveButton("OK", null);
                    adb.setMessage(message);
                    adb.show();
                }
            }
        };
        IntentFilter filter = new IntentFilter();
        filter.addAction(BT_FirebaseMessagingService.ACTION_PUSH_NOTIFICATION);
        registerReceiver(mFirebaseReceiver, filter, RECEIVER_EXPORTED);

	}

    public void requestWindowFeatures() {
        getWindow().requestFeature(Window.FEATURE_ACTION_BAR);
    }

    public void setupActionBar() {
		BT_debugger.logInfo(this, "setupActionBar");
        // setup the android action bar with a custom view for the title text...
        setSupportActionBar((Toolbar)findViewById(R.id.toolbar));
        if (getSupportActionBar()==null) return;
        actionBar = getSupportActionBar();
        actionBar.setDisplayOptions(ActionBar.DISPLAY_SHOW_TITLE, ActionBar.DISPLAY_USE_LOGO);
        actionBar.setDisplayShowTitleEnabled(true);
        actionBar.setDisplayUseLogoEnabled(true);
        actionBar.setDisplayHomeAsUpEnabled(false);
        actionBar.setHomeButtonEnabled(true);
        actionBar.setTitle(getString(R.string.loading));
    }

    protected int getLayoutFile() {
        return R.layout.bt_activity_host;
    }

	// onStart
	@Override
	public void onStart() {
		BT_debugger.showIt(activityName + ":onStart");
		super.onStart();

		// start location manager?
		
		/*	Location Manager Logic (turn on GPS?)
		 	-------------------------------------
			Should this device report it's location? The device will turn on it's GPS and begin
		 	tracking it's location if three things are true:
		 	1) The application's configuration data is set to "startLocationUpdates" 
		 	2) The user has not turned on "allow location tracking" in a BT_screen_settingsLocation
		 	3) The device is capable of tracking it's location (has GPS)
		 */
		// Save battery! We remember the last reported location in the app's delegate so we don't have to turn on the GSP for every screen...
		if(!buzztouch_appDelegate.foundUpdatedLocation && buzztouch_appDelegate.rootApp.getRootDevice().canUseGPS()){
			if(buzztouch_appDelegate.rootApp.getStartLocationUpdates()){
				BT_debugger.showIt(activityName + ": start GPS is set to YES in the applications configuration data, trying to start GPS");
				if(!BT_preferences.getPrefString("userAllowLocation").equalsIgnoreCase("prevent")){
					BT_debugger.showIt(activityName + ": user has not prevented the GPS from starting using a BT_screen_settingsLocation screen");

					// trigger method...
                    BT_locationManager.INSTANCE.getLastGpsLocation(this);

				}else{
					BT_debugger.showIt(activityName + ": user has prevented the GPS from starting using a BT_screen_settingsLocation screen");
				}
			}else{
				BT_debugger.showIt(activityName + ": start GPS is set to NO in the applications configuration data, not starting GPS");
			}
		}// already found location and saved it in the app's delegate.



	}

    // onResume
    @Override
    public void onResume() {
    	BT_debugger.showIt(activityName + ":onResume");



		// report to cloud...must have dataURL's
		if(BT_configuration.INSTANCE.getCanReportToCloud()){
			this.reportToCloud();
		}else{
	        BT_debugger.showIt(activityName + ":reportToCloud no dataURL and / or reportToCloudURL, both required for remote updates, not reporting.");
		}

    	// call super...
    	super.onResume();
   }

    // onPause
    @Override
    public void onPause() {
		BT_debugger.showIt(activityName + ":onPause");
        super.onPause();

		// handle loading message...
		if(this.loadingMessage != null){
			if(this.loadingMessage.isShowing()){
				this.loadingMessage.hide();
			}
		}

    }

    // onStop
	@Override
	public void onStop(){
		BT_debugger.showIt(activityName + ":onStop");
		super.onStop();

		// hide loading message if it's showing...
		if(this.loadingMessage != null){
			if(this.loadingMessage.isShowing()){
				this.loadingMessage.hide();
			}
		}

	}

	// onDestroy
    @Override
    public void onDestroy() {
		BT_debugger.showIt(activityName + ":onDestroy");
		super.onDestroy();

		unregisterReceiver(mFirebaseReceiver);

		// hide loading message if it's showing...
		if(this.loadingMessage != null){
			if(this.loadingMessage.isShowing()){
				this.loadingMessage.hide();
			}
		}

    }

    // onSaveInstanceState...
    @Override
    public void onSaveInstanceState(Bundle saveState) {
		BT_debugger.showIt(activityName + ":onSaveInstanceState");
    	super.onSaveInstanceState(saveState);
    }

    // onActivityResult...
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
   		BT_debugger.showIt(activityName + ":onActivityResult");
       	super.onActivityResult(requestCode, resultCode, data);
    }

    // end Activity life-cycle events
	//////////////////////////////////////////////////////////////////////////

	//////////////////////////////////////////////////////////////////////////
	// configureEnvironment (can't do this until after app's data is loaded...
	public void configureEnvironment(){
		BT_debugger.showIt(activityName + ":configureEnvironment");

		// push
		BT_pushManager.INSTANCE.configureEnvironment(this);

	    // setup accelerometer to listen for shaking events...
        sensorInitialized = false;
        if(buzztouch_appDelegate.rootApp.getRootDevice().canDetectShaking()){
        	sensorManager = (SensorManager) getSystemService(Context.SENSOR_SERVICE);
        	accelerometer = sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER);
        }

		BT_locationManager.INSTANCE.configureEnvironment(this);

		// setup tabs (zero means "show first tab as selected")...
		finishSetup();

	}

	// Permission Checks...

	private boolean classExists(String className) {
		try {
			Class<?> cls = Class.forName(className);
			return true;
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		return false;
	}

	public void startSensor() {
		sensorManager.registerListener(this, accelerometer, SensorManager.SENSOR_DELAY_NORMAL);
		// handle accelerometer...
		if(sensorManager != null){
			if(buzztouch_appDelegate.rootApp.getRootDevice().canDetectShaking()){
				sensorManager.registerListener(this, accelerometer, SensorManager.SENSOR_DELAY_NORMAL);
			}
		}
	}

	public void stopSensor() {
		sensorManager.unregisterListener(this);
	}

    @Override
    public void onRequestPermissionsResult(int requestCode, String permissions[], int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
		BT_debugger.logInfo(this, "onRequestPermissionsResult");
        if (currentFragment!=null && currentFragment instanceof BT_fragment) {
            ((BT_fragment)currentFragment).onRequestPermissionsResult(requestCode, permissions, grantResults);
        }

		if (requestCode == BT_permissions.PERMISSION_CODE_LOCATION) {
			// If request is cancelled, the result arrays are empty.
			if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
				// permission was granted, yay!  Handle results
				BT_locationManager.INSTANCE.onPermissionResult(true);
				BT_locationManager.INSTANCE.getLastGpsLocation(this);

			} else {
				// permission denied, boo! Disable the
				// functionality that depends on this permission.
				BT_locationManager.INSTANCE.onPermissionResult(false);
			}

			// other 'case' lines to check for other
			// permissions this app might request
		}

    }

	//////////////////////////////////////////////////////////////////////////
	// tab setup and interface methods...

    protected void finishSetup() {
        BT_debugger.showIt(activityName + ":setupTabs (no tabs, finding home screen)");
//			tmpHomeScreenData.setIsHomeScreen(true);

        // show the home screen fragment...
        this.showAppHomeScreen();
    }


    // ////////////////////////////////////////////////////////////////////////




	/////////////////////////////////////////////////////////////////////////////////////////
 	// fragment changed listener. See BT_fragment.java for interface declaration...
	public void BTFragmentResumed(BT_item theScreenData) {
	 	BT_debugger.showIt(activityName + ":BTFragmentResumed");

	 	// setup the fragment's nav bar...
	 	if(theScreenData != null){

	 		// setup nav bar for this fragment...
	 		this.configureNavBarAndBackgroundForScreen(theScreenData);

	 		// remember this fragment's JSON data as the "current screen" so we can refer to it later...
	 		buzztouch_appDelegate.rootApp.setCurrentScreenData(theScreenData);

	 	}

	}
	/////////////////////////////////////////////////////////////////////////////////////////




	@Override
    public void onBackPressed() {
        super.onBackPressed();
		BT_debugger.showIt(activityName + ":onBackPressed");

		// currentFragment must exist...
		if(currentFragment != null){
		
			
			/*
	        	use reflection to trigger a handleBackButton method in the current plugin...not all
	        	plugins implement this method...but can as needed.
	        	
				In most cases the handleBackButton method should "go back" by popping the current
				fragment off the stack. In other cases it may do something all together different.
				
			*/

			java.lang.reflect.Method backMethod = null;
			try{
				backMethod = currentFragment.getClass().getMethod("handleBackButton");
			}catch(SecurityException e){
				BT_debugger.showIt(activityName + ":onBackPressed. EXCEPTION (0): " + e.toString());
			}catch(NoSuchMethodException e){
				BT_debugger.showIt(activityName + ":onBackPressed. Current fragment does not have an \"onBackPressed\" method implemented");
			}

			// fire the handleBackButton method if we found it...
			if(backMethod != null){
		    	try{
		    		backMethod.invoke(currentFragment);
				}catch(Exception e){
					BT_debugger.showIt(activityName + ":onBackPressed. EXCEPTION (1): " + e.toString());
				}
			}

		}

    }

	// handles rotation events...
	@Override
	public void onConfigurationChanged(Configuration newConfig){
		super.onConfigurationChanged(newConfig);
		BT_debugger.logInfo(this, "onConfigurationChanged");
		switch(newConfig.orientation){
	    	case  Configuration.ORIENTATION_LANDSCAPE:
	    		BT_debugger.showIt(activityName + ":onConfigurationChanged to landscape");
	    		buzztouch_appDelegate.rootApp.getRootDevice().updateDeviceOrientation("landscape");
	    		break;
	      	case Configuration.ORIENTATION_PORTRAIT:
	    		BT_debugger.showIt(activityName + ":onConfigurationChanged to portrait");
	      		buzztouch_appDelegate.rootApp.getRootDevice().updateDeviceOrientation("portrait");
	    		break;
	      	case Configuration.ORIENTATION_UNDEFINED:
	      		BT_debugger.showIt(activityName + ":onConfigurationChanged is unidentified");
	      		buzztouch_appDelegate.rootApp.getRootDevice().updateDeviceOrientation("portrait");
	      		break;
	      	default:
	    }

		// update device size so we can keep track...
		buzztouch_appDelegate.rootApp.getRootDevice().updateDeviceSize();
	}


	// show alert message
	public void showAlert(String theTitle, String theMessage) {
		if(theTitle.equals("")) theTitle = "No Alert Title?";
		if(theMessage.equals("")) theMessage = "No alert message?";

		AlertDialog.Builder adb = new AlertDialog.Builder(this);
		adb.setTitle(theTitle);
		adb.setMessage(theMessage);
		adb.setCancelable(false);
		adb.setNegativeButton(R.string.ok, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog,int id) {
				dialog.cancel();
			}
		});

		// create dialogue
		AlertDialog ad = adb.create();
		ad.show();

	}


	// showToast...
	public void showToast(String theMessage, String shortOrLong){
		Toast toast = null;
		if(shortOrLong.equalsIgnoreCase("short")){
			toast = Toast.makeText(getBaseContext(), theMessage, Toast.LENGTH_SHORT);
		}else{
			toast = Toast.makeText(this, theMessage, Toast.LENGTH_LONG);
		}
		toast.show();
	}





    // showProgress....
    public void showProgress(String theTitle, String theMessage){
		BT_debugger.logInfo(this, "showProgress");
        // title and message may be null or blank...
        if(theTitle == null) theTitle =  "";
        if(theMessage == null) theMessage = "";

        // if we don't have a title or a message, show the spinner...
        if(theTitle.length() < 1 && theMessage.length() < 1){
            if (progressSpinner == null) {
                progressSpinner = BT_utils.createSpinner(this);
                progressSpinner.show();
            } else {
                progressSpinner.show();
            }
        }else{

            if(theTitle.length() < 1) theTitle =  getString(R.string.loading);
            if(theMessage.length() < 1) theMessage =  "";

            // show an Android Progress Dialog...
            loadingMessage = ProgressDialog.show(this, theTitle, theMessage, true);
            loadingMessage.setCanceledOnTouchOutside(false);
            loadingMessage.setCancelable(true);

        }


    }

    // hideProgress...
    public void hideProgress(){
		BT_debugger.logInfo(this, "hideProgress");
        // hide possible spinner...
        if(progressSpinner != null){
            progressSpinner.dismiss();
            progressSpinner = null;
        }

        // hide possible dialogue...
        if(loadingMessage != null){
            loadingMessage.dismiss();
            loadingMessage = null;
        }

    }

    // confirmRefreshAppData...(asked after reportingToCloud)
	public void confirmRefreshAppData(){

		final AlertDialog confirmRefreshAlert = new AlertDialog.Builder(this).create();
		confirmRefreshAlert.setTitle(getString(R.string.confirmRefreshTitle));
		confirmRefreshAlert.setMessage(getString(R.string.confirmRefreshDescription));
		confirmRefreshAlert.setIcon(R.mipmap.ic_launcher);
		confirmRefreshAlert.setButton(DialogInterface.BUTTON_POSITIVE, getString(R.string.yes), new DialogInterface.OnClickListener() {
	      public void onClick(DialogInterface dialog, int which) {

	    	  // refresh data...
	    	  confirmRefreshAlert.dismiss();
	    	  refreshAppData();

	    } });
		confirmRefreshAlert.setButton(DialogInterface.BUTTON_NEGATIVE, getString(R.string.no), new DialogInterface.OnClickListener() {
		      public void onClick(DialogInterface dialog, int which) {
		    	  confirmRefreshAlert.dismiss();
		    } });
		confirmRefreshAlert.show();
	}

	// refreshAppData...
	public void refreshAppData(){
		BT_debugger.showIt(activityName + ":refreshAppData");

        // show spinner...
        showProgress(null, null);

        // pause for a moment so the spinner shows...
        refreshActionHandler.postDelayed(new Runnable() {
            public void run() {
                BT_configuration.INSTANCE.refreshData(new BT_configuration.BT_ConfigurationListener() {
                    @Override
                    public void onConfigurationProcessed() {
                        // restart app
                        startActivity(new Intent(BT_activity_host.this, BT_activity_start.class));
                        finish();
                    }

                    @Override
                    public void onErrorProcessingConfiguration() {
                        // error downloading data
						runOnUiThread(new Runnable() {
							@Override
							public void run() {
								showAlert(getResources().getString(R.string.errorTitle),getResources().getString(R.string.errorConfigData));
							}
						});
                    }

                    @Override
                    public void onDirtyCheck(boolean dirty) {
                        // not used
                    }
                });
            }
        }, 150);

	}


	/////////////////////////////////////////////////////
	// sensor events (device shaking)...

	// onAccuracyChanged...
    @Override
	public void onAccuracyChanged(Sensor arg0, int arg1){
		BT_debugger.logInfo(this, "onAccuracyChanged");
    	// BT_debugger.showIt(activityName + ":onAccuracyChanged (shake detection by accelerometer)");
    	
    	/*
    	 	This method is unused, it's included because this Activity implements
    	 	the SensorEventListener class and this method is required by that class
    	*/

	}

	// onSensorChanged...
    @Override
	public void onSensorChanged(SensorEvent event){
		BT_debugger.logInfo(this, "onSensorChanged");
    	// BT_debugger.showIt(activityName + ":onSensorChanged (shake detection by accelerometer)");

    	// acceleromoter values passed to this onSensorChanged method (event is an array of values passed to this method)...
		// the values passed in are normally float data types. We cast them to integers for simplicity...
		float x = event.values[0];
		float y = event.values[1];

    	if(!sensorInitialized){

    		// flag as initialized...
    		sensorInitialized = true;

    		// remember values...
    		shakeSpeedX = x;
    		shakeSpeedY = y;

    	}else{


    		float currentX = Math.abs(shakeSpeedX - x);
    		float currentY = Math.abs(shakeSpeedY - y);
    		if (currentX < SHAKE_THRESHOLD) currentX = 0;
    		if (currentY < SHAKE_THRESHOLD) currentY = 0;
    		shakeSpeedX = x;
    		shakeSpeedY = y;

        	// is the device shaking...
    		boolean isShaking = false;
     		if(currentX > currentY){
    			// shaking horizontal...
            	// BT_debugger.showIt(activityName + ":onSensorChanged (shake detection). Device is shaking horizontally. X: " + Float.toString(shakeSpeedX) + " Y:" + Float.toString(shakeSpeedY));
            	buzztouch_appDelegate.rootApp.getRootDevice().setIsShaking(true);
            	isShaking = true;
     		}else if(currentY > currentX){
    			// shaking veritcal...
            	// BT_debugger.showIt(activityName + ":onSensorChanged (shake detection). Device is shaking vertically. X: " + Float.toString(shakeSpeedX) + " Y:" + Float.toString(shakeSpeedY));
            	buzztouch_appDelegate.rootApp.getRootDevice().setIsShaking(true);
            	isShaking = true;
     		}else{
    			// not shaking...
            	buzztouch_appDelegate.rootApp.getRootDevice().setIsShaking(false);
            	isShaking = false;
    		}

            // use reflection to trigger the deviceIsShaking method in the current plugin...
     		if(isShaking){

	    		java.lang.reflect.Method shakeMethod = null;
	    		try{
	    			shakeMethod = getCurrentFragment().getClass().getMethod("deviceIsShaking", float.class, float.class);
	    		}catch(SecurityException e){
	    			BT_debugger.showIt(activityName + ":onSensorChanged. EXCEPTION (0): " + e.toString());
	    		}catch(NoSuchMethodException e) {
	    			BT_debugger.showIt(activityName + ":onSensorChanged. Current fragment does not have an \"deviceIsShaking\" method implemented");
	    		}

	    		// fire the deviceIsShaking method if we found it...
	    		if(shakeMethod != null){
	    	    	try{
	    	    		shakeMethod.invoke(getCurrentFragment(), shakeSpeedX, shakeSpeedY);
	    			}catch(Exception e){
	    				BT_debugger.showIt(activityName + ":onSensorChanged. EXCEPTION (1): " + e.toString());
	    			}
	    		}

     		}// isShaking...

    	}
	}

	// end sensor events (device shaking)...
	/////////////////////////////////////////////////////

    protected Fragment getCurrentFragment() {
        return currentFragment;
    }
    
    // initPluginWithScreenData. Returns a Fragment for a plugins JSON...
    public Fragment initPluginWithScreenData(BT_item theScreenData){
    	BT_debugger.logInfo(this, "initPluginWithScreenData");
		if(theScreenData != null){
			BT_debugger.showIt(activityName + ":initPluginWithScreenData. Calling helper method in BT_application");
		}

	 	// this is a helper method that uses the BT_application class to get the fragment...
        Fragment theFrag = buzztouch_appDelegate.rootApp.initPluginWithScreenData(theScreenData);
	 	
		// return...
		return theFrag;
	 	
    }
    
    // showFragmentForPlugin...
    public void showFragmentForPlugin(Fragment theFragment, BT_item theScreenData, boolean addToBackStack, String transitionType){
	 	BT_debugger.logInfo(this, "showFragmentForPlugin");
    	if(theScreenData == null){
	 		BT_debugger.showIt(activityName + ":showFragmentForPlugin \"NULL SCREEN DATA\"");
	 	}else{
			BT_debugger.showIt(activityName + ":showFragmentForPlugin. Showing plugin with JSON itemId: \"" + theScreenData.getItemId() + "\" itemType: \"" + theScreenData.getItemType() + "\" itemNickname: \"" + theScreenData.getItemNickname() + "\"");
	 	}
	 	// remember the "current" screen data as the "previous" screen data...
	 	buzztouch_appDelegate.rootApp.setPreviousScreenData(buzztouch_appDelegate.rootApp.getCurrentScreenData());

	 	// fragment transaction object...
        FragmentManager fm = getSupportFragmentManager();
        FragmentTransaction ft = fm.beginTransaction();
	    ft.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_OPEN);

	 	// find the transition resources if we're using one...
	 	if(transitionType.length() > 1){
	 		
				// ANIMATIONS BETWEEN FRAGMENTS NOT IMPLEMENTED
				//--------------------------------------------
	 			int enterTrans = BT_viewUtilities.getEnterTransitionFromType(transitionType);
	 			int exitTrans = BT_viewUtilities.getExitTransitionFromType(transitionType);
	 			if(enterTrans > 0 && exitTrans > 0){
			 		ft.setCustomAnimations(enterTrans, exitTrans);
	 			}
	 		
	 		
	 	}
	 	
 	 	// assume this fragment should be added to the back-stack...Tab Taps should NOT be added to back-stack...
	 	if(addToBackStack != false){
	 		ft.addToBackStack("itemTap");
	 	}else{
	 		// do nothing...
	 	}
	 	
	 	// show the fragment...
	 	ft.replace(R.id.fragmentBox, theFragment, "currentFragment");
	 	ft.commit();
	 	
 		// remember this fragment's JSON data as the "current screen data" so we can refer to it later...
 		buzztouch_appDelegate.rootApp.setCurrentScreenData(theScreenData);

 		// remember this fragment so we can refer to it later...
 		this.currentFragment = theFragment;
 		
		// setup nav bar and background...
		this.configureNavBarAndBackgroundForScreen(theScreenData);
	 	
    }    
    
    // showAppHomeScreen...
    public void showAppHomeScreen(){
	 	BT_debugger.showIt(activityName + ":showAppHomeScreen");
		  
	 	// get screen data for the app's home screen...
        buzztouch_appDelegate.rootApp.getHomeScreenData(new BT_item.onScreenDataReadyListener() {
            @Override
            public void onDataReady(BT_item homeScreenData) {
                // setup nav bar and background for this fragment...
                BT_activity_host.this.configureNavBarAndBackgroundForScreen(homeScreenData);

                //remember this fragment's JSON data as the "current screen" so we can refer to it later...
                buzztouch_appDelegate.rootApp.setCurrentScreenData(homeScreenData);

                Fragment homeScreenFrag = BT_activity_host.this.initPluginWithScreenData(homeScreenData);

                // show the home screen fragment...
                BT_activity_host.this.showFragmentForPlugin(homeScreenFrag, homeScreenData, false, "");

                // setup nav bar and background for this fragment...
                BT_activity_host.this.configureNavBarAndBackgroundForScreen(homeScreenData);

                //remember this fragment's JSON data as the "current screen" so we can refer to it later...
                buzztouch_appDelegate.rootApp.setCurrentScreenData(homeScreenData);
            }
        });

	 	
    }
    
    
    // configureNavBarAndBackgroundForScreen. Returns a Fragment for a plugins JSON...
    public void configureNavBarAndBackgroundForScreen(BT_item theScreenData){
        BT_debugger.logInfo(this, "configureNavBarAndBackgroundForScreen");
    	if(theScreenData == null){
        	BT_debugger.showIt(activityName + ":configureNavBarAndBackgroundForScreen. NULL SCREEN DATA?");
        }else{
        	BT_debugger.showIt(activityName + ":configureNavBarAndBackgroundForScreen Calling helper methods in BT_viewUtilities...");


			// set action bar title truncating...
	  	    final int actionBarTitle = Resources.getSystem().getIdentifier("action_bar_title", "id", "android");
	   	    final TextView title = (TextView)getWindow().findViewById(actionBarTitle);

	   	    configureActionBar(theScreenData);
	   	    	
		 	// ask BT_viewUtilities to setup this screens background color....
//		 	BT_viewUtilities.updateBackgroundColorsForScreen(this, theScreenData);
	
		 	// ask BT_viewUtilities to setup this screens background image...
//			View containerView = findViewById(R.id.containerView);
//			if (containerView.findViewById(R.id.containerView)!=null) containerView = containerView.findViewById(R.id.containerView);
//			BT_viewUtilities.updateBackgroundImageForScreen(theScreenData, containerView);
		 	
	 	}
	 	
    }

    public void configureActionBar(BT_item theScreenData) {
		BT_debugger.logInfo(this, "configureActionBar");
        // get the JSON data for this screen...

        String hideTabBarWhenScreenLoads = BT_strings.getStyleValueForScreen(theScreenData, "hideTabBarWhenScreenLoads", "");

        // DEPRECATED: older configurations may use "hideBottomTabBarWhenScreenLoads" instead....
        if(hideTabBarWhenScreenLoads.length() < 1){
            hideTabBarWhenScreenLoads = BT_strings.getStyleValueForScreen(theScreenData, "hideBottomTabBarWhenScreenLoads", "");
        }
        if(hideTabBarWhenScreenLoads.length() < 1){
            hideTabBarWhenScreenLoads = "0";
        }
        String navBarTitleText = BT_strings.getJsonPropertyValue(theScreenData.getJsonObject(), "navBarTitleText", "");
        String navBarStyle = BT_strings.getStyleValueForScreen(theScreenData, "navBarStyle", "default");
        String navBarBackgroundColor = BT_strings.getStyleValueForScreen(theScreenData, "navBarBackgroundColor", "");

        // set title...
        if(actionBar != null){

            // set title...
            actionBar.setTitle(navBarTitleText);

            // show or hide the "up/back" arrow on the actionBar if this is or is not the home screen (hide on home screen)..
            actionBar.setDisplayHomeAsUpEnabled(!theScreenData.isHomeScreen());

            // action bar color...
            if(navBarBackgroundColor.length() > 0){
                BT_debugger.logInfo(getLocalClassName(),"navBarBackgroundColor exists - applying:" + navBarBackgroundColor);
                ColorDrawable cd = new ColorDrawable(BT_color.getColorFromHexString(navBarBackgroundColor));
                actionBar.setBackgroundDrawable(cd);
            }

            // hide the action bar as needed...
            if(navBarStyle.equalsIgnoreCase("hidden")){
                actionBar.hide();
            }else{
                actionBar.show();
            }

            // hide the tabs if needed..
            View tabLayout = findViewById(R.id.tabs);
            if (tabLayout!=null) tabLayout.setVisibility(hideTabBarWhenScreenLoads.equalsIgnoreCase("1") ? View.GONE : View.VISIBLE);

        }
    }

	//reportToCloud
	public void reportToCloud(){
		BT_debugger.logInfo(this, activityName + ":reportToCloud");

        // check if data needs to be refreshed
        BT_configuration.INSTANCE.dataIsDirty(new BT_configuration.BT_ConfigurationListener() {
            @Override
            public void onConfigurationProcessed() {

            }

            @Override
            public void onErrorProcessingConfiguration() {

            }

            @Override
            public void onDirtyCheck(boolean dirty) {
                if (dirty) {
                    runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            confirmRefreshAppData();
                        }
                    });

                }
            }
        });

	}

    
    
    
 	/////////////////////////////////////////////////////
	// options menu (action bar menu or hardware button)...
	@Override
	public boolean onCreateOptionsMenu(Menu menu){
	 	// BT_debugger.showIt(activityName + ":onCreateOptionsMenu");

	 	// menu items are added in individual fragments (plugins) BEFORE this method runs...
		
		//return...
		return super.onCreateOptionsMenu(menu);
	}
	
	
	@Override
	public boolean onOptionsItemSelected(MenuItem item){
	 	// BT_debugger.showIt(activityName + ":onOptionsItemSelected");
	 	
		// item selections are handled in individual fragments (plugins) but this must return FALSE to prevent interceptions...
	   switch (item.getItemId()){
           case android.R.id.home:
               BT_debugger.logInfo(activityName, "home/up button pressed...");
               onBackPressed();
	      default:
	  		return false;
	   }	 	
	 	
	}



	
}
