/*  File Version: 4.0.2
 *	Copyright, Chris Ruddell, buzztouch.com
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the
 *	following disclaimer.
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list
 *	of conditions and the following disclaimer in the documentation and/or other materials
 *	provided with the distribution.
 *
 *	Neither the name of David Book, or buzztouch.com nor the names of its contributors
 *	may be used to endorse or promote products derived from this software without specific
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 *	OF SUCH DAMAGE.
 */
package com.buzztouch.ui.bt_screens;

import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.Drawable;
import android.os.Build;
import android.os.Bundle;

import androidx.appcompat.widget.Toolbar;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentStatePagerAdapter;
import androidx.viewpager.widget.ViewPager;

import android.view.View;
import android.view.Window;

import com.buzztouch.buzztouch_appDelegate;
import com.buzztouch.R;
import com.buzztouch.data_models.BT_item;
import com.buzztouch.data_models.BT_tab;
import com.buzztouch.utils.BT_color;
import com.buzztouch.utils.BT_debugger;
import com.buzztouch.utils.BT_fileManager;
import com.buzztouch.utils.BT_strings;
import com.google.android.material.tabs.TabLayout;

import java.util.ArrayList;

/**
 * Created by christopherruddell on 1/8/17.
 */

public class BT_activity_host_tabs extends BT_activity_host implements TabLayout.OnTabSelectedListener {
    private static final String TAG = "BT_activity_host_tabs";
    private static final boolean DEBUG_LOG = true;
    private TabLayout mTabLayout;
    private ViewPager mViewPager;
    private ViewPagerAdapter mAdapter;
    private Toolbar mToolbar;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        if (DEBUG_LOG) BT_debugger.logInfo(TAG, "onCreate");
        super.onCreate(savedInstanceState);


    }

    @Override
    public void requestWindowFeatures() {
        this.supportRequestWindowFeature(Window.FEATURE_NO_TITLE);
    }

    @Override
    public void setupActionBar() {

        if (getActionBar()!=null) getActionBar().hide();
        mToolbar = (Toolbar) findViewById(R.id.toolbar);
        mToolbar.setNavigationOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                onBackPressed();
            }
        });
        setSupportActionBar(mToolbar);
    }

    @Override
    public void onBackPressed() {
        if (DEBUG_LOG) BT_debugger.logInfo(TAG, "onBackPressed()");
        // override and simulate going on backstack with tabs and fragments
        BT_tab selectedTab = mAdapter.getTab(mViewPager.getCurrentItem());
        boolean backAllowed = selectedTab.onBackPressed();
        if (!backAllowed) finish();
        else mAdapter.notifyDataSetChanged();
    }

    @Override
    public void onStart() {
        super.onStart();
        // configure navbar for first tab
        BT_tab firstTab = mAdapter.getTab(0);
        if (firstTab.getTopFragment() instanceof BT_fragment) {
            BT_item theScreenData = ((BT_fragment)firstTab.getTopFragment()).screenData;
            configureActionBar(theScreenData);
            String navBarTitleText = getNavBarTitle(theScreenData);
            setTitle(navBarTitleText);
        }
    }

    @Override
    protected int getLayoutFile() {
        return R.layout.bt_activity_host_tabs;
    }

    @Override
    protected void finishSetup() {
        BT_debugger.logInfo(TAG, "setting up environment with " + buzztouch_appDelegate.rootApp.getTabs().size() + " tabs...");

        // add tabs and setup controllers for each tab
        mTabLayout = (TabLayout)findViewById(R.id.tabs);
        mViewPager = (ViewPager)findViewById(R.id.viewpager);


        mAdapter = new ViewPagerAdapter(getSupportFragmentManager());

        for(int t = 0; t < buzztouch_appDelegate.rootApp.getTabs().size(); t++) {
            BT_item thisTab = buzztouch_appDelegate.rootApp.getTabs().get(t);
            BT_debugger.logInfo(getLocalClassName(), "setting up tab:" + t + " withData:" + thisTab.getJsonObject().toString());

            // get title and icon for this tab...
            String title = BT_strings.getJsonPropertyValue(thisTab.getJsonObject(), "textLabel", "");
            Drawable d = null;
            String iconName = BT_strings.getJsonPropertyValue(thisTab.getJsonObject(), "iconName", "");
            if(iconName.length() > 0){
                d = BT_fileManager.getDrawableByName(iconName);
            }

            // get fragment for tab
            String homeScreenItemId = BT_strings.getJsonPropertyValue(thisTab.getJsonObject(), "homeScreenItemId", "");
            BT_item screenData = buzztouch_appDelegate.rootApp.getScreenDataByItemId(homeScreenItemId);
            screenData.setHomeScreen(true);
            Fragment fragment = buzztouch_appDelegate.rootApp.initPluginWithScreenData(screenData);
            if (fragment instanceof BT_fragment) {
                ((BT_fragment)fragment).setTab(t);
            }

            // add fragment to each page
            BT_tab tab = new BT_tab(title, d, fragment);
            mAdapter.addFragment(tab);

        }

        mViewPager.setAdapter(mAdapter);

        mTabLayout.addOnTabSelectedListener(this);
        mTabLayout.setupWithViewPager(mViewPager);

    }

    @Override
    protected Fragment getCurrentFragment() {
        int tabIndex = mTabLayout.getSelectedTabPosition();
        return mAdapter.getTab(tabIndex).fragment;
    }

    @Override
    public void showAppHomeScreen() {
        if (DEBUG_LOG) BT_debugger.logInfo(TAG, "showAppHomeScreen()");
        onTabSelected(mTabLayout.getTabAt(0));

//        final int tabIndex = 0;
//
//        // get the JSON data for the selected tab...
//        BT_item thisTab = buzztouch_appDelegate.rootApp.getTabs().get(tabIndex);
//        String homeScreenItemId = BT_strings.getJsonPropertyValue(thisTab.getJsonObject(), "homeScreenItemId", "");
//
//        // get the JSON data for this screen...
//        buzztouch_appDelegate.rootApp.getScreenDataByItemId(homeScreenItemId, new BT_item.onScreenDataReadyListener() {
//            @Override
//            public void onDataReady(BT_item screenData) {
//                if (DEBUG_LOG) BT_debugger.logInfo(TAG, "screenData retrieved - showing fragment...");
//                // set as "homeScreen" if this is tab zero...
//                screenData.setIsHomeScreen(tabIndex == 0);
//
//                // create the fragment for the selected tab...
//                Fragment theFrag = buzztouch_appDelegate.rootApp.initPluginWithScreenData(screenData);
//
//                // remember the selected tab index in the BT_application object...
//                buzztouch_appDelegate.rootApp.setSelectedTab(tabIndex);
//
//                // show the fragment...
//                if (DEBUG_LOG) BT_debugger.logInfo(TAG, "fragment created - showing now...");
//                BT_activity_host_tabs.this.showFragmentForPlugin(theFrag, screenData, false, "");
//            }
//        });
    }

    public void showFragmentForPlugin(Fragment theFragment, BT_item theScreenData, boolean addToBackStack, String transitionType, int tabNumber) {
        if (DEBUG_LOG) BT_debugger.logInfo(TAG, "showFragmentForPlugin");
        if(theScreenData == null){
            BT_debugger.logInfo(TAG, ":showFragmentForPlugin \"NULL SCREEN DATA\"");
        }else{
            BT_debugger.logInfo(TAG, ":showFragmentForPlugin. Showing plugin with JSON itemId: \"" + theScreenData.getItemId() + "\" itemType: \"" + theScreenData.getItemType() + "\" itemNickname: \"" + theScreenData.getItemNickname() + "\"");
        }
        // remember the "current" screen data as the "previous" screen data...
        buzztouch_appDelegate.rootApp.setPreviousScreenData(buzztouch_appDelegate.rootApp.getCurrentScreenData());

        // remember this fragment's JSON data as the "current screen data" so we can refer to it later...
        buzztouch_appDelegate.rootApp.setCurrentScreenData(theScreenData);

        // remember this fragment so we can refer to it later...
        this.currentFragment = theFragment;

        // update viewpager with new tab
        if (DEBUG_LOG) BT_debugger.logInfo(TAG, "updating viewPager with new fragment for tab using plugin:" + theFragment.getClass().getSimpleName() + "...");
        BT_tab tab = mAdapter.getTab(tabNumber);
        tab.addFragment(theFragment);
        mAdapter.updateTab(tab, tabNumber);

        // setup nav bar and background...
//        if (DEBUG_LOG) BT_debugger.logInfo(TAG, "configuring navbar and background...");
//        this.configureNavBarAndBackgroundForScreen(theScreenData);
    }

    @Override
    public void showFragmentForPlugin(Fragment theFragment, BT_item theScreenData, boolean addToBackStack, String transitionType) {
        showFragmentForPlugin(theFragment, theScreenData, addToBackStack, transitionType, 0);
    }

    private String getNavBarTitle(BT_item theScreenData) {
        return BT_strings.getJsonPropertyValue(theScreenData.getJsonObject(), "navBarTitleText", "");
    }

    @Override
    public void configureActionBar(BT_item theScreenData) {
        if (DEBUG_LOG) BT_debugger.logInfo(TAG, "configureActionBar");
        // get the JSON data for this screen...

        String hideTabBarWhenScreenLoads = BT_strings.getStyleValueForScreen(theScreenData, "hideTabBarWhenScreenLoads", "");

        // DEPRECATED: older configurations may use "hideBottomTabBarWhenScreenLoads" instead....
        if(hideTabBarWhenScreenLoads.length() < 1){
            hideTabBarWhenScreenLoads = BT_strings.getStyleValueForScreen(theScreenData, "hideBottomTabBarWhenScreenLoads", "");
        }
        if(hideTabBarWhenScreenLoads.length() < 1){
            hideTabBarWhenScreenLoads = "0";
        }
        String navBarTitleText = getNavBarTitle(theScreenData);
        String navBarStyle = BT_strings.getStyleValueForScreen(theScreenData, "navBarStyle", "default");
        String navBarBackgroundColor = BT_strings.getStyleValueForScreen(theScreenData, "navBarBackgroundColor", "#616161");

        // set navbar options
        if (mToolbar==null) {
            BT_debugger.logError(TAG, "ToolBar is null!");
            mToolbar = (Toolbar)findViewById(R.id.toolbar);
        }
        if(mToolbar != null){

            // set title...
            mToolbar.setTitle(navBarTitleText);

            // show or hide the "up/back" arrow on the actionBar if this is or is not the home screen (hide on home screen)..
            mToolbar.setNavigationIcon(theScreenData.isHomeScreen() ? null : getResources().getDrawable(R.drawable.bt_arrow_left));


            // action bar color...
            if(navBarBackgroundColor.length() > 0){
                BT_debugger.logInfo(getLocalClassName(),"navBarBackgroundColor exists - applying:" + navBarBackgroundColor);
                ColorDrawable cd = new ColorDrawable(BT_color.getColorFromHexString(navBarBackgroundColor));
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN) {
                    mToolbar.setBackground(cd);
                }
                else {
                    mToolbar.setBackgroundDrawable(cd);
                }
            }

            // hide the action bar as needed...
            if(navBarStyle.equalsIgnoreCase("hidden")){
                mToolbar.setVisibility(View.GONE);
            }else{
                mToolbar.setVisibility(View.VISIBLE);
            }

            // hide the tabs if needed..
            View tabLayout = findViewById(R.id.tabs);
            if (tabLayout!=null) tabLayout.setVisibility(hideTabBarWhenScreenLoads.equalsIgnoreCase("1") ? View.GONE : View.VISIBLE);

        }
    }

    private boolean hasShownSelection = false;
    @Override
    public void onTabSelected(TabLayout.Tab tab) {
        if (DEBUG_LOG) BT_debugger.logInfo(TAG, "onTabSelected(" + tab.getPosition() + ")");
        mViewPager.setCurrentItem(tab.getPosition());
        BT_tab thisTab = mAdapter.getTab(tab.getPosition());
        Fragment thisFragment = thisTab.getTopFragment();
        if (thisFragment instanceof BT_fragment) {
            configureActionBar(((BT_fragment)thisFragment).screenData);
        }
    }

    @Override
    public void onTabUnselected(TabLayout.Tab tab) {

    }

    @Override
    public void onTabReselected(TabLayout.Tab tab) {
        if (DEBUG_LOG) BT_debugger.logInfo(TAG, "onTabReselected");
//        if (tab.getPosition()==buzztouch_appDelegate.rootApp.getSelectedTab() && hasShownSelection) {
//            if (DEBUG_LOG) BT_debugger.logInfo(TAG, "already shown - skipping");
//            hasShownSelection = false;
//            return;
//        }
//        hasShownSelection = true;
//        onTabSelected(tab);

    }


    class ViewPagerAdapter extends FragmentStatePagerAdapter {
        private static final String TAG = "ViewPagerAdapter";
        private final ArrayList<BT_tab> mTabList = new ArrayList<>();


        ViewPagerAdapter(FragmentManager manager) {
            super(manager);
        }

        @Override
        public Fragment getItem(int position) {
            if (DEBUG_LOG) BT_debugger.logInfo(TAG, "getItem(" + position + ") = " + mTabList.get(position).getTopFragment().getClass().getSimpleName());
            return mTabList.get(position).getTopFragment();
        }

        @Override
        public int getItemPosition(Object object) {
            return POSITION_NONE;
        }

        @Override
        public int getCount() {
            return mTabList.size();
        }

        void addFragment(BT_tab tab) {
            mTabList.add(tab);
        }

        @Override
        public CharSequence getPageTitle(int position) {
            return mTabList.get(position).title;
        }

        BT_tab getTab(int position) {
            return mTabList.get(position);
        }

        void updateTab(BT_tab tab, int index) {
            if (DEBUG_LOG)BT_debugger.logInfo(TAG, "updateTab(" + index + ")");
            mTabList.set(index, tab);
            notifyDataSetChanged();
        }
    }
}
