/*  File Version: 4.0
 *	Copyright, David Book, buzztouch.com
 *  Updates made by Chris Ruddell, 1/6/2017
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are 
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the 
 *	following disclaimer. 
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list 
 *	of conditions and the following disclaimer in the documentation and/or other materials 
 *	provided with the distribution. 
 *
 *	Neither the name of David Book, or buzztouch.com nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without specific 
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *	OF SUCH DAMAGE. 
 */
package com.buzztouch.ui.bt_screens

import android.content.Intent
import android.os.Bundle
import androidx.appcompat.app.AppCompatActivity
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentTransaction
import com.buzztouch.buzztouch_appDelegate
import com.buzztouch.R
import com.buzztouch.core.BT_configuration
import com.buzztouch.extensions.BtLog
import com.buzztouch.extensions.errorAlert
import com.buzztouch.utils.BT_strings
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlin.coroutines.CoroutineContext

class BT_activity_start : AppCompatActivity(), CoroutineScope {
    override val coroutineContext: CoroutineContext get() = Dispatchers.Main
    var activityName = "BT_activity_start"

    // onCreate...
    public override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        BtLog("$activityName:onCreate")

        // inflate the view for this activity...
        setContentView(R.layout.bt_activity_start)

        // load app data...
        launch { loadAppConfigData() }
    }

    // loadAppConfigData...
    suspend fun loadAppConfigData() {
        BtLog("$activityName:loadAppConfigData")
        if (BT_configuration.loadAppConfigData()) configureEnvironment()
        else errorAlert(this, getString(R.string.errorTitle), getString(R.string.errorConfigData))
    }

    // configureEnvironment...shows app's first screen after parsing config data...
    private fun configureEnvironment() {
        BtLog("$activityName:configureEnvironment")

        /*
 			This method is fired by the BT_fragment_load_config.java class after
 			loading all the app's data...Activities that use the BT_fragment_load_config
 			fragment should implement their own "configureEnvironment" method because the
 			loader fragment calls a method with that name in the host activity after it
 			completes. This is useful when "reloading" the app's data.
 		*/

        // get the theme data to find a possible splash screen...
        val theThemeData = buzztouch_appDelegate.rootApp.rootTheme
        val splashScreenItemId =
            BT_strings.getJsonPropertyValue(theThemeData.jsonObject, "splashScreenItemId", "")

        // load a splash screen fragment if used...
        if (splashScreenItemId.length > 1) {
            BtLog("$activityName:configureEnvironment This app uses a splash screen with JSON itemId: $splashScreenItemId")
            BtLog("$activityName:configureEnvironment Starting BT_activity_splash")
            val splashScreenJSON = buzztouch_appDelegate.rootApp.getCachedScreenByItemId(splashScreenItemId)
            val splashFrag = buzztouch_appDelegate.rootApp.initPluginWithScreenData(splashScreenJSON)
            showFragmentForSplashScreen(splashFrag)
        } else {

            // no splash screen is used, kill this Activity and show the BT_activity_host...
            BtLog("$activityName:configureEnvironment This app does not use a splash screen")
            BtLog("$activityName:configureEnvironment Starting BT_activity_host")

            // do we use tabs?
            val launchActivity = if (buzztouch_appDelegate.rootApp.tabs.isNotEmpty()) BT_activity_host_tabs::class.java else BT_activity_host::class.java
            val i = Intent(this@BT_activity_start, launchActivity)
            startActivity(i)

            // finish this activity (remove it from the back stack)...
            finish()

            // fade in next screen...
            overridePendingTransition(R.anim.bt_fadein, R.anim.bt_fadeout)
        } // splashScreenItemId...
    }

    // showFragmentForSplashScreen...
    fun showFragmentForSplashScreen(theFragment: Fragment?) {
        BtLog("$activityName:showFragmentForSplashScreen")
        if (theFragment != null) {

            // fragment transaction object...
            val fm = supportFragmentManager
            val ft = fm.beginTransaction()
            ft.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_OPEN)

            // show the fragment...
            ft.replace(R.id.fragmentBox, theFragment, "currentFragment")
            ft.commit()
        } else {
            BtLog("$activityName:showFragmentForSplashScreen ERROR: Splash Screen fragment is null?")
        }
    }
}