/*  File Version: 3.0
 *	Copyright, David Book, buzztouch.com
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are 
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the 
 *	following disclaimer. 
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list 
 *	of conditions and the following disclaimer in the documentation and/or other materials 
 *	provided with the distribution. 
 *
 *	Neither the name of David Book, or buzztouch.com nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without specific 
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *	OF SUCH DAMAGE. 
 */
package com.buzztouch.ui.bt_screens

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import com.buzztouch.R
import com.buzztouch.extensions.propertyValue
import com.buzztouch.utils.BT_debugger

class BT_fragment_missing : BT_fragment() {
    private var missingFragmentName = ""

    // onCreateView...BT_activity_host needs a view to display when it creates this fragment...
    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {

        // set the fragment name to the Class name (outputs in LogCat console when using the BT_debugger)...
        fragmentName = BT_fragment::class.java.simpleName

        // show life-cycle event in LogCat console...
        BT_debugger.showIt("$fragmentName:onCreateView for screen with itemId: $screenItemId")

        // inflate the view for this screen...
        val thisScreensView = inflater.inflate(R.layout.bt_fragment_missing, container, false)

        // find the text view and add some info...
        val tmpTextView = thisScreensView.findViewById<View>(R.id.missingTextView) as TextView

        // get error text
        val errorMsg = screenData.propertyValue("errorMessage")
        val errorText = when {
            errorMsg.isEmpty() && missingFragmentName.isNotEmpty() -> "Missing Plugin: The Android class file for this screen ($missingFragmentName) was not compiled in the project."
            errorMsg.isEmpty() -> "An unknown error has occurred!"
            else -> errorMsg
        }

        tmpTextView.text = errorText

        // return...
        return thisScreensView
    }

    companion object {
        fun newInstance(missingFragmentName: String): BT_fragment_missing {
            val fragment = BT_fragment_missing()
            fragment.missingFragmentName = missingFragmentName
            return fragment
        }
    }
}
