/*
 *	Copyright 2011, David Book, buzztouch.com
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are 
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the 
 *	following disclaimer. 
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list 
 *	of conditions and the following disclaimer in the documentation and/or other materials 
 *	provided with the distribution. 
 *
 *	Neither the name of David Book, or buzztouch.com nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without specific 
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *	OF SUCH DAMAGE. 
 */
package com.buzztouch.ui.bt_screens

import android.app.AlertDialog
import android.content.DialogInterface
import android.os.Bundle
import android.view.*
import android.view.animation.AlphaAnimation
import android.widget.Button
import android.widget.ImageView
import com.buzztouch.buzztouch_appDelegate
import com.buzztouch.R
import com.buzztouch.extensions.*
import com.buzztouch.utils.BT_fileManager
import com.buzztouch.utils.BT_viewUtilities
import kotlinx.coroutines.launch

class BT_screen_excelDoc : BT_fragment() {
    var dataURL = ""
    var saveAsFileName = ""
    var openWithButton: Button? = null
    var downloadButton: Button? = null
    private var localFileName = ""
    private var documentIconView: ImageView? = null
    private var confirmRefreshDialogue: AlertDialog? = null
    private var showBrowserBarEmailDocument = false
    private var showBrowserBarRefresh = false
    private var forceRefresh = false

    // onCreateView..
    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {

        /*
 			Note: fragmentName property is already setup in the parent class (BT_fragment). This allows us 
 			to add the 	name of this class file to the LogCat console using the BT_debugger.
		*/
        // show life-cycle event in LogCat console..
        BtLog(fragmentName + ":onCreateView JSON itemId: \"" + screenData.itemId + "\" itemType: \"" + screenData.itemType + "\" itemNickname: \"" + screenData.itemNickname + "\"")

        // inflate the layout file for this screen..
        val thisScreensView = inflater.inflate(R.layout.bt_screen_exceldoc, container, false)

        // reference to the document type icon..
        documentIconView = thisScreensView.findViewById(R.id.documentTypeIcon)
        documentIconView?.setImageDrawable(getDrawable(R.drawable.bt_screen_exceldoc))

        // reference to "open" and "donwload" buttons..
        openWithButton = thisScreensView.findViewById(R.id.openWithButton)
        downloadButton = thisScreensView.findViewById(R.id.downloadButton)

        // fill JSON properties..
        localFileName = screenData.propertyValue("localFileName")
        dataURL = screenData.propertyValue("dataURL")
        showBrowserBarEmailDocument = screenData.propertyValue("showBrowserBarEmailDocument", "1") == "1"
        showBrowserBarRefresh = screenData.propertyValue("showBrowserBarRefresh") == "1"
        forceRefresh = screenData.propertyValue("forceRefresh") == "1"

        // if we have NO localFileName and no dataURL, use the sample file for this plugin..
        if (localFileName.isEmpty() && dataURL.isEmpty()) {
            localFileName = "bt_screen_exceldoc_sample.xls"
        }

        // figure out the saveAsFileName..
        if (localFileName.isNotEmpty()) {

            // use the file name in the JSON data..
            saveAsFileName = localFileName

            // copy the file from the /assets/BT_Docs folder to the cache if it doesn't already exist..
            if (BT_fileManager.doesProjectAssetExist("BT_Docs", saveAsFileName)) {
                if (!BT_fileManager.doesCachedFileExist(saveAsFileName)) {
                    BT_fileManager.copyAssetToCache("BT_Docs", saveAsFileName)
                }
            } else {
                showErrorAlert(R.string.errorDocumentNotInProject)
                BtLog("$fragmentName:onCreate. localFileName \"$saveAsFileName\" does not exist in project! ")
            }
        } else {
            saveAsFileName = screenData.itemId + "_screenData.xls"
        }

        // remove file if we are force-refreshing..
        if (forceRefresh) {
            BT_fileManager.deleteFile(saveAsFileName)
        }

        // change click event for "open with" button..
        openWithButton?.setOnClickListener { openDocInCache(saveAsFileName) }

        // click event for "document image"..
        documentIconView?.setOnClickListener { openDocInCache(saveAsFileName) }

        // click event of the download button..
        downloadButton?.setOnClickListener { downloadAndSaveFile(dataURL, saveAsFileName) }

        // if file is not in the cache, disable the openWith button..
        if (!BT_fileManager.doesCachedFileExist(saveAsFileName)) {
            openWithButton?.isEnabled = false
        }

        // hide download button if we don't have a URL..
        if (dataURL.length < 5) {
            downloadButton?.visibility = View.GONE
        } else {
            // if we have a dataURL, AND we have a cached "saveAsFileName", change label to "refresh"..
            if (BT_fileManager.doesCachedFileExist(saveAsFileName)) {
                downloadButton?.text = getString(R.string.refreshFromURL)
                downloadButton?.setOnClickListener { handleRefreshButton() }
            }
        }

        // add face effect to image (it's not a button, it's an image)..
        val alphaFade = AlphaAnimation(0.3f, 1.0f)
        alphaFade.duration = 500
        alphaFade.fillAfter = true
        documentIconView?.startAnimation(alphaFade)

        // return..
        return thisScreensView
    } // onCreateView..

    // openDocInCache..
    private fun openDocInCache(fileName: String?) {
        BtLog("$fragmentName:openDocInCache. Calling helper method in BT_viewUtilities..")

        // make sure doc is in cache already..
        if (BT_fileManager.doesCachedFileExist(fileName)) {

            // use helper in BT_viewUtilities method to open the document..
            BT_viewUtilities.openDocInCacheWithMimeType(fileName, "application/vnd.ms-excel", this.activity)
        } else {

            // show document not cached alert.
            showErrorAlert(R.string.errorDocumentNotInCache)
            BtLog("$fragmentName:handleEmailDocumentButton Cannot email document, document not in the cache?")
        }
    }

    // handleEmailButton..
    private fun handleEmailButton() {
        BtLog("$fragmentName:handleEmailButton. Calling helper method in BT_viewUtilities..")

        // make sure device can send email..
        if (canSendEmail) {

            // make sure doc is in cache already..
            if (BT_fileManager.doesCachedFileExist(saveAsFileName)) {

                // use helper method in BT_viewUtilities to email document..
                BT_viewUtilities.emailDocumentInCacheWithMimeType(saveAsFileName, "application/vnd.ms-excel", this.activity)
            } else {

                // show document not cached alert.
                showAlert(buzztouch_appDelegate.getApplication().getString(R.string.cannotEmailDocument), getString(R.string.errorDocumentNotInCache))
                BtLog("$fragmentName:handleEmailButton Cannot email document, document not in the cache?")
            }
        } else {
            // show cannot email alert..
            showAlert(buzztouch_appDelegate.getApplication().getString(R.string.cannotEmailDocument), getString(R.string.noNativeAppDescription))
            BtLog("$fragmentName:handleEmailButton Cannot email document, device cannot send emails?")
        }
    }

    private fun handleRefreshButton() {
        if (dataURL.isNotEmpty()) {
            BtLog("$fragmentName:handleRefreshButton Current URL: $dataURL")

            // we have a dataURL, confirm if we have already downloaded it..
            when (BT_fileManager.doesCachedFileExist(saveAsFileName)) {
                true -> confirmRefreshDocument()
                false -> downloadAndSaveFile(dataURL, saveAsFileName)
            }
        } else {
            // show no URL alert..
            showAlert(buzztouch_appDelegate.getApplication().getString(R.string.cannotRefreshDocument), getString(R.string.errorNoURL))
            BtLog("$fragmentName:handleRefreshButton cannot refresh document, no dataURL provided")
        }
    }

    private fun confirmRefreshDocument() {
        confirmRefreshDialogue = AlertDialog.Builder(this.activity).create()
        confirmRefreshDialogue?.apply {
            setTitle(getString(R.string.confirm))
            setMessage(getString(R.string.confirmRefreshDocument))
            setIcon(R.mipmap.ic_launcher)
            setButton(DialogInterface.BUTTON_POSITIVE, getString(R.string.yes)) { _, _ ->
                confirmRefreshDialogue?.dismiss()
                downloadAndSaveFile(dataURL, saveAsFileName)
            }
            setButton(DialogInterface.BUTTON_NEGATIVE, getString(R.string.no)) { _, _ ->
                confirmRefreshDialogue?.dismiss()
            }
        }
        confirmRefreshDialogue?.show()
    }

    private fun downloadAndSaveFile(dataURL: String?, saveAsFileName: String) {
        dataURL ?: return
        launch {
            // hide graphic..
            documentIconView?.visibility = View.GONE

            downloadScreenData(dataURL, saveAsFileName)

            hideProgress()
            documentIconView?.visibility = View.VISIBLE
            openWithButton?.isEnabled = true
            downloadButton?.text = getString(R.string.refreshFromURL)

            // change click handler of download button to confirm refresh..
            downloadButton?.setOnClickListener { handleRefreshButton() }
        }
    }

    override fun onCreateOptionsMenu(menu: Menu, inflater: MenuInflater) {
        val context = context ?: return

        // refresh page..
        if (showBrowserBarRefresh) {
            menu.addIfRoom(context, MENU_ITEM_REFRESH, R.string.browserRefresh, R.drawable.bt_refresh)
        }

        // email document..
        if (showBrowserBarEmailDocument) {
            menu.addIfRoom(context, MENU_ITEM_EMAIL, R.string.browserEmailDocument, R.drawable.bt_email)
        }

        // call super..
        super.onCreateOptionsMenu(menu, inflater)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean = when (item.itemId) {
        MENU_ITEM_REFRESH -> true.also { handleRefreshButton() }
        MENU_ITEM_EMAIL -> true.also { handleEmailButton() }
        else -> super.onOptionsItemSelected(item)
    }

    companion object {
        const val MENU_ITEM_REFRESH = 1
        const val MENU_ITEM_EMAIL = 2
    }
}
