/*
 *	Copyright 2011, David Book, buzztouch.com
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are 
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the 
 *	following disclaimer. 
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list 
 *	of conditions and the following disclaimer in the documentation and/or other materials 
 *	provided with the distribution. 
 *
 *	Neither the name of David Book, or buzztouch.com nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without specific 
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *	OF SUCH DAMAGE. 
 */
package com.buzztouch.ui.bt_screens

import android.content.res.Configuration
import android.graphics.Color
import android.graphics.drawable.BitmapDrawable
import android.graphics.drawable.Drawable
import android.os.Bundle
import android.view.Gravity
import android.view.LayoutInflater
import android.view.Menu
import android.view.MenuInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import android.view.animation.AlphaAnimation
import android.view.animation.Animation
import android.widget.Button
import android.widget.HorizontalScrollView
import android.widget.ImageView
import android.widget.ImageView.ScaleType
import android.widget.LinearLayout
import android.widget.RelativeLayout
import android.widget.ScrollView
import android.widget.TableLayout
import android.widget.TableRow
import android.widget.TextView
import com.buzztouch.buzztouch_appDelegate
import com.buzztouch.R
import com.buzztouch.data_models.BT_item
import com.buzztouch.extensions.deviceWidth
import com.buzztouch.extensions.downloadImage
import com.buzztouch.extensions.downloadScreenData
import com.buzztouch.extensions.isLargeDevice
import com.buzztouch.extensions.loadScreen
import com.buzztouch.extensions.styleValue
import com.buzztouch.extensions.styleValueInt
import com.buzztouch.utils.*
import kotlinx.coroutines.launch
import org.json.JSONArray
import org.json.JSONObject
import java.util.Locale
import kotlin.collections.ArrayList
import kotlin.math.floor

class BT_screen_menuButtons : BT_fragment(), View.OnClickListener {
    private var didCreate = false
//    private var getButtonImageWorkerThread: GetButtonImageWorkerThread? = null
    private var containerView: LinearLayout? = null
    private var containerViewHorizontal: RelativeLayout? = null
    private var containerViewHorizontalBottom: LinearLayout? = null
    private var containerHorizontalButtons: LinearLayout? = null
    private var verticalScrollView: ScrollView? = null
    private var horizontalScrollView: HorizontalScrollView? = null
    private var tableLayout: TableLayout? = null
    private var selectedIndex = -1
    private var JSONData = ""
    private var childItems: ArrayList<BT_item>? = null
    private var buttonBoxes: ArrayList<RelativeLayout>? = null
    private var buttonSquares: ArrayList<RelativeLayout>? = null
    private var buttonImages: ArrayList<Drawable?>? = null

    // properties from JSON
    private var dataURL = ""
    private var saveAsFileName = ""
    private var preventAllScrolling = ""
    private var buttonLayoutStyle = ButtonLayoutType.Grid
    private var buttonLabelLayoutStyle = LabelLayoutStyle.Below
    private var buttonLabelFontColor = ""
    private var buttonBackgroundColor = ""
    private var buttonOpacity = 100
    private var buttonLabelFontSize = 0
    private var buttonSize = 0
    private var buttonPadding = 0
    private var buttonCornerRadius = 0

    // onCreateView...
    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {

        /*
 			Note: fragmentName property is already setup in the parent class (BT_fragment). This allows us 
 			to add the 	name of this class file to the LogCat console using the BT_debugger.
		*/
        // show life-cycle event in LogCat console...
        BT_debugger.showIt(fragmentName + ":onCreateView JSON itemId: \"" + screenData.itemId + "\" itemType: \"" + screenData.itemType + "\" itemNickname: \"" + screenData.itemNickname + "\"")

        // inflate the layout file for this screen...
        val thisScreensView = inflater.inflate(R.layout.bt_screen_menubuttons, container, false)

        // init properties for JSON data...
        childItems = ArrayList()
        dataURL = BT_strings.getJsonPropertyValue(screenData.jsonObject, "dataURL", "")
        saveAsFileName = screenData.itemId + "_screenData.txt"
        if (dataURL.length < 1) BT_fileManager.deleteFile(saveAsFileName)

        // array holds list of "button boxes"...
        buttonBoxes = ArrayList()
        buttonSquares = ArrayList()
        buttonImages = ArrayList()

        // /////////////////////////////////////////////////////////////////
        // properties for both large and small devices...
        preventAllScrolling = screenData.styleValue("preventAllScrolling", "0")
        buttonLayoutStyle = screenData.styleValue("buttonLayoutStyle", "grid").layoutType()
        buttonLabelLayoutStyle = screenData.styleValue("buttonLabelLayoutStyle", "below").let { it.labelStyle() }
        buttonLabelFontColor = screenData.styleValue("buttonLabelFontColor", "#707070")
        buttonBackgroundColor = screenData.styleValue("buttonBackgroundColor", "#707070")
        buttonOpacity = screenData.styleValueInt("buttonOpacity", 100)

        // settings that depend on large or small devices...
        if (isLargeDevice) {

            // large devices...
            buttonCornerRadius = screenData.styleValueInt("buttonCornerRadiusLargeDevice", 0)
            buttonLabelFontSize = screenData.styleValueInt("buttonLabelFontSizeLargeDevice", 13)
            buttonPadding = screenData.styleValueInt("buttonPaddingLargeDevice", 15)
            buttonSize = screenData.styleValueInt("buttonSizeLargeDevice", 100)
        } else {

            // small devices...
            buttonCornerRadius = screenData.styleValueInt("buttonCornerRadiusSmallDevice", 0)
            buttonLabelFontSize = screenData.styleValueInt("buttonLabelFontSizeSmallDevice", 13)
            buttonPadding = screenData.styleValueInt("buttonPaddingSmallDevice", 15)
            buttonSize = screenData.styleValueInt("buttonSizeSmallDevice", 100)
        }

        // reference the container views in the layout file...
        containerView = thisScreensView.findViewById(R.id.containerView)
        containerViewHorizontal = thisScreensView.findViewById(R.id.containerViewHorizontal)
        containerViewHorizontalBottom = thisScreensView.findViewById(R.id.containerViewHorizontalBottom)
        containerHorizontalButtons = thisScreensView.findViewById(R.id.containerHorizontalButtons)

        // reference to vertical ScrollView in the layout file (TableView is in scroll view)
        verticalScrollView = thisScreensView.findViewById(R.id.verticalScrollView)
        verticalScrollView?.isHorizontalScrollBarEnabled = false
        verticalScrollView?.isVerticalScrollBarEnabled = false

        // reference to horizontal ScrollView in the layout file
        horizontalScrollView = thisScreensView.findViewById(R.id.horizontalScrollView)
        horizontalScrollView?.isHorizontalScrollBarEnabled = false
        horizontalScrollView?.isVerticalScrollBarEnabled = false

        // reference to TableLayout in the layout file...
        tableLayout = thisScreensView.findViewById(R.id.tableLayout)

        // flag as created..
        didCreate = true

        // return...
        return thisScreensView
    } // onCreateView...

    // onResume
    override fun onResume() {
        super.onResume()
        // BT_debugger.showIt(fragmentName + ":onResume");
        if (saveAsFileName.isNotEmpty()) {

            // check cache...
            var parseData = ""
            if (BT_fileManager.doesCachedFileExist(saveAsFileName)) {
                BT_debugger.showIt("$fragmentName:onStart using cached screen data")
                parseData = BT_fileManager.readTextFileFromCache(saveAsFileName)
                parseScreenData(parseData)
            } else {
                // get data from URL if we have one...
                if (dataURL.length > 1) {
                    BT_debugger.showIt("$fragmentName:onStart downloading screen data from URL")
                    refreshScreenData()
                } else {
                    // parse with "empty" data...
                    BT_debugger.showIt("$fragmentName:onStart using data from app's configuration file")
                    parseScreenData("")
                }
            }
        } // saveAsFileName
    }

    // end activity life-cycle events
    // ////////////////////////////////////////////////////////////////////////
    // need to re-build the button layout on screen orientation changes..
    override fun onConfigurationChanged(newConfig: Configuration) {
        super.onConfigurationChanged(newConfig)
        layoutButtons()
    }

    // handles button clicks
    override fun onClick(view: View) {
        BT_debugger.showIt("$fragmentName:button clicked")
        val buttonIndex = view.id

        // the BT_item
        val tappedItem = childItems?.get(buttonIndex) ?: return

        // add animation to the tapped buttonBox...
        val alphaFade = AlphaAnimation(0.3f, 1.0f)
        alphaFade.duration = 500
        alphaFade.fillAfter = true
        buttonBoxes?.get(view.id)?.startAnimation(alphaFade)

        launch {
            loadScreen(tappedItem)
        }
    }

    private enum class ButtonLayoutType {
        Grid,
        VerticalLeft,
        VerticalRight,
        HorizontalTop,
        HoriztonalBottom
    }

    private fun String?.layoutType(): ButtonLayoutType = when (this?.toLowerCase(Locale.US)) {
        "grid" -> ButtonLayoutType.Grid
        "verticalleft" -> ButtonLayoutType.VerticalLeft
        "verticalright" -> ButtonLayoutType.VerticalRight
        "horizontaltop" -> ButtonLayoutType.HorizontalTop
        "horizontalbottom" -> ButtonLayoutType.HoriztonalBottom
        else -> ButtonLayoutType.Grid
    }

    private enum class LabelLayoutStyle {
        Above,
        Below,
        Top,
        Bottom,
        Middle
    }

    private fun String?.labelStyle(): LabelLayoutStyle = when (this?.toLowerCase(Locale.US)) {
        "above" -> LabelLayoutStyle.Above
        "below" -> LabelLayoutStyle.Below
        "top" -> LabelLayoutStyle.Top
        "bottom" -> LabelLayoutStyle.Bottom
        "middle" -> LabelLayoutStyle.Middle
        else -> LabelLayoutStyle.Below
    }

    // we may need to add some height if labels are "above" or "below" to button's color or image...
    private val addHeight get() = when (buttonLabelLayoutStyle) {
        LabelLayoutStyle.Above, LabelLayoutStyle.Below -> (buttonLabelFontSize * 1.3).toInt()
        else -> 0
    }

    // layout buttons
    fun layoutButtons() {
        BT_debugger.showIt("$fragmentName:layoutButtons")

        // must have childItems...
        if (childItems.isNullOrEmpty()) {
            showAlert(getString(R.string.errorTitle), getString(R.string.errorItems))
        } else {

            // empty list of buttons...
            buttonBoxes?.clear()
            buttonSquares?.clear()
            buttonImages?.clear()

            // flag for if we are using a tableLayout or not...
            val usingTable = when (buttonLayoutStyle) {
                ButtonLayoutType.Grid -> true
                ButtonLayoutType.VerticalLeft -> true
                ButtonLayoutType.VerticalRight -> true
                else -> false
            }

            // font-size should not be more than 20...
            if (buttonLabelFontSize > 20) {
                buttonLabelFontSize = 18
            }

            when (buttonLayoutStyle) {
                ButtonLayoutType.Grid -> layoutGrid()
                ButtonLayoutType.VerticalRight -> layoutVertical()
                ButtonLayoutType.VerticalLeft -> layoutVertical()
                ButtonLayoutType.HoriztonalBottom -> layoutHoriztonal()
                ButtonLayoutType.HorizontalTop -> layoutHoriztonal()
            }

            // if we used a table layout...
            // add a few empty row at end so scrolling shows enough of bottom margin...
            if (usingTable) {
                for (t in 0..2) {
                    val tr = TableRow(this.activity)
                    tr.setPadding(0, 0, 0, 0)
                    val buttonBox = RelativeLayout(this.activity)
                    buttonBox.minimumWidth = buttonSize
                    buttonBox.minimumHeight = buttonSize
                    tr.addView(buttonBox, buttonSize, buttonSize)
                    tableLayout?.addView(tr, TableLayout.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT))
                }

                // invalidate tableLayout...
                tableLayout?.invalidate()
            }
        } // childItems.count	

        // hide progress...
        hideProgress()
    }

    private fun layoutGrid() {
        // hide horizontal scroller, not used...
        containerViewHorizontal?.visibility = View.GONE

        // tableLayout gets top padding same as button padding...
        tableLayout?.setPadding(0, buttonPadding, 0, buttonPadding)
        tableLayout?.removeAllViews()

        // figure out rows / cols...
        val buttonsPerRow = floor((deviceWidth / (buttonSize + buttonPadding)).toDouble()).toInt()
        val numButtons = childItems?.size ?: 0
        var rows = numButtons
        val cols = buttonsPerRow
        var cnt = 0
        // layout params for each table row...
        val trLp = ViewGroup.LayoutParams(ViewGroup.LayoutParams.FILL_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT)

        // outer loop rows..
        var tmpRowCounter = 0

        // must have some buttons...
        while (cnt < numButtons) {

            // table row holds parts...
            var tr: TableRow? = null
            if (tmpRowCounter < buttonsPerRow) {
                tr = TableRow(this.activity)
                tr.layoutParams = trLp
                tr.setPadding(0, 0, 0, 0)
            }

            // re-set for each row...
            tmpRowCounter++
            if (tmpRowCounter >= buttonsPerRow) tmpRowCounter = 0

            // inner loop cols..
            for (c in 0 until cols) {

                // increment counter here if NOT a vertical layout...
                if (cnt < numButtons) {

                    // BT_item...
                    val tmpItem = childItems?.get(cnt) ?: continue

                    // relative layout holds button parts...this is the button "size" and the background color...
                    val buttonBox = getButton(tmpItem, cnt, buttonSize, buttonSize + addHeight, buttonPadding, addHeight)

                    // add to list of "button boxes" to keep track of it...
                    if (buttonBox != null) buttonBoxes?.add(buttonBox)

                    // add buttonBox to the table row...
                    tr?.addView(buttonBox, buttonSize + buttonPadding, buttonSize + buttonPadding + addHeight)

                    // increment counter...
                    cnt++
                } // cnt < numButtons...
            } // for num cols...

            // add the row...
            tableLayout?.addView(tr, TableLayout.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT))

            // if addHeight was used we need a spacer between each row..
            if (addHeight > 0) {
                val spacerRow = TableRow(this.activity)
                spacerRow.setPadding(0, 0, 0, 0)
                val spacerBox = RelativeLayout(this.activity)
                spacerBox.minimumHeight = 15
                spacerRow.addView(spacerBox, 15, 15)
                tableLayout?.addView(spacerRow, TableLayout.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT))
            }
        } // for num buttons...
    }

    private fun layoutVertical() {
        // hide horizontal container, not used...
        containerViewHorizontal?.visibility = View.GONE

        // tableLayout gets top padding same as button padding...
        tableLayout?.setPadding(0, buttonPadding, 0, buttonPadding)
        tableLayout?.removeAllViews()

        // counts...
        // layout params for each table row...
        val trLp = ViewGroup.LayoutParams(ViewGroup.LayoutParams.FILL_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT)

        // left or right...
        val boxGravity = when (buttonLayoutStyle) {
            ButtonLayoutType.VerticalRight -> Gravity.END
            else -> Gravity.START
        }

        childItems?.forEachIndexed { index, tmpItem ->

            var tr = TableRow(this.activity)
            tr = TableRow(this.activity)
            tr.layoutParams = trLp
            tr.setPadding(0, 0, 0, 0)
            tr.gravity = boxGravity

            // relative layout holds button parts...this is the button "size" and the background color...
            val buttonBox = getButton(tmpItem, index, buttonSize, buttonSize + addHeight, buttonPadding, addHeight)

            // add to list of "button boxes" to keep track of it...
            if (buttonBox != null) buttonBoxes?.add(buttonBox)

            // add to layout...
            val lp = RelativeLayout.LayoutParams(buttonSize, buttonSize)
            if (buttonLayoutStyle == ButtonLayoutType.VerticalRight) {
                lp.addRule(RelativeLayout.ALIGN_PARENT_RIGHT)
            }
            buttonBox?.layoutParams = lp

            // add buttonBox to the table row...
            tr.addView(buttonBox, buttonSize + buttonPadding * 2, buttonSize + buttonPadding + addHeight)

            // add the row...
            tableLayout?.addView(tr, TableLayout.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT))
        }

        // if addHeight was used we need a spacer between each row..
        if (addHeight > 0) {
            val spacerRow = TableRow(this.activity)
            spacerRow.setPadding(0, 0, 0, 0)
            val spacerBox = RelativeLayout(this.activity)
            spacerBox.minimumHeight = 30
            spacerRow.addView(spacerBox, 30, 30)
            tableLayout?.addView(spacerRow, TableLayout.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT))
        }
    }

    private fun layoutHoriztonal() {
        // hide vertical scroller, not used...
        verticalScrollView?.visibility = View.GONE

        // remove horizontal buttons...
        containerHorizontalButtons?.removeAllViews()

        // assume horizontalTop
        containerViewHorizontalBottom?.setPadding(0, 0, 0, 0)

        // move the container view to the bottom of the screen if needed...
        if (buttonLayoutStyle == ButtonLayoutType.HoriztonalBottom) {
            containerViewHorizontalBottom?.setPadding(0, 0, 0, buttonPadding)
            val lp = RelativeLayout.LayoutParams(ViewGroup.LayoutParams.FILL_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT)
            lp.addRule(RelativeLayout.ALIGN_PARENT_BOTTOM)
            containerViewHorizontalBottom?.layoutParams = lp
        }

        // button box gravity...
        val numButtons = childItems?.size ?: 0
        var cnt = 0

        // loop numButtons...
        for (c in 0 until numButtons) {

            // increment counter here if NOT a vertical layout...
            if (cnt < numButtons) {

                // BT_item...
                val tmpItem = childItems?.get(cnt)
                if (tmpItem == null) {
                    cnt++
                    continue
                }

                // relative layout holds button parts...this is the button "size" and the background color...
                val buttonBox = getButton(tmpItem, cnt, buttonSize, buttonSize + buttonPadding + addHeight, buttonPadding, addHeight)

                // add to list of "button boxes" to keep track of it...
                if (buttonBox != null) buttonBoxes?.add(buttonBox)

                // add to layout...
                val lp = RelativeLayout.LayoutParams(buttonSize + buttonPadding, buttonSize + buttonPadding + addHeight)
                buttonBox?.layoutParams = lp

                // add buttonBox to the table row...
                containerHorizontalButtons?.addView(buttonBox)

                // increment counter...
                cnt++
            } // cnt > numButtons
        } // for

        // add a few more to right side so scroller is long enough...
        for (x in 0..1) {
            val buttonBox = RelativeLayout(this.activity)
            buttonBox.minimumWidth = buttonSize
            buttonBox.minimumHeight = buttonSize
            containerHorizontalButtons?.addView(buttonBox)
        }

        // invalidate scroller...
        containerHorizontalButtons?.invalidate()
    }

    // creates and returns a RelativeLayout with button parts...
    fun getButton(buttonJSON: BT_item, buttonIndex: Int, buttonWidth: Int, buttonHeight: Int, buttonPadding: Int, addHeight: Int): RelativeLayout? {
        val context = context ?: return null
        // BT_item...
        val titleText = BT_strings.getJsonPropertyValue(buttonJSON.jsonObject, "titleText", "")
        val iconName = BT_strings.getJsonPropertyValue(buttonJSON.jsonObject, "iconName", "")
        var imageName = BT_strings.getJsonPropertyValue(buttonJSON.jsonObject, "imageNameSmallDevice", "")
        var imageURL = BT_strings.getJsonPropertyValue(buttonJSON.jsonObject, "imageURLSmallDevice", "")
        if (buzztouch_appDelegate.rootApp.rootDevice.isLargeDevice) {
            imageName = BT_strings.getJsonPropertyValue(buttonJSON.jsonObject, "imageNameLargeDevice", "")
            imageURL = BT_strings.getJsonPropertyValue(buttonJSON.jsonObject, "imageURLLargeDevice", "")
        }

        // buttonBox is a square holding all the view parts...
        val buttonBox = RelativeLayout(context)

        // buttonBox and textLabel is taller for "above" and "below"...
        var tmpWidth = buttonSize + buttonPadding
        var tmpHeight = buttonSize + buttonPadding + addHeight
        if (buttonLayoutStyle == ButtonLayoutType.VerticalRight || buttonLayoutStyle == ButtonLayoutType.VerticalLeft) {
            tmpWidth += buttonPadding
            tmpHeight += buttonPadding
        }

        // we may need to add to the height of the box if we are using "above" or "below" textViews...
        val boxLp = RelativeLayout.LayoutParams(tmpWidth, tmpHeight)
        buttonBox.layoutParams = boxLp
        buttonBox.minimumWidth = tmpWidth
        buttonBox.minimumHeight = tmpHeight

        // create a square for background image / color...
        val bgColor = RelativeLayout(this.activity)
        val colorLp = RelativeLayout.LayoutParams(buttonSize, buttonSize)
        colorLp.addRule(RelativeLayout.CENTER_HORIZONTAL)

        when (buttonLabelLayoutStyle) {
            LabelLayoutStyle.Above -> colorLp.addRule(RelativeLayout.ALIGN_PARENT_BOTTOM)
            LabelLayoutStyle.Below -> colorLp.addRule(RelativeLayout.ALIGN_PARENT_TOP)
            else -> colorLp.addRule(RelativeLayout.CENTER_VERTICAL)
        }

        bgColor.layoutParams = colorLp
        bgColor.setBackgroundColor(BT_colorKt.getColorFromHexString(buttonBackgroundColor))
        buttonBox.addView(bgColor)

        // create an invisible button...
        val b = Button(this.activity)
        b.setBackgroundColor(Color.TRANSPARENT)
        b.setOnClickListener(this)
        b.id = buttonIndex
        b.minimumWidth = buttonSize
        b.minimumHeight = buttonSize

        // add the bgColor "square" to the buttonSquares array so we can updates it's background later if we need..
        buttonSquares?.add(buttonIndex, bgColor)

        // if we have text...
        if (titleText.length > 1) {
            val t = TextView(this.activity)
            t.text = titleText
            t.textSize = buttonLabelFontSize.toFloat()
            t.setTextColor(BT_colorKt.getColorFromHexString(buttonLabelFontColor))
            t.setLines(1)
            t.isSingleLine = true
            t.layoutParams = ViewGroup.LayoutParams(tmpWidth, tmpHeight)
            t.width = tmpWidth
            t.minimumWidth = tmpWidth
            t.maxWidth = tmpWidth
            t.height = tmpHeight
            t.minimumHeight = tmpHeight
            t.maxHeight = tmpHeight

            // gravity depends on button position..."above" and "below" need a bigger buttonBox...
            when (buttonLabelLayoutStyle) {
                LabelLayoutStyle.Middle -> t.gravity = Gravity.CENTER_VERTICAL or Gravity.CENTER_HORIZONTAL
                LabelLayoutStyle.Bottom, LabelLayoutStyle.Below -> t.gravity = Gravity.BOTTOM or Gravity.CENTER_HORIZONTAL
                LabelLayoutStyle.Top, LabelLayoutStyle.Above -> t.gravity = Gravity.TOP or Gravity.CENTER_HORIZONTAL
            }

            // add text label to background box...
            buttonBox.addView(t)
        }

        // fill our buttonImages array with an empty image...We do this so the buttonImages array has something
        // in this position (null to begin with). The buttonImageHandler will replace this null value with the
        // actual image after it loads it.
        buttonImages?.add(null)

        // if we have a background image (name or url)...
        if (imageName.length > 1 || imageURL.length > 1) {

            // if we have an image name and it's in the assets...
            if (imageName.length > 1) {
                if (BT_fileManager.getResourceIdFromBundle("drawable", imageName) > 0) {
                    BT_debugger.showIt("$fragmentName: using button image from project bundle: \"$imageName\"")

                    // do we need to round the image?
                    var d: Drawable
                    if (buttonCornerRadius > 0) {
                        d = BT_fileManager.getDrawableByName(imageName)

                        // we have a drawable, our rounding method needs a bitmap...
                        var bg = (d as BitmapDrawable).bitmap
                        bg = BT_viewUtilities.getRoundedImage(bg, buttonCornerRadius)

                        // convert it back to a drawable...
                        d = BitmapDrawable(bg)
                    } else {
                        d = BT_fileManager.getDrawableByName(imageName)
                    }
                    bgColor.setBackgroundDrawable(d)
                }
            } else {
                if (imageURL.length > 1) {

                    // get image from cache or download it...
                    launch {
                        val drawable = context.downloadImage(imageURL) ?: return@launch
                        val bmp = BT_viewUtilities.getRoundedImage(drawable.bitmap, buttonCornerRadius)
                        bgColor.background = BitmapDrawable(context.resources, bmp)
                    }
                }
            }
        }

        // if we have an icon name (must be in assets folder)...put on top of background image...
        if (iconName.length > 1) {
            if (BT_fileManager.getResourceIdFromBundle("drawable", iconName) > 0) {
                val i = ImageView(this.activity)
                i.layoutParams = ViewGroup.LayoutParams(ViewGroup.LayoutParams.FILL_PARENT, ViewGroup.LayoutParams.FILL_PARENT)
                i.scaleType = ScaleType.CENTER_INSIDE
                i.maxWidth = buttonSize
                i.maxHeight = buttonSize
                i.setImageDrawable(BT_fileManager.getDrawableByName(iconName))
                bgColor.addView(i)
            }
        }

        // handle opacity...
        if (buttonOpacity > 0) {
            val useAlpha = ("" + buttonOpacity).toDouble() / 100
            val opacityAnimation: Animation = AlphaAnimation(1.0f, useAlpha.toFloat())
            opacityAnimation.fillAfter = true
            bgColor.startAnimation(opacityAnimation)
        }

        // add the button to the buttonBox...
        bgColor.addView(b)

        // return...
        return buttonBox
    }

    // parse screenData...
    fun parseScreenData(theJSONString: String) {
        BT_debugger.showIt("$fragmentName:parseScreenData")
        // BT_debugger.showIt(fragmentName + ":parseScreenData " + theJSONString);
        // parse JSON string
        try {

            // empty data if previously filled...
            childItems?.clear()

            // if theJSONString is empty, look for child items in this screen's config data..
            var items: JSONArray? = null
            if (theJSONString.length < 1) {
                if (screenData.jsonObject.has("childItems")) {
                    items = screenData.jsonObject.getJSONArray("childItems")
                }
            } else {
                val raw = JSONObject(theJSONString)
                if (raw.has("childItems")) {
                    items = raw.getJSONArray("childItems")
                }
            }

            // loop items..
            if (items != null) {
                for (i in 0 until items.length()) {
                    val tmpJson = items.getJSONObject(i)
                    val tmpItem = BT_item()
                    if (tmpJson.has("itemId")) tmpItem.itemId = tmpJson.getString("itemId")
                    if (tmpJson.has("itemType")) tmpItem.itemType = tmpJson.getString("itemType")
                    tmpItem.jsonObject = tmpJson
                    childItems?.add(tmpItem)
                } // for
            } else {
                BT_debugger.showIt("$fragmentName:parseScreenData NO CHILD ITEMS?")
            }
        } catch (e: Exception) {
            BT_debugger.showIt("$fragmentName:parseScreenData EXCEPTION $e")
        }

        // layout buttons...
        layoutButtons()
    }

    // refresh screenData
    fun refreshScreenData() = launch {
        BT_debugger.showIt("$fragmentName:refreshScreenData")
        downloadScreenData(dataURL, saveAsFileName)
        if (!JSONData.isNullOrEmpty()) {
            parseScreenData(JSONData)
        }
    }

    // onCreateOptionsMenu...
    override fun onCreateOptionsMenu(menu: Menu, inflater: MenuInflater) {
        BT_debugger.showIt("$fragmentName:onCreateOptionsMenu JSON itemId: \"$screenItemId\"")

        /*
	 	 	Screens that extend BT_fragment should implement their own onCreateOptionsMenu() method to add
	 	 	action bar items specific to that screen. They should call super.onCreateOptionsMenu() AFTER
	 	 	adding customized items for that screen. 	  
	 	*/

        // for individual menu items...
        var menuItem: MenuItem? = null

        // back...
        if (dataURL.length > 1) {
            menuItem = menu.add(Menu.NONE, 1, 0, getString(R.string.refreshScreenData))
            menuItem.icon = BT_fileManager.getDrawableByName("bt_refresh.png")
            menuItem.setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM)
        }

        // call super...
        super.onCreateOptionsMenu(menu, inflater)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        BT_debugger.showIt(fragmentName + ":onOptionsItemSelected JSON itemId: \"" + screenItemId + "\" Selected Item's Id: " + item.itemId)
        when (item.itemId) {
            1 -> {
                // refreshScreenData...
                refreshScreenData()
                return true
            }
        }

        // return...
        return super.onOptionsItemSelected(item)
    }
}
