/*
 *	Copyright 2011, David Book, buzztouch.com
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are 
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the 
 *	following disclaimer. 
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list 
 *	of conditions and the following disclaimer in the documentation and/or other materials 
 *	provided with the distribution. 
 *
 *	Neither the name of David Book, or buzztouch.com nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without specific 
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *	OF SUCH DAMAGE. 
 */
package com.buzztouch.ui.bt_screens

import android.graphics.drawable.ColorDrawable
import android.os.Bundle
import android.view.LayoutInflater
import android.view.Menu
import android.view.MenuInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import android.widget.AbsListView
import android.widget.AdapterView.OnItemClickListener
import android.widget.BaseAdapter
import android.widget.ListView
import android.widget.TextView
import com.buzztouch.buzztouch_appDelegate
import com.buzztouch.R
import com.buzztouch.data_models.BT_item
import com.buzztouch.extensions.doesCachedFileExist
import com.buzztouch.extensions.downloadScreenData
import com.buzztouch.extensions.getDrawable
import com.buzztouch.extensions.isLargeDevice
import com.buzztouch.extensions.propertyValue
import com.buzztouch.extensions.styleValue
import com.buzztouch.utils.BT_color
import com.buzztouch.utils.BT_colorKt
import com.buzztouch.utils.BT_debugger
import com.buzztouch.utils.BT_fileManager
import kotlinx.coroutines.launch
import org.json.JSONArray
import org.json.JSONObject

class BT_screen_menuListSimple : BT_fragment(), AbsListView.OnScrollListener {
    var didCreate = false
    var didLoadData = false
    var JSONData = ""
    var childItems: ArrayList<BT_item>? = null
    var childItemAdapter: ChildItemAdapter? = null
    var myListView: ListView? = null
    var selectedIndex = -1
    var currentFirstVisibleItem = -1
    var currentVisibleItemCount = -1
    var currentTotalItemCount = -1
    var currentScrollState = -1

    // properties from JSON
    var dataURL = ""
    var saveAsFileName = ""
    var listStyle = ""
    var preventAllScrolling = ""
    var listBackgroundColor = ""
    var listRowBackgroundColor = ""
    var listRowSelectionStyle = ""
    var listTitleFontColor = ""
    var listRowSeparatorColor = ""

    // these depend on large or small device...
    var listRowHeight = 0
    var listTitleHeight = 0
    var listTitleFontSize = 0

    // ////////////////////////////////////////////////////////////////////////
    // fragment life-cycle events.
    // onCreateView...
    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {

        /*
 			Note: fragmentName property is already setup in the parent class (BT_fragment). This allows us 
 			to add the 	name of this class file to the LogCat console using the BT_debugger.
		*/
        // show life-cycle event in LogCat console...
        BT_debugger.showIt(fragmentName + ":onCreateView JSON itemId: \"" + screenData.itemId + "\" itemType: \"" + screenData.itemType + "\" itemNickname: \"" + screenData.itemNickname + "\"")

        // inflate the layout file for this screen...
        val thisScreensView = inflater.inflate(R.layout.bt_screen_menulistsimple, container, false)

        // ////////////////////////////////////////////////////////////////////////////////////////
        // Load all the variables for this screen using the screenData's JSON values...
        childItems = ArrayList()
        dataURL = screenData.propertyValue("dataURL")
        saveAsFileName = screenData.itemId + "_screenData.txt"
        if (dataURL.isEmpty()) BT_fileManager.deleteFile(saveAsFileName)

        // /////////////////////////////////////////////////////////////////
        // properties for both large and small devices...
        preventAllScrolling = screenData.styleValue("preventAllScrolling", "0")
        listBackgroundColor = screenData.styleValue("listBackgroundColor", "clear")
        listRowBackgroundColor = screenData.styleValue("listRowBackgroundColor", "clear")
        listRowSelectionStyle = screenData.styleValue("listRowSelectionStyle")
        listTitleFontColor = screenData.styleValue("listTitleFontColor", "#999999")
        listRowSeparatorColor = screenData.styleValue("listRowSeparatorColor", "#999999")

        // settings that depend on large or small devices...
        if (isLargeDevice) {

            // large devices...
            listRowHeight = screenData.styleValue("listRowHeightLargeDevice", "70").toIntOrNull() ?: 70
            listTitleHeight = screenData.styleValue("listTitleHeightLargeDevice", "60").toIntOrNull() ?: 60
            listTitleFontSize = screenData.styleValue("listTitleFontSizeLargeDevice", "25").toIntOrNull() ?: 25
        } else {
            // small devices...
            listRowHeight = screenData.styleValue("listRowHeightSmallDevice", "50").toIntOrNull() ?: 50
            listTitleHeight = screenData.styleValue("listTitleHeightSmallDevice", "50").toIntOrNull() ?: 50
            listTitleFontSize = screenData.styleValue("listTitleFontSizeSmallDevice", "22").toIntOrNull() ?: 22
        }

        // the list view will not show the last divider under the last element. We have a view here to trick it.
        val lastDivider = thisScreensView.findViewById(R.id.listViewEnd) as View
        lastDivider.setBackgroundColor(BT_colorKt.getColorFromHexString(listRowSeparatorColor))

        // reference to ListView in the layout file...
        myListView = thisScreensView.findViewById<View>(R.id.listView) as ListView
        myListView?.apply {
            setOnScrollListener(this@BT_screen_menuListSimple)
            setOnScrollListener(this@BT_screen_menuListSimple)
            isVerticalScrollBarEnabled = false
            isHorizontalScrollBarEnabled = false
            isFastScrollEnabled = false
            isSmoothScrollbarEnabled = false
        }

        // setup adapter for data...
        childItemAdapter = ChildItemAdapter()
        myListView?.adapter = childItemAdapter

        // prevent scrolling....
        preventAllScrolling = "1"
        if (preventAllScrolling.equals("1", ignoreCase = true)) {
            // prevent scrolling...
        }
        if (listBackgroundColor.isNotEmpty()) {
            myListView?.cacheColorHint = BT_colorKt.getColorFromHexString(listBackgroundColor)
            myListView?.setBackgroundColor(BT_colorKt.getColorFromHexString(listBackgroundColor))
        }

        // separator color...
        if (listRowSeparatorColor.isNotEmpty()) {
            val dividerColor = ColorDrawable(BT_colorKt.getColorFromHexString(listRowSeparatorColor))
            myListView?.divider = dividerColor
            myListView?.dividerHeight = 1
        }

        // flag as created..
        didCreate = true

        loadData()

        // return view...
        return thisScreensView
    }

    private fun loadData() {
        val parseData: String? = if (saveAsFileName.doesCachedFileExist()) BT_fileManager.readTextFileFromCache(saveAsFileName) else null
        parseData?.let { parseScreenData(it) } ?: launch {
            if (dataURL.isNotEmpty()) {
                BT_debugger.showIt("$fragmentName:onStart downloading screen data from URL")
                downloadScreenData(dataURL)
                if (JSONData.isEmpty()) showAlert(getString(R.string.errorTitle), getString(R.string.errorDownloadingData))
            } else {
                BT_debugger.showIt("$fragmentName:onStart using data from app's configuration file")
                parseScreenData("")
            }
        }
    }

    // onResume
    override fun onResume() {
        super.onResume()
        BT_debugger.showIt("$fragmentName:onResume")

        // select previous item if coming "back"...
        if (didCreate && didLoadData && selectedIndex > -1 && myListView?.adapter?.count ?: 0 > selectedIndex) {
            myListView?.apply {
                requestFocusFromTouch()
                setSelection(selectedIndex)
                setSelectionFromTop(selectedIndex, listRowHeight * selectedIndex)
            }
        }
    }

    // end fragment life-cycle events
    // ////////////////////////////////////////////////////////////////////////
    // handles onScrollStateChanged..
    override fun onScrollStateChanged(view: AbsListView, scrollState: Int) {
        // BT_debugger.showIt(fragmentName + ":onScrollStateChanged");
        currentScrollState = scrollState
    }

    // onScroll...
    override fun onScroll(view: AbsListView, firstVisibleItem: Int, visibleItemCount: Int, totalItemCount: Int) {
        // BT_debugger.showIt(fragmentName + ":onScroll");
        currentFirstVisibleItem = firstVisibleItem
        currentVisibleItemCount = visibleItemCount
        currentTotalItemCount = totalItemCount
    } // add something to end of list when scroll is done?...

    // parse screenData...
    fun parseScreenData(theJSONString: String) {
        BT_debugger.showIt("$fragmentName:parseScreenData")
        // BT_debugger.showIt(fragmentName + ":parseScreenData " + theJSONString);
        // parse JSON string
        try {

            // empty data if previously filled...
            childItems?.clear()

            // if theJSONString is empty, look for child items in this screen's config data..
            var items: JSONArray? = null
            if (theJSONString.length < 1) {
                if (screenData.jsonObject.has("childItems")) {
                    items = screenData.jsonObject.getJSONArray("childItems")
                }
            } else {
                val raw = JSONObject(theJSONString)
                if (raw.has("childItems")) {
                    items = raw.getJSONArray("childItems")
                }
            }

            // loop items..
            if (items != null) {
                for (i in 0 until items.length()) {
                    val tmpJson = items.getJSONObject(i)
                    val tmpItem = BT_item()
                    if (tmpJson.has("itemId")) tmpItem.itemId = tmpJson.getString("itemId")
                    if (tmpJson.has("itemType")) tmpItem.itemType = tmpJson.getString("itemType")
                    tmpItem.jsonObject = tmpJson
                    childItems?.add(tmpItem)
                } // for

                // flag data loaded...
                didLoadData = true
            } else {
                showAlert(getString(R.string.errorTitle), getString(R.string.errorItems))
                BT_debugger.showIt("$fragmentName:parseScreenData NO CHILD ITEMS?")
            }
        } catch (e: Exception) {
            BT_debugger.showIt("$fragmentName:parseScreenData EXCEPTION $e")
        }

        // setup list click listener
        val listItemClickHandler = OnItemClickListener { _, _, position, _ ->
            // remember the selected item's index...
            selectedIndex = position

            // the BT_item
            val tappedItem = childItems?.get(position)
            val loadScreenWithItemId = tappedItem.propertyValue("loadScreenWithItemId")
            val loadScreenWithNickname = tappedItem.propertyValue("loadScreenWithNickname")

            // bail if none...
            if (loadScreenWithItemId.equals("none", ignoreCase = true)) {
                return@OnItemClickListener
            }

            // itemId, nickname or object...
            var tapScreenLoadObject: BT_item? = null
            if (loadScreenWithItemId.length > 1 && !loadScreenWithItemId.equals("none", ignoreCase = true)) {
                BT_debugger.showIt("$fragmentName:handleItemTap loads screen with itemId: \"$loadScreenWithItemId\"")
                tapScreenLoadObject = buzztouch_appDelegate.rootApp.getScreenDataByItemId(loadScreenWithItemId)
            } else {
                if (loadScreenWithNickname.length > 1) {
                    BT_debugger.showIt("$fragmentName:handleItemTap loads screen with nickname: \"$loadScreenWithNickname\"")
                    tapScreenLoadObject = buzztouch_appDelegate.rootApp.getScreenDataByItemNickname(loadScreenWithNickname)
                } else {
                    try {
                        val obj = tappedItem?.jsonObject ?: return@OnItemClickListener
                        if (obj.has("loadScreenObject")) {
                            BT_debugger.showIt("$fragmentName:handleItemTap button loads screen object configured with JSON object.")
                            val tmpLoadScreen = obj.getJSONObject("loadScreenObject")
                            tapScreenLoadObject = BT_item()
                            if (tmpLoadScreen.has("itemId")) tapScreenLoadObject.itemId = tmpLoadScreen.getString("itemId")
                            if (tmpLoadScreen.has("itemNickname")) tapScreenLoadObject.itemNickname = tmpLoadScreen.getString("itemNickname")
                            if (tmpLoadScreen.has("itemType")) tapScreenLoadObject.itemType = tmpLoadScreen.getString("itemType")
                            if (obj.has("loadScreenObject")) tapScreenLoadObject.jsonObject = tmpLoadScreen
                        }
                    } catch (e: Exception) {
                        BT_debugger.showIt("$fragmentName:handleItemTap EXCEPTION reading screen-object for item: $e")
                    }
                }
            }

            // if we have a screen object to load from the right-button tap, build a BT_itme object...
            if (tapScreenLoadObject != null) {

                // call loadScreenObject in BT_fragment class...
                loadScreenObject(tappedItem, tapScreenLoadObject)
            } else {
                BT_debugger.showIt("$fragmentName:handleItemTap ERROR. No screen is connected to this item?")
                showAlert(getString(R.string.errorTitle), getString(R.string.errorNoScreenConnected))
            }
        }
        myListView?.onItemClickListener = listItemClickHandler

        // set the item adapter...
        myListView?.adapter = childItemAdapter
        myListView?.setSelectionFromTop(currentFirstVisibleItem, 0)

        // invalidate list so it repaints...
        myListView?.invalidate()

        // hide progress...
        hideProgress()
    }

    // refresh screenData
    fun refreshScreenData() {
        BT_debugger.showIt("$fragmentName:refreshScreenData")
        if (dataURL.isNotEmpty()) launch { downloadScreenData(dataURL, saveAsFileName) }
    }

    // /////////////////////////////////////////////////////////////////
    // Adapter for Child Items
    inner class ChildItemAdapter : BaseAdapter() {
        override fun getCount(): Int = childItems?.size ?: 0

        override fun getItem(position: Int): Any? = childItems?.get(position)

        override fun getItemId(position: Int): Long = childItems?.get(position)?.itemIndex?.toLong() ?: -1

        // returns the view for each row...only inflate once to enable view recyling..
        override fun getView(position: Int, convertView: View?, parent: ViewGroup): View {
            val rowView = convertView ?: LayoutInflater.from(parent.context).inflate(R.layout.bt_screen_menulistsimple_item, parent, false)

            // the BT_item values for this item..
            val tmpItem = childItems?.get(position)
            val titleText = tmpItem.propertyValue("titleText")

            // get layout parts for this row..
            val titleView = rowView.findViewById<View>(R.id.titleView) as TextView
            titleView.textSize = listTitleFontSize.toFloat()
            titleView.height = listRowHeight

            // tag the view...
            rowView.tag = tmpItem

            // title font color
            if (listTitleFontColor.length > 0) {
                titleView.setTextColor(BT_colorKt.getColorFromHexString(listTitleFontColor))
            }

            // title font size
            if (listTitleFontSize > 0) {
                titleView.textSize = listTitleFontSize.toFloat()
            }

            // title text...
            titleView.text = titleText

            // return
            return rowView
        }
    }

    // END child items adapter...

    // onCreateOptionsMenu...
    override fun onCreateOptionsMenu(menu: Menu, inflater: MenuInflater) {
        BT_debugger.showIt("$fragmentName:onCreateOptionsMenu JSON itemId: \"$screenItemId\"")

        /*
	 	 	Screens that extend BT_fragment should implement their own onCreateOptionsMenu() method to add
	 	 	action bar items specific to that screen. They should call super.onCreateOptionsMenu() AFTER
	 	 	adding customized items for that screen. 	  
	 	*/

        // for individual menu items...
        var menuItem: MenuItem? = null

        // refresh screen data...
        if (dataURL.length > 1) {
            menuItem = menu.add(Menu.NONE, 1, 0, getString(R.string.refreshScreenData))
            menuItem.icon = getDrawable(R.drawable.bt_refresh)
            menuItem.setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM)
        }

        // call super...
        super.onCreateOptionsMenu(menu, inflater)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        BT_debugger.showIt(fragmentName + ":onOptionsItemSelected JSON itemId: \"" + screenItemId + "\" Selected Item's Id: " + item.itemId)
        when (item.itemId) {
            1 -> {
                // refreshScreenData...
                refreshScreenData()
                return true
            }
        }

        // return...
        return super.onOptionsItemSelected(item)
    }
}
