/*
 *	Copyright 2011, David Book, buzztouch.com
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are 
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the 
 *	following disclaimer. 
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list 
 *	of conditions and the following disclaimer in the documentation and/or other materials 
 *	provided with the distribution. 
 *
 *	Neither the name of David Book, or buzztouch.com nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without specific 
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *	OF SUCH DAMAGE. 
 */
package com.buzztouch.ui.bt_screens

import android.app.AlertDialog
import android.content.DialogInterface
import android.os.Bundle
import android.view.LayoutInflater
import android.view.Menu
import android.view.MenuInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import android.view.animation.AlphaAnimation
import android.widget.Button
import android.widget.ImageView
import com.buzztouch.buzztouch_appDelegate
import com.buzztouch.R
import com.buzztouch.extensions.downloadScreenData
import com.buzztouch.extensions.propertyValue
import com.buzztouch.utils.BT_debugger
import com.buzztouch.utils.BT_fileManager
import com.buzztouch.utils.BT_viewUtilities
import kotlinx.coroutines.launch

class BT_screen_pptDoc : BT_fragment() {
    var didCreate = false
    var dataURL = ""
    var localFileName = ""
    var saveAsFileName = ""
    var openWithButton: Button? = null
    var downloadButton: Button? = null
    var documentIconView: ImageView? = null
    var confirmRefreshDialogue: AlertDialog? = null
    var showBrowserBarEmailDocument = ""
    var showBrowserBarRefresh = ""
    var forceRefresh = ""

    // onCreateView...
    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {

        /*
 			Note: fragmentName property is already setup in the parent class (BT_fragment). This allows us 
 			to add the 	name of this class file to the LogCat console using the BT_debugger.
		*/
        // show life-cycle event in LogCat console...
        BT_debugger.showIt(fragmentName + ":onCreateView JSON itemId: \"" + screenData.itemId + "\" itemType: \"" + screenData.itemType + "\" itemNickname: \"" + screenData.itemNickname + "\"")

        // inflate the layout file for this screen...
        val thisScreensView = inflater.inflate(R.layout.bt_screen_pptdoc, container, false)

        // reference to the document type icon...
        documentIconView = thisScreensView.findViewById<View>(R.id.documentTypeIcon) as ImageView
        documentIconView?.setImageDrawable(BT_fileManager.getDrawableByName("bt_screen_pptdoc.png"))

        // reference to "open" and "donwload" buttons...
        openWithButton = thisScreensView.findViewById<View>(R.id.openWithButton) as Button
        downloadButton = thisScreensView.findViewById<View>(R.id.downloadButton) as Button

        // fill JSON properties...
        localFileName = screenData.propertyValue("localFileName")
        dataURL = screenData.propertyValue("dataURL")
        showBrowserBarEmailDocument = screenData.propertyValue("showBrowserBarEmailDocument", "1")
        showBrowserBarRefresh = screenData.propertyValue("showBrowserBarRefresh", "0")
        forceRefresh = screenData.propertyValue("forceRefresh")

        // if we have NO localFileName and no dataURL, use the sample file for this plugin...
        if (localFileName.isEmpty() && dataURL.isEmpty()) {
            localFileName = "bt_screen_pptdoc_sample.ppt"
        }

        // figure out the saveAsFileName...
        if (localFileName.isNotEmpty()) {

            // use the file name in the JSON data...
            saveAsFileName = localFileName

            // copy the file from the /assets/BT_Docs folder to the cache if it doesn't already exist...
            if (BT_fileManager.doesProjectAssetExist("BT_Docs", saveAsFileName)) {
                if (!BT_fileManager.doesCachedFileExist(saveAsFileName)) {
                    BT_fileManager.copyAssetToCache("BT_Docs", saveAsFileName)
                }
            } else {
                showAlert(buzztouch_appDelegate.getApplication().getString(R.string.errorTitle), getString(R.string.errorDocumentNotInProject))
                BT_debugger.showIt("$fragmentName:onCreate. localFileName \"$saveAsFileName\" does not exist in project! ")
            }
        } else {
            saveAsFileName = screenData.itemId + "_screenData.ppt"
        }

        // remove file if we are force-refreshing...
        if (forceRefresh == "1") {
            BT_fileManager.deleteFile(saveAsFileName)
        }

        // change click event for "open with" button...
        openWithButton?.setOnClickListener { openDocInCache(saveAsFileName) }

        // click event for "document image"...
        documentIconView?.setOnClickListener { openDocInCache(saveAsFileName) }

        // click event of the download button...
        downloadButton?.setOnClickListener { downloadAndSaveFile(dataURL, saveAsFileName) }

        // if file is not in the cache, disable the openWith button...
        if (!BT_fileManager.doesCachedFileExist(saveAsFileName)) {
            openWithButton?.isEnabled = false
        }

        // hide download button if we don't have a URL...
        if (dataURL.length < 5) {
            downloadButton?.visibility = View.GONE
        } else {
            // if we have a dataURL, AND we have a cached "saveAsFileName", change label to "refresh"...
            if (BT_fileManager.doesCachedFileExist(saveAsFileName)) {
                downloadButton?.text = getString(R.string.refreshFromURL)
                downloadButton?.setOnClickListener { handleRefreshButton() }
            }
        }

        // add face effect to image (it's not a button, it's an image)...
        val alphaFade = AlphaAnimation(0.3f, 1.0f)
        alphaFade.duration = 500
        alphaFade.fillAfter = true
        documentIconView?.startAnimation(alphaFade)

        // return...
        return thisScreensView
    } // onCreateView...

    // openDocInCache...
    fun openDocInCache(fileName: String?) {
        BT_debugger.showIt("$fragmentName:openDocInCache. Calling helper method in BT_viewUtilities...")

        // make sure doc is in cache already...
        if (BT_fileManager.doesCachedFileExist(saveAsFileName)) {

            // use helper in BT_viewUtilities method to open the document...
            BT_viewUtilities.openDocInCacheWithMimeType(saveAsFileName, "application/ms-powerpoint", this.activity)
        } else {

            // show document not cached alert..
            showAlert(buzztouch_appDelegate.getApplication().getString(R.string.errorTitle), getString(R.string.errorDocumentNotInCache))
            BT_debugger.showIt("$fragmentName:handleEmailDocumentButton Cannot email document, document not in the cache?")
        }
    }

    // handleEmailButton...
    fun handleEmailButton() {
        BT_debugger.showIt("$fragmentName:handleEmailButton. Calling helper method in BT_viewUtilities...")

        // make sure device can send email...
        if (buzztouch_appDelegate.rootApp.rootDevice.canSendEmail()) {

            // make sure doc is in cache already...
            if (BT_fileManager.doesCachedFileExist(saveAsFileName)) {

                // use helper method in BT_viewUtilities to email document...
                BT_viewUtilities.emailDocumentInCacheWithMimeType(saveAsFileName, "application/ms-powerpoint", this.activity)
            } else {

                // show document not cached alert..
                showAlert(buzztouch_appDelegate.getApplication().getString(R.string.cannotEmailDocument), getString(R.string.errorDocumentNotInCache))
                BT_debugger.showIt("$fragmentName:handleEmailButton Cannot email document, document not in the cache?")
            }
        } else {

            // show cannot email alert...
            showAlert(buzztouch_appDelegate.getApplication().getString(R.string.cannotEmailDocument), getString(R.string.noNativeAppDescription))
            BT_debugger.showIt("$fragmentName:handleEmailButton Cannot email document, device cannot send emails?")
        }
    }

    // refresh button...
    fun handleRefreshButton() {
        if (dataURL.length > 1) {
            BT_debugger.showIt("$fragmentName:handleRefreshButton Current URL: $dataURL")

            // we have a dataURL, confirm if we have already downloaded it...
            if (BT_fileManager.doesCachedFileExist(saveAsFileName)) {

                // confirm...
                confirmRefreshDocument()
            } else {

                // download document...
                downloadAndSaveFile(dataURL, saveAsFileName)
            }
        } else {

            // show no URL alert...
            showAlert(buzztouch_appDelegate.getApplication().getString(R.string.cannotRefreshDocument), getString(R.string.errorNoURL))
            BT_debugger.showIt("$fragmentName:handleRefreshButton cannot refresh document, no dataURL provided")
        }
    }

    // confirmRefreshDocument...
    fun confirmRefreshDocument() {
        confirmRefreshDialogue = AlertDialog.Builder(this.activity).create()
        confirmRefreshDialogue?.apply {
            setTitle(getString(R.string.confirm))
            setMessage(getString(R.string.confirmRefreshDocument))
            setIcon(R.mipmap.ic_launcher)
            setButton(DialogInterface.BUTTON_POSITIVE, getString(R.string.yes)) { _, _ ->
                confirmRefreshDialogue?.dismiss()
                downloadAndSaveFile(dataURL, saveAsFileName)
            }
            setButton(DialogInterface.BUTTON_NEGATIVE, getString(R.string.no)) { _, _ ->
                confirmRefreshDialogue?.dismiss()
            }
        }

        // show the confirmation box...
        confirmRefreshDialogue?.show()
    }

    // download and save file....
    fun downloadAndSaveFile(dataURL: String?, saveAsFileName: String) {
        dataURL ?: return
        launch {
            documentIconView?.visibility = View.GONE
            downloadScreenData(dataURL, saveAsFileName)
            documentIconView?.visibility = View.VISIBLE
            openWithButton?.isEnabled = true
            downloadButton?.text = getString(R.string.refreshFromURL)

            // change click handler of download button to confirm refresh...
            downloadButton?.setOnClickListener { handleRefreshButton() }
        }
    }

    // END DownloadScreenDataThread and Handler
    // /////////////////////////////////////////////////////////////////	
    // onCreateOptionsMenu...
    override fun onCreateOptionsMenu(menu: Menu, inflater: MenuInflater) {
        BT_debugger.showIt("$fragmentName:onCreateOptionsMenu JSON itemId: \"$screenItemId\"")

        /*
	 	 	Screens that extend BT_fragment should implement their own onCreateOptionsMenu() method to add
	 	 	action bar items specific to that screen. They should call super.onCreateOptionsMenu() AFTER
	 	 	adding customized items for that screen. 	  
	 	*/

        // for individual menu items...
        var menuItem: MenuItem? = null

        // refresh page...
        if (showBrowserBarRefresh.equals("1", ignoreCase = true)) {
            menuItem = menu.add(Menu.NONE, 1, 0, getString(R.string.browserRefresh))
            menuItem.icon = BT_fileManager.getDrawableByName("bt_refresh.png")
            menuItem.setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM)
        }

        // email document...
        if (showBrowserBarEmailDocument.equals("1", ignoreCase = true)) {
            menuItem = menu.add(Menu.NONE, 2, 0, getString(R.string.browserEmailDocument))
            menuItem.icon = BT_fileManager.getDrawableByName("bt_email.png")
            menuItem.setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM)
        }

        // call super...
        super.onCreateOptionsMenu(menu, inflater)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        BT_debugger.showIt(fragmentName + ":onOptionsItemSelected JSON itemId: \"" + screenItemId + "\" Selected Item's Id: " + item.itemId)
        when (item.itemId) {
            1 -> {
                // refresh button...
                handleRefreshButton()
                return true
            }
            2 -> {
                // email button...
                handleEmailButton()
                return true
            }
        }

        // return...
        return super.onOptionsItemSelected(item)
    }
}
