/*
 *	Copyright 2012, David Book, buzztouch.com
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are 
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the 
 *	following disclaimer. 
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list 
 *	of conditions and the following disclaimer in the documentation and/or other materials 
 *	provided with the distribution. 
 *
 *	Neither the name of David Book, or buzztouch.com nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without specific 
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *	OF SUCH DAMAGE. 
 */
package com.buzztouch.ui.bt_screens

import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.graphics.drawable.BitmapDrawable
import android.media.AudioManager
import android.media.MediaPlayer
import android.os.Bundle
import android.os.Handler
import android.os.Message
import android.view.*
import android.view.animation.AlphaAnimation
import android.view.animation.Animation
import android.view.animation.TranslateAnimation
import android.widget.Button
import android.widget.ImageView
import android.widget.RelativeLayout
import android.widget.TextView
import com.buzztouch.buzztouch_appDelegate
import com.buzztouch.R
import com.buzztouch.data_models.BT_item
import com.buzztouch.extensions.downloadScreenData
import com.buzztouch.extensions.getStringOrNull
import com.buzztouch.extensions.hexColor
import com.buzztouch.extensions.propertyValue
import com.buzztouch.utils.*
import kotlinx.coroutines.launch
import org.json.JSONArray
import org.json.JSONObject
import kotlin.collections.ArrayList

class BT_screen_quiz : BT_fragment() {
    var didCreate = false
    private var JSONData = ""
    private var childItems: ArrayList<BT_item?>? = null
    var questionPool: ArrayList<BT_item?>? = null

    // bonus images...
    var bonusImages: ArrayList<Bitmap>? = null

    // media players for sound effects...
    var soundPlayerCorrect: MediaPlayer? = null
    var soundPlayerIncorrect: MediaPlayer? = null
    var soundPlayerFinished: MediaPlayer? = null
    var soundPlayerReward: MediaPlayer? = null

    // properties from JSON
    private var dataURL = ""
    private var saveAsFileName = ""
    private val quizStartCountdownSeconds = 3
    private var quizCountdownSeconds = 0
    private var quizNumberOfQuestions = 0
    private var quizQuestionDelay = 0
    private var quizRandomizeQuestions = ""
    private var quizShowCorrectAnswers = ""
    private var quizFinishScreenItemId = ""
    private var quizPointsPerAnswer = 0
    private var quizRewardIfPointsOver = 0
    private var quizRewardScreenItemId = ""
    private var quizSoundEffectFileNameCorrect = ""
    private var quizSoundEffectFileNameIncorrect = ""
    private var quizSoundEffectFileNameFinished = ""
    private var quizSoundEffectFileNameReward = ""
    private var quizShowTimer = ""
    private var sendResultsToURL = ""
    private var quizImageCornerRadius = 0
    private var quizFontColorQuestions = ""
    private var quizFontColorAnswers = ""
    private var quizButtonColorAnswers = ""
    private var quizButtonColorCorrect = ""
    private var quizButtonColorIncorrect = ""
    private var quizQuestionFontSizeSmallDevice = 0
    private var quizButtonFontSizeSmallDevice = 0
    private var quizQuestionFontSizeLargeDevice = 0
    private var quizButtonFontSizeLargeDevice = 0

    // quiz vars...
    var numberRight = 0
    var numberWrong = 0
    var numberStreak = 0
    var currentQuestion = 0
    var totalSeconds = 0
    var totalScore = 0
    var startTime: Long? = null
    var allowNext = 0
    var quizIsComplete = 0

    // UI Components...
    var imgQuestion: ImageView? = null
    var imgRight: ImageView? = null
    var imgWrong: ImageView? = null
    var imgBonus: ImageView? = null
    var txtQuestion: TextView? = null
    var txtScore: TextView? = null
    var txtTimer: TextView? = null
    var btnAnswer_1: Button? = null
    var btnAnswer_2: Button? = null
    var btnAnswer_3: Button? = null
    var btnAnswer_4: Button? = null
    var btnPanel: RelativeLayout? = null

    // onCreateView...
    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {

        /*
 			Note: fragmentName property is already setup in the parent class (BT_fragment). This allows us 
 			to add the 	name of this class file to the LogCat console using the BT_debugger.
		*/
        // show life-cycle event in LogCat console...
        BT_debugger.showIt(fragmentName + ":onCreateView JSON itemId: \"" + screenData.itemId + "\" itemType: \"" + screenData.itemType + "\" itemNickname: \"" + screenData.itemNickname + "\"")

        // inflate the layout file for this screen...
        val thisScreensView = inflater.inflate(R.layout.bt_screen_quiz, container, false)

        // init properties for JSON data...
        childItems = ArrayList()
        dataURL = BT_strings.getJsonPropertyValue(screenData.jsonObject, "dataURL", "")
        saveAsFileName = screenData.itemId + "_screenData.txt"
        sendResultsToURL = BT_strings.getJsonPropertyValue(screenData.jsonObject, "sendResultsToURL", "")
        quizRandomizeQuestions = BT_strings.getJsonPropertyValue(screenData.jsonObject, "quizRandomizeQuestions", "1")
        quizShowCorrectAnswers = BT_strings.getJsonPropertyValue(screenData.jsonObject, "quizShowCorrectAnswers", "1")
        quizFinishScreenItemId = BT_strings.getJsonPropertyValue(screenData.jsonObject, "quizFinishScreenItemId", "")
        quizRewardScreenItemId = BT_strings.getJsonPropertyValue(screenData.jsonObject, "quizRewardScreenItemId", "")
        quizSoundEffectFileNameCorrect = BT_strings.getJsonPropertyValue(screenData.jsonObject, "quizSoundEffectFileNameCorrect", "bt_screen_quiz_right.mp3")
        quizSoundEffectFileNameIncorrect = BT_strings.getJsonPropertyValue(screenData.jsonObject, "quizSoundEffectFileNameIncorrect", "")
        quizSoundEffectFileNameFinished = BT_strings.getJsonPropertyValue(screenData.jsonObject, "quizSoundEffectFileNameFinished", "")
        quizSoundEffectFileNameReward = BT_strings.getJsonPropertyValue(screenData.jsonObject, "quizSoundEffectFileNameReward", "")
        quizShowTimer = BT_strings.getJsonPropertyValue(screenData.jsonObject, "quizShowTimer", "1")
        quizFontColorQuestions = BT_strings.getStyleValueForScreen(screenData, "quizFontColorQuestions", "#707070")
        quizFontColorAnswers = BT_strings.getStyleValueForScreen(screenData, "quizFontColorAnswers", "#000000")
        quizButtonColorAnswers = BT_strings.getStyleValueForScreen(screenData, "quizButtonColorAnswers", "#C7C7C7")
        quizButtonColorCorrect = BT_strings.getStyleValueForScreen(screenData, "quizButtonColorCorrect", "#336600")
        quizButtonColorIncorrect = BT_strings.getStyleValueForScreen(screenData, "quizButtonColorIncorrect", "#990000")
        quizNumberOfQuestions = BT_strings.getJsonPropertyValue(screenData.jsonObject, "quizNumberOfQuestions", "0").toInt()
        quizQuestionDelay = BT_strings.getJsonPropertyValue(screenData.jsonObject, "quizQuestionDelay", "2").toInt()
        quizPointsPerAnswer = BT_strings.getJsonPropertyValue(screenData.jsonObject, "quizPointsPerAnswer", "150").toInt()
        quizRewardIfPointsOver = BT_strings.getJsonPropertyValue(screenData.jsonObject, "quizRewardIfPointsOver", "0").toInt()
        quizImageCornerRadius = BT_strings.getJsonPropertyValue(screenData.jsonObject, "quizImageCornerRadius", "10").toInt()
        quizQuestionFontSizeSmallDevice = BT_strings.getJsonPropertyValue(screenData.jsonObject, "quizQuestionFontSizeSmallDevice", "25").toInt()
        quizButtonFontSizeSmallDevice = BT_strings.getJsonPropertyValue(screenData.jsonObject, "quizButtonFontSizeSmallDevice", "18").toInt()
        quizQuestionFontSizeLargeDevice = BT_strings.getJsonPropertyValue(screenData.jsonObject, "quizQuestionFontSizeLargeDevice", "30").toInt()
        quizButtonFontSizeLargeDevice = BT_strings.getJsonPropertyValue(screenData.jsonObject, "quizButtonFontSizeLargeDevice", "20").toInt()

        // tell android we want device to control volume for possible sound effects...
        this.activity?.volumeControlStream = AudioManager.STREAM_MUSIC

        // init sound effect players to null..."startCountdown" will load them with media...
        soundPlayerCorrect = null
        soundPlayerIncorrect = null
        soundPlayerFinished = null
        soundPlayerReward = null
        numberRight = 0
        numberWrong = 0
        numberStreak = 0
        currentQuestion = 0
        totalSeconds = 0
        totalScore = 0
        allowNext = 0
        quizIsComplete = 0

        // init child items array...
        childItems = ArrayList()
        dataURL = BT_strings.getJsonPropertyValue(screenData.jsonObject, "dataURL", "")
        saveAsFileName = screenData.itemId + "_screenData.txt"
        if (dataURL.length < 1) BT_fileManager.deleteFile(saveAsFileName)

        // init image components...
        imgQuestion = thisScreensView.findViewById<View>(R.id.imgQuestion) as ImageView
        imgRight = thisScreensView.findViewById<View>(R.id.imgRight) as ImageView
        imgRight?.visibility = View.INVISIBLE
        imgWrong = thisScreensView.findViewById<View>(R.id.imgWrong) as ImageView
        imgWrong?.visibility = View.INVISIBLE
        imgBonus = thisScreensView.findViewById<View>(R.id.imgBonus) as ImageView
        imgBonus?.visibility = View.INVISIBLE

        // init text view components...
        txtQuestion = thisScreensView.findViewById<View>(R.id.txtQuestion) as TextView
        txtScore = thisScreensView.findViewById<View>(R.id.txtScore) as TextView
        txtTimer = thisScreensView.findViewById<View>(R.id.txtTimer) as TextView

        // set question, timer, and score font color...
        txtQuestion?.setTextColor(BT_colorKt.getColorFromHexString(quizFontColorQuestions))
        txtTimer?.setTextColor(BT_colorKt.getColorFromHexString(quizFontColorQuestions))
        txtScore?.setTextColor(BT_colorKt.getColorFromHexString(quizFontColorQuestions))

        // init button components...
        btnAnswer_1 = thisScreensView.findViewById<View>(R.id.btnAnswer_1) as Button
        btnAnswer_2 = thisScreensView.findViewById<View>(R.id.btnAnswer_2) as Button
        btnAnswer_3 = thisScreensView.findViewById<View>(R.id.btnAnswer_3) as Button
        btnAnswer_4 = thisScreensView.findViewById<View>(R.id.btnAnswer_4) as Button
        btnPanel = thisScreensView.findViewById<View>(R.id.layoutButtons) as RelativeLayout
        btnPanel?.visibility = View.INVISIBLE

        // set button text colors...
        btnAnswer_1?.setTextColor(BT_colorKt.getColorFromHexString(quizFontColorAnswers))
        btnAnswer_2?.setTextColor(BT_colorKt.getColorFromHexString(quizFontColorAnswers))
        btnAnswer_3?.setTextColor(BT_colorKt.getColorFromHexString(quizFontColorAnswers))
        btnAnswer_4?.setTextColor(BT_colorKt.getColorFromHexString(quizFontColorAnswers))

        // set button background colors...
        btnAnswer_1?.setBackgroundColor(BT_colorKt.getColorFromHexString(quizButtonColorAnswers))
        btnAnswer_2?.setBackgroundColor(BT_colorKt.getColorFromHexString(quizButtonColorAnswers))
        btnAnswer_3?.setBackgroundColor(BT_colorKt.getColorFromHexString(quizButtonColorAnswers))
        btnAnswer_4?.setBackgroundColor(BT_colorKt.getColorFromHexString(quizButtonColorAnswers))

        // set font size for buttons and question, this depends on device type...
        if (buzztouch_appDelegate.rootApp.rootDevice.isLargeDevice) {
            txtQuestion?.textSize = quizQuestionFontSizeLargeDevice.toFloat()
            btnAnswer_1?.textSize = quizButtonFontSizeLargeDevice.toFloat()
            btnAnswer_2?.textSize = quizButtonFontSizeLargeDevice.toFloat()
            btnAnswer_3?.textSize = quizButtonFontSizeLargeDevice.toFloat()
            btnAnswer_4?.textSize = quizButtonFontSizeLargeDevice.toFloat()
        } else {
            txtQuestion?.textSize = quizQuestionFontSizeSmallDevice.toFloat()
            btnAnswer_1?.textSize = quizButtonFontSizeSmallDevice.toFloat()
            btnAnswer_2?.textSize = quizButtonFontSizeSmallDevice.toFloat()
            btnAnswer_3?.textSize = quizButtonFontSizeSmallDevice.toFloat()
            btnAnswer_4?.textSize = quizButtonFontSizeSmallDevice.toFloat()
        }

        // setup click handlers
        btnAnswer_1?.setOnClickListener { v -> answerClick(v) }
        btnAnswer_2?.setOnClickListener { v -> answerClick(v) }
        btnAnswer_3?.setOnClickListener { v -> answerClick(v) }
        btnAnswer_4?.setOnClickListener { v -> answerClick(v) }

        // load up bonus images for better performance
        bonusImages = ArrayList()
        var tmpImg: Bitmap
        tmpImg = BitmapFactory.decodeResource(resources, R.drawable.bt_blank)
        bonusImages?.add(tmpImg)
        tmpImg = BitmapFactory.decodeResource(resources, R.drawable.bt_blank)
        bonusImages?.add(tmpImg)
        tmpImg = BitmapFactory.decodeResource(resources, R.drawable.bt_blank)
        bonusImages?.add(tmpImg)
        tmpImg = BitmapFactory.decodeResource(resources, R.drawable.bt_screen_quiz_3x)
        bonusImages?.add(tmpImg)
        tmpImg = BitmapFactory.decodeResource(resources, R.drawable.bt_screen_quiz_4x)
        bonusImages?.add(tmpImg)
        tmpImg = BitmapFactory.decodeResource(resources, R.drawable.bt_screen_quiz_5x)
        bonusImages?.add(tmpImg)
        tmpImg = BitmapFactory.decodeResource(resources, R.drawable.bt_screen_quiz_6x)
        bonusImages?.add(tmpImg)
        tmpImg = BitmapFactory.decodeResource(resources, R.drawable.bt_screen_quiz_7x)
        bonusImages?.add(tmpImg)
        tmpImg = BitmapFactory.decodeResource(resources, R.drawable.bt_screen_quiz_8x)
        bonusImages?.add(tmpImg)
        tmpImg = BitmapFactory.decodeResource(resources, R.drawable.bt_screen_quiz_9x)
        bonusImages?.add(tmpImg)
        tmpImg = BitmapFactory.decodeResource(resources, R.drawable.bt_screen_quiz_10x)
        bonusImages?.add(tmpImg)

        // flag as created..
        didCreate = true

        // return...
        return thisScreensView
    } // onCreateView...

    // onStart...
    override fun onStart() {
        // BT_debugger.showIt(fragmentName + ":onStart");
        super.onStart()
    }

    // onResume...
    override fun onResume() {
        // BT_debugger.showIt(fragmentName + ":onResume");
        super.onResume()
        if (saveAsFileName.length > 1) {

            // check cache...
            var parseData = ""
            if (BT_fileManager.doesCachedFileExist(saveAsFileName)) {
                BT_debugger.showIt("$fragmentName:onStart using cached screen data")
                parseData = BT_fileManager.readTextFileFromCache(saveAsFileName)
                parseScreenData(parseData)
            } else {
                // get data from URL if we have one...
                if (dataURL.length > 1) {
                    BT_debugger.showIt("$fragmentName:onStart downloading screen data from URL")
                    refreshScreenData()
                } else {
                    // parse with "empty" data...
                    BT_debugger.showIt("$fragmentName:onStart using data from app's configuration file")
                    parseScreenData("")
                }
            }
        } // saveAsFileName
    }

    // onPause
    override fun onPause() {
        super.onPause()
        // BT_debugger.showIt(fragmentName + ":onPause");

        // kill possible sound effect players...
        killQuizSounds()
    }

    // onStop
    override fun onStop() {
        super.onStop()

        // kill timer for countdown timer...
        if (updateCountdownHandler != null) {
            updateCountdownHandler?.removeCallbacks(mUpdateCountdownTask)
        }

        // kill timer for running quiz timer...
        if (updateTimeHandler != null) {
            updateTimeHandler?.removeCallbacks(mUpdateTimeTask)
        }
        if (delayHandler != null) {
            delayHandler?.removeCallbacks(mDelayTask)
        }

        // kill possible sound effect players...
        killQuizSounds()
    } // onStop...

    // parse screenData...
    fun parseScreenData(theJSONString: String) {
        BT_debugger.showIt("$fragmentName:parseScreenData")
        // BT_debugger.showIt(fragmentName + ":parseScreenData " + theJSONString);
        // parse JSON string

        // hide controls
        txtTimer?.visibility = View.INVISIBLE
        txtScore?.visibility = View.INVISIBLE
        txtQuestion?.visibility = View.INVISIBLE
        btnPanel?.visibility = View.INVISIBLE

        // init question pool...
        questionPool = ArrayList()
        childItems = ArrayList()
        try {

            // if theJSONString is empty, look for child items in this screen's config data..
            var items: JSONArray? = null
            if (theJSONString.length < 1) {
                if (screenData.jsonObject.has("childItems")) {
                    items = screenData.jsonObject.getJSONArray("childItems")
                }
            } else {
                val raw = JSONObject(theJSONString)
                if (raw.has("childItems")) {
                    items = raw.getJSONArray("childItems")
                }
            }

            // loop items..
            if (items != null) {
                for (i in 0 until items.length()) {
                    val tmpJson = items.getJSONObject(i)
                    val tmpItem = BT_item()
                    if (tmpJson.has("itemId")) tmpItem.itemId = tmpJson.getString("itemId")
                    if (tmpJson.has("itemType")) tmpItem.itemType = tmpJson.getString("itemType")
                    tmpItem.jsonObject = tmpJson
                    childItems?.add(tmpItem)
                    questionPool?.add(tmpItem)
                } // for

                // figure out quizNumberOfQuestions...
                val childItemSize = childItems?.size ?: 0
                if (quizNumberOfQuestions == 0 || quizNumberOfQuestions > childItemSize) quizNumberOfQuestions = childItemSize

                // hide progress...
                hideProgress()

                // start countdown to quiz start..
                startCountdown()

                // bail...
                return
            } else {
                BT_debugger.showIt("$fragmentName:parseScreenData NO CHILD ITEMS?")
            }
        } catch (e: Exception) {
            BT_debugger.showIt("$fragmentName:parseScreenData EXCEPTION $e")
        }

        // hide progress...
        hideProgress()

        // must have childItems...
        if (childItems.isNullOrEmpty()) {
            showAlert(getString(R.string.errorTitle), getString(R.string.errorItems))
        }
    }

    // refresh screenData
    fun refreshScreenData() {
        BT_debugger.showIt("$fragmentName:refreshScreenData")
        showProgress(null, null)

        // hide controls
        txtTimer?.visibility = View.INVISIBLE
        txtScore?.visibility = View.INVISIBLE
        txtQuestion?.visibility = View.INVISIBLE
        btnPanel?.visibility = View.INVISIBLE
        if (dataURL.length > 1) {
            launch {
                buzztouch_appDelegate.rootApp.currentScreenData = screenData
                JSONData = downloadScreenData(dataURL, saveAsFileName)
            }
        } else {
            BT_debugger.showIt("$fragmentName:refreshScreenData NO DATA URL for this screen? Not downloading.")
        }
    }

    // sends results to URL on quiz end (not fired if sendResultsToURL is empty)...
    fun sendQuizResultsToURL() {
        BT_debugger.showIt("$fragmentName:sendQuizResultsToURL")
        if (sendResultsToURL.length > 1) {
            // TODO: Why was this included? Does not appear to be used
            // val url = "$sendResultsToURL&totalPoints=$totalScore&totalSeconds=$totalSeconds&numberQuestions=$totalScore&numberCorrect=$numberRight&numberIncorrect=$numberWrong"

            // merge remaining vars...
            val useURL = BT_strings.mergeBTVariablesInString(dataURL)
            BT_debugger.showIt("$fragmentName: sending results to: $useURL")

            // fetch the URL (ignore results)...
            launch {
                downloadScreenData(dataURL)
            }
        }
    }

    // setup quiz sound effect players...
    fun setupQuizSounds() {
        BT_debugger.showIt("$fragmentName:setupQuizSounds")

        // four possible sound effect file names provided in JSON. If a file name is setup, the file must
        // exist in the /assets/BT_Audio folder
        // 	quizSoundEffectFileNameCorrect
        // 	quizSoundEffectFileNameIncorrect
        // 	quizSoundEffectFileNameFinished
        // 	quizSoundEffectFileNameReward

        // quizSoundEffectFileNameCorrect...
        if (quizSoundEffectFileNameCorrect.isNotEmpty()) {
            if (BT_fileManager.doesProjectAssetExist("BT_Audio", quizSoundEffectFileNameCorrect)) {
                try {
                    val afd = buzztouch_appDelegate.getApplication().assets.openFd("BT_Audio/$quizSoundEffectFileNameCorrect")
                    soundPlayerCorrect = MediaPlayer()
                    soundPlayerCorrect?.setDataSource(afd.fileDescriptor, afd.startOffset, afd.length)
                    soundPlayerCorrect?.prepareAsync()
                } catch (e: Exception) {
                    BT_debugger.showIt("$fragmentName:EXCEPTION setting up soundPlayerCorrect. Message: $e")
                }
            }
        }

        // quizSoundEffectFileNameIncorrect...
        if (quizSoundEffectFileNameIncorrect.length > 1) {
            if (BT_fileManager.doesProjectAssetExist("BT_Audio", quizSoundEffectFileNameIncorrect)) {
                try {
                    val afd = buzztouch_appDelegate.getApplication().assets.openFd("BT_Audio/$quizSoundEffectFileNameIncorrect")
                    soundPlayerIncorrect = MediaPlayer()
                    soundPlayerIncorrect?.setDataSource(afd.fileDescriptor, afd.startOffset, afd.length)
                    soundPlayerIncorrect?.prepareAsync()
                } catch (e: Exception) {
                    BT_debugger.showIt("$fragmentName:EXCEPTION setting up soundPlayerIncorrect. Message: $e")
                }
            }
        }

        // quizSoundEffectFileNameFinished...
        if (quizSoundEffectFileNameFinished.length > 1) {
            if (BT_fileManager.doesProjectAssetExist("BT_Audio", quizSoundEffectFileNameFinished)) {
                try {
                    val afd = buzztouch_appDelegate.getApplication().assets.openFd("BT_Audio/$quizSoundEffectFileNameFinished")
                    soundPlayerFinished = MediaPlayer()
                    soundPlayerFinished?.setDataSource(afd.fileDescriptor, afd.startOffset, afd.length)
                    soundPlayerFinished?.prepareAsync()
                } catch (e: Exception) {
                    BT_debugger.showIt("$fragmentName:EXCEPTION setting up quizSoundEffectFileNameFinished. Message: $e")
                }
            }
        }

        // quizSoundEffectFileNameReward...
        if (quizSoundEffectFileNameReward.length > 1) {
            if (BT_fileManager.doesProjectAssetExist("BT_Audio", quizSoundEffectFileNameReward)) {
                try {
                    val afd = buzztouch_appDelegate.getApplication().assets.openFd("BT_Audio/$quizSoundEffectFileNameReward")
                    soundPlayerReward = MediaPlayer()
                    soundPlayerReward?.setDataSource(afd.fileDescriptor, afd.startOffset, afd.length)
                    soundPlayerReward?.prepareAsync()
                } catch (e: Exception) {
                    BT_debugger.showIt("$fragmentName:EXCEPTION setting up quizSoundEffectFileNameReward. Message: $e")
                }
            }
        }
    }

    // kill possible sound effects..
    fun killQuizSounds() {
        BT_debugger.showIt("$fragmentName:killQuizSounds")

        // soundPlayerCorrect...
        soundPlayerCorrect?.let {
            if (it.isPlaying) it.stop()
            it.release()
            soundPlayerCorrect = null
        }

        // soundPlayerIncorrect...
        soundPlayerIncorrect?.let {
            if (it.isPlaying) it.stop()
            it.release()
            soundPlayerIncorrect = null
        }

        // soundPlayerFinished...
        soundPlayerFinished?.let {
            if (it.isPlaying) it.stop()
            it.release()
            soundPlayerFinished = null
        }

        // soundPlayerReward...
        soundPlayerReward?.let {
            if (it.isPlaying) it.stop()
            it.release()
            soundPlayerReward = null
        }
    }

    // answer button clicks
    fun answerClick(v: View) {
        BT_debugger.showIt("$fragmentName:answerClick")
        val clickedButton = v as Button

        // buttons depend on whether quiz is over or not
        if (quizIsComplete < 1) {
            val selectedAnswer = clickedButton.text.toString()
            BT_debugger.showIt("$fragmentName:answerClick The selected answer was: \"$selectedAnswer\"")

            // prevents super fast button taps..
            if (allowNext > 0) {
                if (currentQuestion > -1 && !childItems.isNullOrEmpty()) {

                    // flag..
                    allowNext = 0
                    val tmpQuestion = childItems?.get(currentQuestion)
                    val correctAnswer = tmpQuestion?.jsonObject.getStringOrNull("correctAnswerText") ?: ""

                    // do we show correct answers???
                    if (quizShowCorrectAnswers.equals("1", ignoreCase = true)) {

                        // change button backgrounds, red for wrong, green for right
                        if (correctAnswer == btnAnswer_1?.text.toString()) {
                            btnAnswer_1?.setBackgroundColor(BT_colorKt.getColorFromHexString(quizButtonColorCorrect))
                        } else {
                            btnAnswer_1?.setBackgroundColor(BT_colorKt.getColorFromHexString(quizButtonColorIncorrect))
                        }
                        if (correctAnswer == btnAnswer_2?.text.toString()) {
                            btnAnswer_2?.setBackgroundColor(BT_colorKt.getColorFromHexString(quizButtonColorCorrect))
                        } else {
                            btnAnswer_2?.setBackgroundColor(BT_colorKt.getColorFromHexString(quizButtonColorIncorrect))
                        }
                        if (correctAnswer == btnAnswer_3?.text.toString()) {
                            btnAnswer_3?.setBackgroundColor(BT_colorKt.getColorFromHexString(quizButtonColorCorrect))
                        } else {
                            btnAnswer_3?.setBackgroundColor(BT_colorKt.getColorFromHexString(quizButtonColorIncorrect))
                        }
                        if (correctAnswer == btnAnswer_4?.text.toString()) {
                            btnAnswer_4?.setBackgroundColor(BT_colorKt.getColorFromHexString(quizButtonColorCorrect))
                        } else {
                            btnAnswer_4?.setBackgroundColor(BT_colorKt.getColorFromHexString(quizButtonColorIncorrect))
                        }
                    } // show correct answers...

                    // show image, calculate score..
                    if (selectedAnswer == correctAnswer.toString()) {

                        // counts..
                        numberRight = numberRight + 1
                        numberStreak = numberStreak + 1
                        if (numberStreak >= 10) numberStreak = 10

                        // set score
                        totalScore = if (numberStreak > 2) {
                            totalScore + quizPointsPerAnswer * numberStreak
                        } else {
                            totalScore + quizPointsPerAnswer
                        }

                        // increment score
                        txtScore?.text = totalScore.toString()

                        // set bonus image
                        bonusImages?.get(numberStreak)?.let { imgBonus?.setImageBitmap(it) }

                        // play possible correct sound...
                        if (soundPlayerCorrect != null) {
                            soundPlayerCorrect?.start()
                        }

                        // fade images in...
                        fadeImageIn(imgRight)
                        fadeImageIn(imgBonus)

                        // fade images out...
                        fadeImageOut(imgRight)
                        fadeImageOut(imgBonus)
                    } else {

                        // play possible incorrect sound...
                        soundPlayerIncorrect?.start()
                        numberWrong += 1
                        numberStreak = 0
                        fadeImageIn(imgWrong)
                        fadeImageOut(imgWrong)
                    }

                    // increment question
                    currentQuestion += 1

                    // timer handle's showNextQuestion
                    if (quizQuestionDelay > 10) quizQuestionDelay = 2
                    delayHandler?.removeCallbacks(mDelayTask)
                    delayHandler?.postDelayed(mDelayTask, (quizQuestionDelay * 1000).toLong())
                } else {
                    showAlert("Quiz Not Started", "Try using the refresh button on the devices menu?")
                }
            } // allowNext
        } else {

            // ////////////////////////////////////////////////////
            // button clicks are different if quiz is complete...
            // button_1 becomes "try again"
            // button_2 becomes "continue" or "show reward"
            // button_3 becomes "quit"

            // button_1
            if (clickedButton.id == R.id.btnAnswer_1) {
                startCountdown()
            }

            // button_2
            if (clickedButton.id == R.id.btnAnswer_2) {

                // holds the next screen to show...
                var nextScreenObject: BT_item? = null

                // reward screen and user earned enough points...
                if (quizRewardScreenItemId.length > 0 && totalScore >= quizRewardIfPointsOver) {

                    // instantiate the reward screen..
                    BT_debugger.showIt("$fragmentName:answerClick creating reward screen with itemId: $quizRewardScreenItemId")
                    nextScreenObject = buzztouch_appDelegate.rootApp.getScreenDataByItemId(quizRewardScreenItemId)
                } else {

                    // if we have a quizFinishScreenItemId...
                    if (quizFinishScreenItemId.length > 1) {

                        // instantiate the finish screen..
                        BT_debugger.showIt("$fragmentName:answerClick creating finished screen with itemId: $quizFinishScreenItemId")
                        nextScreenObject = buzztouch_appDelegate.rootApp.getScreenDataByItemId(quizFinishScreenItemId)
                    }
                } // reward screen and user earned enough points...

                // show next screen if we created on, else quit...
                if (nextScreenObject != null) {

                    // load the next screen...
                    loadScreenObject(null, nextScreenObject)
                } else {
                    (activity as BT_activity_host?)?.onBackPressed()
                }
            }

            // button_3
            if (clickedButton.id == R.id.btnAnswer_3) {
                (activity as BT_activity_host?)?.onBackPressed()
            }
        }
    }

    // start countdown...
    fun startCountdown() {
        BT_debugger.showIt("$fragmentName:startCountdown")

        // kill possible sounds...
        killQuizSounds()

        // reset possible sounds...
        setupQuizSounds()

        // set image to default...
        var tmpImageName = "bt_screen_quiz_bgsmall.png"
        if (buzztouch_appDelegate.rootApp.rootDevice.isLargeDevice) {
            tmpImageName = "bt_screen_quiz_bglarge.png"
        }
        var d = BT_fileManager.getDrawableByName(tmpImageName)
        if (quizImageCornerRadius > 0) {
            var b = (d as BitmapDrawable).bitmap
            b = BT_viewUtilities.getRoundedImage(b, quizImageCornerRadius)
            d = BitmapDrawable(b)
        }
        imgQuestion?.setImageDrawable(d)
        imgQuestion?.visibility = View.VISIBLE
        imgQuestion?.setAlpha(255)

        // hide controls
        txtTimer?.visibility = View.INVISIBLE
        txtScore?.visibility = View.VISIBLE
        txtQuestion?.visibility = View.INVISIBLE
        btnPanel?.visibility = View.INVISIBLE

        // set score to countdown number...
        txtScore?.text = quizStartCountdownSeconds.toString()

        // start countdown timer...
        quizCountdownSeconds = 0
        updateCountdownHandler?.removeCallbacks(mUpdateCountdownTask)
        updateCountdownHandler?.postDelayed(mUpdateCountdownTask, 100)
    }

    // ///////////////////////////////////////////////////////////////////
    // handles timer updates for quiz "countdown starting"
    var updateCountdownHandler: Handler? = object : Handler() {
        override fun handleMessage(msg: Message) {

            // quizCountdownSeconds is counter for quizStartCountdownSeconds
            if (quizCountdownSeconds > quizStartCountdownSeconds) {

                // reset quizCountdown seconds for next time...
                quizCountdownSeconds = 0

                // start quiz...
                startQuiz()
            } else {

                // update score text to show countdown...
                if (quizCountdownSeconds >= quizStartCountdownSeconds) {
                    txtScore?.text = "Go!"
                } else {
                    txtScore?.text = "" + (quizStartCountdownSeconds - quizCountdownSeconds)
                }

                // add one to  quizCountdownSeconds...
                quizCountdownSeconds = quizCountdownSeconds + 1

                // reset timer to trigger this again in 1 second (1000 milliseconds)...
                this.removeCallbacks(mUpdateCountdownTask)
                this.postDelayed(mUpdateCountdownTask, 1000)
            }
        }
    }
    private val mUpdateCountdownTask = Runnable {
        if (quizCountdownSeconds - 1 <= quizStartCountdownSeconds) {
            val message = updateCountdownHandler?.obtainMessage() ?: return@Runnable
            updateCountdownHandler?.sendMessage(message)
        }
    }

    // end timer stuff
    // ///////////////////////////////////////////////////////////////////
    // starts quiz..
    fun startQuiz() {
        BT_debugger.showIt("$fragmentName:startQuiz")

        // stop countdown timer...
        updateCountdownHandler?.removeCallbacks(mUpdateCountdownTask)

        // reset values...
        numberRight = 0
        numberWrong = 0
        numberStreak = 0
        totalSeconds = 0
        totalScore = 0
        currentQuestion = 0
        allowNext = 1
        quizIsComplete = 0

        // reset text
        txtQuestion?.text = ""
        txtScore?.text = ""
        txtTimer?.text = ""
        txtTimer?.visibility = View.VISIBLE
        txtScore?.visibility = View.VISIBLE
        txtQuestion?.visibility = View.VISIBLE
        btnAnswer_1?.text = ""
        btnAnswer_2?.text = ""
        btnAnswer_3?.text = ""
        btnAnswer_4?.text = ""

        // randomize questions from pool then grab "x" number for quiz
        val questions = (questionPool?.filterNotNull() ?: listOf()).toMutableList()
        val shouldRandomize = (quizRandomizeQuestions == "1")
            .or(quizRandomizeQuestions.equals("YES", ignoreCase = true))
        if (shouldRandomize) {
            questions.shuffle()
        }

        // start timer...
        startTime = System.currentTimeMillis()
        updateTimeHandler?.removeCallbacks(mUpdateTimeTask)
        updateTimeHandler?.postDelayed(mUpdateTimeTask, 100)

        // show next question
        showNextQuestion()
    }

    // shows next question
    fun showNextQuestion() {
        BT_debugger.showIt(fragmentName + ":showNextQuestion Question Number: " + (currentQuestion + 1))

        // hide current image...
        val tmpBlankImg = "bt_blank.png"
        val dblank = BT_fileManager.getDrawableByName(tmpBlankImg)
        imgQuestion?.setImageDrawable(dblank)
        showQuestionImage()

        // set button backgrounds and show the buttons...
        listOfNotNull(btnAnswer_1, btnAnswer_2, btnAnswer_3, btnAnswer_4).forEach {
            it.setBackgroundColor(quizButtonColorAnswers.hexColor())
            it.visibility = View.VISIBLE
        }

        // quiz may be over
        if (currentQuestion >= childItems?.size ?: 0) {
            imgQuestion?.visibility = View.INVISIBLE
            endQuiz()
        } else {

            // slide in buttons..
            slideButtons()

            // slide in question
            slideQuestion()

            // get next question object (BT_item)...
            val nextQuestion = childItems?.get(currentQuestion)

            // set question text
            val questionText = BT_strings.getJsonPropertyValue(nextQuestion?.jsonObject, "questionText", "")
            txtQuestion?.text = questionText
            BT_debugger.showIt("$fragmentName:showNextQuestion. \"$questionText\"")

            // load image?
            var tmpImageFileName = nextQuestion?.jsonObject.getStringOrNull("imageNameSmallDevice") ?: ""
            var tmpImageURL = nextQuestion?.jsonObject.getStringOrNull("imageURLSmallDevice") ?: ""

            // images are different if this is a large device...
            if (buzztouch_appDelegate.rootApp.rootDevice.isLargeDevice) {
                tmpImageFileName = nextQuestion?.jsonObject.getStringOrNull("imageNameLargeDevice") ?: ""
                tmpImageURL = nextQuestion?.jsonObject.getStringOrNull("imageURLLargeDevice") ?: ""
            }

            // do we have an image name or URL in the JSON?...
            if (tmpImageFileName.length > 1 || tmpImageURL.length > 1) {

                // image in project...
                if (tmpImageFileName.length > 1) {
                    BT_debugger.showIt("$fragmentName:showNextQuestion. Setting image from /drawables directory: \"$tmpImageFileName\"")
                    var d = BT_fileManager.getDrawableByName(tmpImageFileName)
                    if (quizImageCornerRadius > 0) {
                        var b = (d as BitmapDrawable).bitmap
                        b = BT_viewUtilities.getRoundedImage(b, quizImageCornerRadius)
                        d = BitmapDrawable(b)
                    }
                    imgQuestion?.setImageDrawable(d)
                    imgQuestion?.setAlpha(0)
                    imgQuestion?.visibility = View.INVISIBLE
                    showQuestionImage()
                } else {

                    // image from URL
                    if (tmpImageURL.length > 1) {
                        val useImageName = BT_strings.getSaveAsFileNameFromURL(tmpImageURL)
                        if (BT_fileManager.doesCachedFileExist(useImageName)) {
                            BT_debugger.showIt("$fragmentName:showNextQuestion. Using cached image previously downloaded from: \"$tmpImageURL\"")
                            var d = BT_fileManager.getDrawableFromCache(useImageName)
                            if (quizImageCornerRadius > 0) {
                                var b = (d as BitmapDrawable).bitmap
                                b = BT_viewUtilities.getRoundedImage(b, quizImageCornerRadius)
                                d = BitmapDrawable(b)
                            }
                            imgQuestion?.setImageDrawable(d)
                            imgQuestion?.setAlpha(0)
                            imgQuestion?.visibility = View.INVISIBLE
                            showQuestionImage()
                        } else {
                            BT_debugger.showIt("$fragmentName:showNextQuestion. Downloading image from: \"$tmpImageURL\"")
                            val objDownloader = BT_downloader(tmpImageURL)
                            objDownloader.saveAsFileName = useImageName
                            var d = objDownloader.downloadDrawable()
                            if (quizImageCornerRadius > 0) {
                                var b = (d as BitmapDrawable).bitmap
                                b = BT_viewUtilities.getRoundedImage(b, quizImageCornerRadius)
                                d = BitmapDrawable(b)
                            }
                            imgQuestion?.setImageDrawable(d)
                            imgQuestion?.setAlpha(0)
                            imgQuestion?.visibility = View.INVISIBLE
                            showQuestionImage()
                        }
                    }
                }
            } else {
                BT_debugger.showIt(fragmentName + ":showNextQuestion. Question number " + (currentQuestion + 1) + " does not use an image")
            }

            // randomize array of answers on buttons
            val answers = listOf("incorrectText1", "incorrectText2", "incorrectText3", "correctAnswerText").map { key ->
                nextQuestion?.jsonObject.propertyValue(key)
            }.shuffled()

            // show on buttons
            btnAnswer_1?.text = answers[0]
            btnAnswer_2?.text = answers[1]
            btnAnswer_3?.text = answers[2]
            btnAnswer_4?.text = answers[3]
        }

        // flag allow next
        allowNext = 1
    }

    // end quiz
    fun endQuiz() {
        BT_debugger.showIt("$fragmentName:endQuiz")

        // set image to default...
        var tmpImageName = "bt_screen_quiz_bgsmall.png"
        if (buzztouch_appDelegate.rootApp.rootDevice.isLargeDevice) {
            tmpImageName = "bt_screen_quiz_bglarge.png"
        }
        var d = BT_fileManager.getDrawableByName(tmpImageName)
        if (quizImageCornerRadius > 0) {
            var b = (d as BitmapDrawable).bitmap
            b = BT_viewUtilities.getRoundedImage(b, quizImageCornerRadius)
            d = BitmapDrawable(b)
        }
        imgQuestion?.setImageDrawable(d)
        imgQuestion?.visibility = View.VISIBLE
        imgQuestion?.setAlpha(255)

        // flag
        quizIsComplete = 1

        // clear values
        currentQuestion = 0
        txtQuestion?.text = ""
        txtScore?.text = "Quiz Complete"

        // turn off timer
        updateTimeHandler?.removeCallbacks(mUpdateTimeTask)
        val millis = System.currentTimeMillis() - (startTime ?: 0L)
        var seconds = (millis / 1000).toInt()
        val minutes = seconds / 60
        seconds %= 60
        val tmpTime = when (seconds) {
            in 0..9 -> "$minutes:0$seconds"
            else -> "$minutes:seconds"
        }

        totalSeconds = seconds

        // send results to URL if provided...
        if (sendResultsToURL.length > 1) {
            sendQuizResultsToURL()
        }

        // show finished options, this method also plays possible sound effects...
        showFinishedOptions()

        // show alert
        val tmpResults = "Answers: $numberRight / ${childItems?.size ?: 0}\nPoints: $totalScore\nElapsed Time: $tmpTime\n"
        showAlert("Quiz Complete", tmpResults)
    }

    // shows finished options...
    fun showFinishedOptions() {
        BT_debugger.showIt("$fragmentName:showFinishedOptions")

        // NOTE: we may be arriving here on reward screen or finished screen "back" press...

        // set image to default...
        var tmpImageName = "bt_screen_quiz_bgsmall.png"
        if (buzztouch_appDelegate.rootApp.rootDevice.isLargeDevice) {
            tmpImageName = "bt_screen_quiz_bglarge.png"
        }
        var d = BT_fileManager.getDrawableByName(tmpImageName)
        if (quizImageCornerRadius > 0) {
            var b = (d as BitmapDrawable).bitmap
            b = BT_viewUtilities.getRoundedImage(b, quizImageCornerRadius)
            d = BitmapDrawable(b)
        }
        imgQuestion?.setImageDrawable(d)
        imgQuestion?.visibility = View.VISIBLE
        imgQuestion?.setAlpha(255)

        // hide controls
        txtTimer?.visibility = View.INVISIBLE
        txtScore?.visibility = View.INVISIBLE
        txtQuestion?.visibility = View.INVISIBLE
        btnPanel?.visibility = View.INVISIBLE

        // button 1 becomes try again...
        btnAnswer_1?.text = "Try Again"

        // button 2 becomes show reward or continue...
        // if we have a quizRewardScreenItemId AND a totalPoints >
        if (quizRewardScreenItemId.length > 0 && totalScore >= quizRewardIfPointsOver) {
            btnAnswer_2?.text = "Show Reward"

            // play possible reward sound...
            if (soundPlayerReward != null) {
                soundPlayerReward?.start()
            }
        } else {
            btnAnswer_2?.text = "Continue"

            // play possible finished sound...
            if (soundPlayerFinished != null) {
                soundPlayerFinished?.start()
            }
        }

        // button 3 becomes quit...
        btnAnswer_3?.text = "Quit"

        // button 4 is unused...
        btnAnswer_4?.visibility = View.INVISIBLE

        // slide in buttons..
        slideButtons()
    }

    // ///////////////////////////////////////////////////////////////////
    // handles timer updates for quiz "running time"

    var updateTimeHandler: Handler? = object : Handler() {
        override fun handleMessage(msg: Message) {

            // final long start = startTime;
            val millis = System.currentTimeMillis() - (startTime ?: 0L)
            var seconds = (millis / 1000).toInt()
            val minutes = seconds / 60
            seconds %= 60
            txtTimer?.text = if (quizShowTimer == "1" || quizShowTimer.equals("YES", ignoreCase = true)) {
                when (seconds) {
                    in 0..9 -> "$minutes:0$seconds"
                    else -> "$minutes:$seconds"
                }
            } else ""

            // re-trigger timer
            this.removeCallbacks(mUpdateTimeTask)
            this.postDelayed(mUpdateTimeTask, 100)
        }
    }
    private val mUpdateTimeTask = Runnable {
        val message = updateTimeHandler?.obtainMessage() ?: return@Runnable
        updateTimeHandler?.sendMessage(message)
    }

    // end timer stuff
    // ///////////////////////////////////////////////////////////////////
    // ///////////////////////////////////////////////////////////////////
    // handles question delay updates after each answer
    var delayHandler: Handler? = object : Handler() {
        override fun handleMessage(msg: Message) {
            this.removeCallbacks(mUpdateTimeTask)
        }
    }
    private val mDelayTask = Runnable {
        val message = delayHandler?.obtainMessage() ?: return@Runnable
        updateTimeHandler?.sendMessage(message)
        showNextQuestion()
    }

    // end timer stuff
    // ///////////////////////////////////////////////////////////////////
    // fade image in
    fun fadeImageIn(theImageView: ImageView?) {
        BT_debugger.showIt("$fragmentName:fadeImageIn")
        try {
            theImageView?.visibility = View.INVISIBLE
            val animation: Animation = AlphaAnimation(0.0f, 1.0f)
            animation.duration = 1200
            theImageView?.startAnimation(animation)
            theImageView?.visibility = View.VISIBLE
        } catch (ex: Exception) {
        }
    }

    // fade image out
    fun fadeImageOut(theImageView: ImageView?) {
        BT_debugger.showIt("$fragmentName:fadeImageOut")
        try {
            theImageView?.visibility = View.VISIBLE
            val animation: Animation = AlphaAnimation(1.0f, 0.0f)
            animation.duration = 1200
            theImageView?.startAnimation(animation)
            theImageView?.visibility = View.INVISIBLE
        } catch (ex: Exception) {
        }
    }

    // transitions question image..
    fun showQuestionImage() {
        BT_debugger.showIt("$fragmentName:showQuestionImage")
        try {
            imgQuestion?.setAlpha(255)
            val animation: Animation = AlphaAnimation(0.0f, 1.0f)
            animation.duration = 1200
            imgQuestion?.startAnimation(animation)
            imgQuestion?.visibility = View.VISIBLE
        } catch (ex: Exception) {
        }
    }

    // slide buttons
    fun slideButtons() {
        BT_debugger.showIt("$fragmentName:slideButtons")
        try {
            btnPanel?.visibility = View.VISIBLE
            val animation: Animation = TranslateAnimation(
                Animation.RELATIVE_TO_PARENT, -1.0f, Animation.RELATIVE_TO_PARENT, 0.0f,
                Animation.RELATIVE_TO_PARENT, 0.0f, Animation.RELATIVE_TO_PARENT, 0.0f
            )
            animation.duration = 500
            btnPanel?.startAnimation(animation)
        } catch (ex: Exception) {
        }
    }

    // slide question
    fun slideQuestion() {
        BT_debugger.showIt("$fragmentName:slideQuestion")
        try {
            txtQuestion?.visibility = View.VISIBLE
            val animation: Animation = TranslateAnimation(
                Animation.RELATIVE_TO_PARENT, +1.0f, Animation.RELATIVE_TO_PARENT, 0.0f,
                Animation.RELATIVE_TO_PARENT, 0.0f, Animation.RELATIVE_TO_PARENT, 0.0f
            )
            animation.duration = 500
            txtQuestion?.startAnimation(animation)
        } catch (ex: Exception) {
        }
    }

    // /////////////////////////////////////////////////////////////////
    // DownloadScreenDataThread and Handler
    var downloadScreenDataHandler: Handler = object : Handler() {
        override fun handleMessage(msg: Message) {
            if (JSONData.length < 1) {
                hideProgress()
                showAlert(getString(R.string.errorTitle), getString(R.string.errorDownloadingData))
            } else {
                parseScreenData(JSONData)
            }
        }
    }

    // END DownloadScreenDataThread and Handler
    // /////////////////////////////////////////////////////////////////
    // onCreateOptionsMenu...
    override fun onCreateOptionsMenu(menu: Menu, inflater: MenuInflater) {
        BT_debugger.showIt("$fragmentName:onCreateOptionsMenu JSON itemId: \"$screenItemId\"")

        /*
	 	 	Screens that extend BT_fragment should implement their own onCreateOptionsMenu() method to add
	 	 	action bar items specific to that screen. They should call super.onCreateOptionsMenu() AFTER
	 	 	adding customized items for that screen. 	  
	 	*/

        // for individual menu items...
        var menuItem: MenuItem? = null

        // quit quiz...
        menuItem = menu.add(Menu.NONE, 1, 0, getString(R.string.quit))
        menuItem.icon = BT_fileManager.getDrawableByName("bt_stop.png")
        menuItem.setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM)

        // restart quiz...
        menuItem = menu.add(Menu.NONE, 2, 0, getString(R.string.restart))
        menuItem.icon = BT_fileManager.getDrawableByName("bt_play.png")
        menuItem.setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM)

        // call super...
        super.onCreateOptionsMenu(menu, inflater)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        BT_debugger.showIt(fragmentName + ":onOptionsItemSelected JSON itemId: \"" + screenItemId + "\" Selected Item's Id: " + item.itemId)
        when (item.itemId) {
            1 -> {
                // quit quiz...

                // remove fragment from back stack...
                activity?.supportFragmentManager?.let {
                    if (it.backStackEntryCount > 0) it.popBackStackImmediate()
                }
                activity?.fragmentManager?.let {
                    if (it.backStackEntryCount > 0) it.popBackStackImmediate()
                }

                return true
            }
            2 -> {
                // restart quiz...
                startCountdown()
                return true
            }
        }

        // return...
        return super.onOptionsItemSelected(item)
    }
}
