/*
 *	Copyright 2011, David Book, buzztouch.com
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are 
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the 
 *	following disclaimer. 
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list 
 *	of conditions and the following disclaimer in the documentation and/or other materials 
 *	provided with the distribution. 
 *
 *	Neither the name of David Book, or buzztouch.com nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without specific 
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *	OF SUCH DAMAGE. 
 */
package com.buzztouch.ui.bt_screens

import android.app.Activity
import android.app.AlertDialog
import android.content.DialogInterface
import android.graphics.drawable.ColorDrawable
import android.os.Bundle
import android.os.Handler
import android.view.LayoutInflater
import android.view.Menu
import android.view.MenuInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import android.widget.ArrayAdapter
import android.widget.CheckBox
import android.widget.ListView
import android.widget.TextView
import com.buzztouch.buzztouch_appDelegate
import com.buzztouch.R
import com.buzztouch.core.BT_configuration
import com.buzztouch.utils.*

class BT_screen_settingsDevice : BT_fragment() {
    private var didLoadData = false
    var didCreate = false
    var JSONData = ""
    var childItemAdapter: ChildItemAdapter? = null
    private var myListView: ListView? = null
    var confirmClearCacheAlert: AlertDialog? = null

    // layout properties (probably not used on settings screens?)...
    var listStyle = ""
    private var preventAllScrolling = ""
    private var listBackgroundColor = ""
    private var listRowBackgroundColor = ""
    var listTitleFontColor = ""
    private var listRowSeparatorColor = ""

    // these depend on the device size..
    var listRowHeight = 0
    var listTitleHeight = 0
    var listTitleFontSize = 0

    // onCreateView...
    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {

        /*
 			Note: fragmentName property is already setup in the parent class (BT_fragment). This allows us 
 			to add the 	name of this class file to the LogCat console using the BT_debugger.
		*/
        // show life-cycle event in LogCat console...
        BT_debugger.showIt(fragmentName + ":onCreateView JSON itemId: \"" + screenData.itemId + "\" itemType: \"" + screenData.itemType + "\" itemNickname: \"" + screenData.itemNickname + "\"")

        // inflate the layout file for this screen...
        val thisScreensView = inflater.inflate(R.layout.bt_screen_settingsdevice, container, false)

        // /////////////////////////////////////////////////////////////////
        // settings for any sized device...
        listStyle = BT_strings.getStyleValueForScreen(screenData, "listStyle", "plain")
        preventAllScrolling = BT_strings.getStyleValueForScreen(screenData, "preventAllScrolling", "0")
        listBackgroundColor = BT_strings.getStyleValueForScreen(screenData, "listBackgroundColor", "clear")
        listRowBackgroundColor = BT_strings.getStyleValueForScreen(screenData, "listRowBackgroundColor", "clear")
        listTitleFontColor = BT_strings.getStyleValueForScreen(screenData, "listTitleFontColor", "#999999")
        listRowSeparatorColor = BT_strings.getStyleValueForScreen(screenData, "listRowSeparatorColor", "#999999")

        // settings based on device size...
        if (buzztouch_appDelegate.rootApp.rootDevice.isLargeDevice) {

            // large devices...
            listRowHeight = BT_strings.getStyleValueForScreen(screenData, "listRowHeightLargeDevice", "45").toInt()
            listTitleHeight = BT_strings.getStyleValueForScreen(screenData, "listTitleHeightLargeDevice", "25").toInt()
            listTitleFontSize = BT_strings.getStyleValueForScreen(screenData, "listTitleFontSizeLargeDevice", "20").toInt()
        } else {

            // small devices...
            listRowHeight = BT_strings.getStyleValueForScreen(screenData, "listRowHeightSmallDevice", "45").toInt()
            listTitleHeight = BT_strings.getStyleValueForScreen(screenData, "listTitleHeightSmallDevice", "25").toInt()
            listTitleFontSize = BT_strings.getStyleValueForScreen(screenData, "listTitleFontSizeSmallDevice", "20").toInt()
        }

        // reference to list..
        myListView = thisScreensView.findViewById<View>(R.id.listView) as ListView
        myListView?.isVerticalScrollBarEnabled = false
        myListView?.isHorizontalScrollBarEnabled = false

        // prevent scrolling...
        if (preventAllScrolling.equals("1", ignoreCase = true)) {
            // Need to figure out how to prevent scrolling?
        }

        // list background color...
        if (listBackgroundColor.length > 0) {
            myListView?.setBackgroundColor(BT_colorKt.getColorFromHexString(listBackgroundColor))
        }

        // row background color...
        if (listRowBackgroundColor.length > 0) {
            // not useful, use listBackgroundColor instead.
        }

        // divider / separator color...
        if (listRowSeparatorColor.length > 0) {
            val dividerColor = ColorDrawable(BT_colorKt.getColorFromHexString(listRowSeparatorColor))
            myListView?.divider = dividerColor
            myListView?.dividerHeight = 1
        }

        // flag as created..
        didCreate = true

        // return...
        return thisScreensView
    } // onCreateView...

    // onStart
    override fun onStart() {
        // BT_debugger.showIt(fragmentName + ":onStart");
        super.onStart()

        // ignore this if we already created the screen...
        if (!didLoadData) {
            parseScreenData()
        }
    }

    // parse screenData...
    fun parseScreenData() {
        BT_debugger.showIt("$fragmentName:parseScreenData")

        // invalidate list...
        myListView?.invalidate()

        // setup dataModel for listView
        val activity = activity ?: return
        val adapter: ArrayAdapter<ChildItemsDataModel> = ChildItemAdapter(activity, model)
        myListView?.adapter = adapter
    }

    // /////////////////////////////////////////////////////////////////
    // childItems Data Model for selectable list items..
    inner class ChildItemsDataModel(var name: String) {
        var isSelected = false
    }

    // flag, return...
    // END Data Model for selectable items
    // /////////////////////////////////////////////////////////////////
    // fill data model with childItems..
    private val model: List<ChildItemsDataModel> = listOf(
        // device model:
        getChildItem("${getString(R.string.deviceModel)}: ${buzztouch_appDelegate.rootApp.rootDevice.deviceModel}"),
        // device screen size:
        getChildItem("${getString(R.string.deviceResolution)}: ${buzztouch_appDelegate.rootApp.rootDevice.deviceWidth} x ${buzztouch_appDelegate.rootApp.rootDevice.deviceHeight}"),
        // device cache size:
        getChildItem("${getString(R.string.deviceCacheSize)}: ${BT_strings.formatBytes(BT_fileManager.getCachedDataSize().toLong(), true)}"),
        // device can make phone calls:
        (getChildItem(getString(R.string.deviceCanMakePhoneCalls))).also {
            it.isSelected = buzztouch_appDelegate.rootApp.rootDevice.canMakeCalls()
        },
        // device can take pictures:
        (getChildItem(getString(R.string.deviceCanTakePictures))).also {
            it.isSelected = buzztouch_appDelegate.rootApp.rootDevice.canTakePictures()
        },
        // device can send email:
        (getChildItem(getString(R.string.deviceCanSendEmail))).also {
            it.isSelected = buzztouch_appDelegate.rootApp.rootDevice.canSendEmail()
        },
        // device can send SMS:
        (getChildItem(getString(R.string.deviceCanSendSMS))).also {
            it.isSelected = buzztouch_appDelegate.rootApp.rootDevice.canSendSMS()
        },
        // device GPS capable:
        (getChildItem(getString(R.string.deviceGPSCapable))).also {
            it.isSelected = buzztouch_appDelegate.rootApp.rootDevice.canUseGPS()
        },
        // add last empty item so we have a border below last row (tricky!)...
        getChildItem("")
    ).also { didLoadData = true }

    private fun getChildItem(s: String): ChildItemsDataModel {
        return ChildItemsDataModel(s)
    }

    // /////////////////////////////////////////////////////////////////
    // Adapter for Child Items
    inner class ChildItemAdapter(
        private val context: Activity,
        private val list: List<ChildItemsDataModel>
    ) : ArrayAdapter<ChildItemsDataModel>(context, R.layout.bt_screen_settingsdevice_checkbox, list) {
        private inner class ViewHolder {
            var text: TextView? = null
            var checkbox: CheckBox? = null
        }

        override fun getView(position: Int, convertView: View?, parent: ViewGroup): View {
            var view: View? = null
            if (convertView == null) {
                val inflator = context.layoutInflater
                view = inflator.inflate(R.layout.bt_screen_settingsdevice_checkbox, null)
                val viewHolder: ViewHolder = ViewHolder()
                viewHolder.text = view.findViewById<View>(R.id.titleView) as TextView
                viewHolder.checkbox = view.findViewById<View>(R.id.checkboxView) as CheckBox
                viewHolder.checkbox?.setOnClickListener { // delay refresh...prevents user from changing checkboxes..
                    val mHandler = Handler()
                    mHandler.postDelayed(mLaunchTask, 200)
                }
                // do nothing on item change...
                viewHolder.checkbox?.setOnCheckedChangeListener { buttonView, isChecked -> }
                view.tag = viewHolder
                viewHolder.checkbox?.tag = list[position]
            } else {
                view = convertView
                (view.tag as ViewHolder).checkbox?.tag = list[position]
            }
            val holder = view?.tag as ViewHolder
            holder.text?.text = list[position].name
            holder.checkbox?.isChecked = list[position].isSelected

            // first three items have no checkbox...
            if (position < 3) {
                holder.checkbox?.visibility = View.GONE
            } else {
                holder.checkbox?.visibility = View.VISIBLE
            }

            // hide the last checkbox (it's the last empty row)
            if (list[position].name.length < 1) {
                holder.checkbox?.visibility = View.GONE
            }

            // return...
            return view
        }
    }

    // END child items adapter...
    // /////////////////////////////////////////////////////////////////
    private val mLaunchTask = Runnable {
        try {
            parseScreenData()
        } catch (e: Exception) {
        }
    }

    // confirm confirmClearCache...
    fun confirmClearCache() {
        confirmClearCacheAlert = AlertDialog.Builder(this.activity).create()
        confirmClearCacheAlert?.apply {
            setTitle(getString(R.string.confirm))
            setMessage(getString(R.string.confirmClearCacheDescription))
            setIcon(R.mipmap.ic_launcher)
            setButton(DialogInterface.BUTTON_POSITIVE, getString(R.string.yes)) { _, _ -> // refresh data...
                dismiss()
                clearCache()
            }
            setButton(DialogInterface.BUTTON_NEGATIVE, getString(R.string.no)) { _, _ ->
                dismiss()
            }
        }
        confirmClearCacheAlert?.show()
    }

    // clear cache...
    fun clearCache() {
        BT_debugger.showIt("$fragmentName:clearCache")

        // remove cached data....
        BT_fileManager.deleteAllCachedData(BT_configuration.CACHED_CONFIGURATION_NAME)

        // delay refresh...
        val mHandler = Handler()
        mHandler.postDelayed(mLaunchTask, 500)

        // show alert
        showAlert(getString(R.string.ok), getString(R.string.confirmClearCacheComplete))
    }

    // onCreateOptionsMenu...
    override fun onCreateOptionsMenu(menu: Menu, inflater: MenuInflater) {
        BT_debugger.showIt("$fragmentName:onCreateOptionsMenu JSON itemId: \"$screenItemId\"")

        /*
	 	 	Screens that extend BT_fragment should implement their own onCreateOptionsMenu() method to add
	 	 	action bar items specific to that screen. They should call super.onCreateOptionsMenu() AFTER
	 	 	adding customized items for that screen. 	  
	 	*/

        // for individual menu items...
        var menuItem: MenuItem? = null

        // clearCache...
        menuItem = menu.add(Menu.NONE, 1, 0, getString(R.string.clearCache))
        menuItem.icon = BT_fileManager.getDrawableByName("bt_action.png")
        menuItem.setShowAsAction(MenuItem.SHOW_AS_ACTION_IF_ROOM)

        // call super...
        super.onCreateOptionsMenu(menu, inflater)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        BT_debugger.showIt(fragmentName + ":onOptionsItemSelected JSON itemId: \"" + screenItemId + "\" Selected Item's Id: " + item.itemId)
        when (item.itemId) {
            1 -> {
                // confirmClearCache...
                confirmClearCache()
                return true
            }
        }

        // return...
        return super.onOptionsItemSelected(item)
    }
}
