/*
 *	Copyright 2011, David Book, buzztouch.com
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are 
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the 
 *	following disclaimer. 
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list 
 *	of conditions and the following disclaimer in the documentation and/or other materials 
 *	provided with the distribution. 
 *
 *	Neither the name of David Book, or buzztouch.com nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without specific 
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *	OF SUCH DAMAGE. 
 */
package com.buzztouch.ui.bt_screens

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.EditText
import android.widget.ImageView
import android.widget.TextView
import com.buzztouch.buzztouch_appDelegate
import com.buzztouch.R
import com.buzztouch.extensions.downloadScreenData
import com.buzztouch.utils.*
import kotlinx.coroutines.launch
import org.json.JSONObject

class BT_screen_settingsLogIn : BT_fragment() {
    var didCreate = false
    private var JSONData = ""
    private var dataURL = ""
    private var logInIdLabel: TextView? = null
    private var logInIdText: EditText? = null
    private var logInPasswordLabel: TextView? = null
    private var logInPasswordText: EditText? = null
    private var submitButton: Button? = null
    private var iconStatusView: ImageView? = null
    private var textOnBackgroundColor = ""
    private var labelLogInId = ""
    private var labelPassword = ""

    // results from server...
    var userGuid = ""
    var userDisplayName = ""
    var userEmail = ""

    // onCreateView...
    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {

        /*
 			Note: fragmentName property is already setup in the parent class (BT_fragment). This allows us 
 			to add the 	name of this class file to the LogCat console using the BT_debugger.
		*/
        // show life-cycle event in LogCat console...
        BT_debugger.showIt(fragmentName + ":onCreateView JSON itemId: \"" + screenData.itemId + "\" itemType: \"" + screenData.itemType + "\" itemNickname: \"" + screenData.itemNickname + "\"")

        // inflate the layout file for this screen...
        val thisScreensView = inflater.inflate(R.layout.bt_screen_settingslogin, container, false)

        // /////////////////////////////////////////////////////////////////
        // values from JSON in screen data...
        dataURL = BT_strings.getJsonPropertyValue(screenData.jsonObject, "dataURL", "")
        textOnBackgroundColor = BT_strings.getStyleValueForScreen(screenData, "textOnBackgroundColor", "#707070")
        labelLogInId = BT_strings.getJsonPropertyValue(screenData.jsonObject, "labelLogInId", getString(R.string.logInIdLabel))
        labelPassword = BT_strings.getJsonPropertyValue(screenData.jsonObject, "labelPassword", getString(R.string.logInPasswordLabel))

        // logInIdLabel
        logInIdLabel = thisScreensView.findViewById<View>(R.id.logInIdLabel) as TextView
        logInIdLabel?.setTextColor(BT_colorKt.getColorFromHexString(textOnBackgroundColor))
        logInIdLabel?.text = labelLogInId

        // logInIdText
        logInIdText = thisScreensView.findViewById<View>(R.id.logInIdText) as EditText
        logInIdText?.setText(BT_preferencesKt.getString("userLogInId"))

        // logInPasswordLabel
        logInPasswordLabel = thisScreensView.findViewById<View>(R.id.logInPasswordLabel) as TextView
        logInPasswordLabel?.setTextColor(BT_colorKt.getColorFromHexString(textOnBackgroundColor))
        logInPasswordLabel?.text = labelPassword

        // logInPasswordText
        logInPasswordText = thisScreensView.findViewById<View>(R.id.logInPasswordText) as EditText

        // statusImage
        iconStatusView = thisScreensView.findViewById<View>(R.id.iconStatusView) as ImageView

        // submitButton
        submitButton = thisScreensView.findViewById<View>(R.id.submitButton) as Button
        if (buzztouch_appDelegate.rootApp.rootUser.isLoggedIn == true) {
            submitButton?.text = getString(R.string.logOut)
            iconStatusView?.setImageResource(R.drawable.bt_screen_settingslogin_dot_green)
        } else {
            submitButton?.text = getString(R.string.logInSubmit)
            iconStatusView?.setImageDrawable(BT_fileManager.getDrawableByName("bt_blank.png"))
        }

        // invalidate icon...
        iconStatusView?.invalidate()

        // submit click listener..
        submitButton?.setOnClickListener {
            if (buzztouch_appDelegate.rootApp.rootUser.isLoggedIn) {
                logOut()
            } else {
                logIn()
            }
        }

        // toast...icon...
        if (buzztouch_appDelegate.rootApp.rootUser.isLoggedIn == true) {
            showToast(getString(R.string.loggedInYes), "long")
            iconStatusView?.setImageResource(R.drawable.bt_screen_settingslogin_dot_green)
        } else {
            showToast(getString(R.string.loggedInNo), "long")
            iconStatusView?.setImageResource(R.drawable.bt_screen_settingslogin_dot_red)
        }

        // flag as created..
        didCreate = true

        // return...
        return thisScreensView
    } // onCreateView...

    // end activity life-cycle events
    // ////////////////////////////////////////////////////////////////////////
    // login
    fun logIn() {
        BT_debugger.showIt("$fragmentName:logIn")
        if (logInIdText?.text.isNullOrEmpty() || logInPasswordText?.text.isNullOrEmpty()) {
            showAlert(getString(R.string.errorTitle), getString(R.string.logInIdAndPasswordRequired))
        } else {

            // remember some values...
            BT_preferencesKt.setString("userLogInId", logInIdText?.text.toString())
            BT_preferencesKt.setString("userLogInPassword", logInPasswordText?.text.toString())
            buzztouch_appDelegate.rootApp.rootUser.userLogInId = logInIdText?.text.toString()
            buzztouch_appDelegate.rootApp.rootUser.userLogInPassword = logInPasswordText?.text.toString()

            if (dataURL.length > 1) {
                launch {
                    JSONData = downloadScreenData(dataURL)
                    parseScreenData(JSONData)
                }
            } else {
                showAlert(getString(R.string.errorTitle), getString(R.string.errorNoURL))
                BT_debugger.showIt("$fragmentName:refreshScreenData NO DATA URL for this screen? Not downloading.")
            }
        }
    }

    // logout
    fun logOut() {
        BT_debugger.showIt("$fragmentName:logOut")

        // forget prefs
        BT_preferencesKt.setString("userId", "")
        BT_preferencesKt.setString("userGuid", "")
        BT_preferencesKt.setString("userDisplayName", "")
        BT_preferencesKt.setString("userEmail", "")
        BT_preferencesKt.setString("userLogInId", "")
        BT_preferencesKt.setString("userLogInPassword", "")

        // forget values in rootUser (logout)
        buzztouch_appDelegate.rootApp.rootUser.clearData()

        // clear text boxes...
        logInPasswordText?.setText("")

        // show alert
        showAlert(getString(R.string.logOutSuccessTitle), getString(R.string.logOutSuccessDescription))

        // update label, icon..
        submitButton?.text = getString(R.string.logInSubmit)
        iconStatusView?.setImageResource(R.drawable.bt_screen_settingslogin_dot_red)
    }

    // parse results...
    fun parseScreenData(JSONString: String) {
        BT_debugger.showIt("$fragmentName:parseScreenData $JSONString")
        hideProgress()

        // values from server...
        var validLogIn = false
        try {

            // if theJSONString is empty, look for child items in this screen's config data..
            if (JSONString.length < 10) {
                validLogIn = false
            } else {
                val raw = JSONObject(JSONString)
                if (raw.has("result")) {
                    val result = raw.getJSONObject("result")
                    if (result.has("status")) {
                        if (result.getString("status").equals("valid", ignoreCase = true)) {

                            // flag
                            validLogIn = true
                            if (result.has("userGuid")) userGuid = result.getString("userGuid")
                            if (result.has("userDisplayName")) userDisplayName = result.getString("userDisplayName")
                            if (result.has("userEmail")) userEmail = result.getString("userEmail")
                        }
                    }
                }
            }

            // valid or invalid?
            if (validLogIn) {

                // success

                // remember prefs
                BT_preferencesKt.setString("userId", userGuid)
                BT_preferencesKt.setString("userGuid", userGuid)
                BT_preferencesKt.setString("userDisplayName", userDisplayName)
                BT_preferencesKt.setString("userEmail", userEmail)
                BT_preferencesKt.setString("userLogInId", logInIdText?.text.toString())
                BT_preferencesKt.setString("userLogInPassword", logInPasswordText?.text.toString())

                // remember values in rootUser (logout)
                buzztouch_appDelegate.rootApp.rootUser.isLoggedIn = true
                buzztouch_appDelegate.rootApp.rootUser.userId = userGuid
                buzztouch_appDelegate.rootApp.rootUser.userDisplayName = userDisplayName
                buzztouch_appDelegate.rootApp.rootUser.userEmail = userEmail
                buzztouch_appDelegate.rootApp.rootUser.userLogInId = logInIdText?.text.toString()
                buzztouch_appDelegate.rootApp.rootUser.userLogInPassword = logInPasswordText?.text.toString()

                // toast...
                if (userDisplayName.length > 1) {
                    showToast(getString(R.string.welcome), "long")
                }

                // clear password box
                logInPasswordText?.setText("")

                // show alert
                showAlert(getString(R.string.logInSuccessTitle), getString(R.string.logInSuccessDescription))

                // set button, image...
                submitButton?.text = getString(R.string.logOut)
                iconStatusView?.setImageResource(R.drawable.bt_screen_settingslogin_dot_green)
            } else {

                // failed

                // forget prefs
                BT_preferences.setPrefString("userId", "")
                BT_preferences.setPrefString("userGuid", "")
                BT_preferences.setPrefString("userDisplayName", "")
                BT_preferences.setPrefString("userEmail", "")
                BT_preferences.setPrefString("userLogInId", "")
                BT_preferences.setPrefString("userLogInPassword", "")

                // forget values in rootUser (logout)
                buzztouch_appDelegate.rootApp.rootUser.clearData()

                // show alert...
                showAlert(getString(R.string.logInFailedTitle), getString(R.string.logInFailedDescription))

                // set button, image...
                submitButton?.text = getString(R.string.logOut)
                iconStatusView?.setImageResource(R.drawable.bt_screen_settingslogin_dot_red)
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }
}