package com.buzztouch.utils

import kotlin.coroutines.Continuation
import kotlin.coroutines.resume
import kotlin.coroutines.suspendCoroutine

typealias SafeCoroutineTask<T> = (continuation: BT_SafeSuspendedCoroutine.SafeContinuation<T>) -> Unit
class BT_SafeSuspendedCoroutine<T> {
    var cont: Continuation<T>? = null
    private lateinit var safeContinuation: SafeContinuation<T>
    private val callback: (res: T) -> Unit = {
        cont?.resume(it)
        cont = null
    }

    suspend fun runTask(task: SafeCoroutineTask<T>) = suspendCoroutine<T> { continuation ->
        cont = continuation
        safeContinuation = SafeContinuation(callback)
        task(safeContinuation)
    }

    class SafeContinuation<in T>(val callback: (res: T) -> Unit) {
        fun resume(result: T) {
            callback(result)
        }
    }
}

suspend fun <T> safeSuspendCoroutine(task: SafeCoroutineTask<T>): T = BT_SafeSuspendedCoroutine<T>().runTask(task)
