package com.buzztouch.utils

import android.app.AlertDialog
import android.content.Context
import android.content.DialogInterface
import com.buzztouch.R

object BT_alertUtils {

    suspend fun showConfirm(context: Context, title: String, message: String, icon: Int): Int = safeSuspendCoroutine { cont ->
        val confirmLaunchInNativeAppDialogue = AlertDialog.Builder(context).create()
        confirmLaunchInNativeAppDialogue.setTitle(title)
        confirmLaunchInNativeAppDialogue.setMessage(message)
        confirmLaunchInNativeAppDialogue.setIcon(icon)

        // YES
        confirmLaunchInNativeAppDialogue.setButton(DialogInterface.BUTTON_POSITIVE, context.getString(R.string.yes)) { _, _ ->
            confirmLaunchInNativeAppDialogue.dismiss()
            cont.resume(DialogInterface.BUTTON_POSITIVE)
        }

        // NO
        confirmLaunchInNativeAppDialogue.setButton(DialogInterface.BUTTON_NEGATIVE, context.getString(R.string.no)) { _, _ ->
            confirmLaunchInNativeAppDialogue.dismiss()
            cont.resume(DialogInterface.BUTTON_NEGATIVE)
        }

        // show the confirmation box...
        confirmLaunchInNativeAppDialogue.show()
    }

    fun showWithStyle(context: Context, style: (alert: AlertDialog) -> Unit) {
        val myAlert = AlertDialog.Builder(context).create()
        style(myAlert)
        myAlert.show()
    }

    suspend fun showAlertAndWait(context: Context, title: String, message: String): Unit = safeSuspendCoroutine{ cont ->
        showWithStyle(context) {
            it.setTitle(title)
            it.setMessage(message)
            it.setOnDismissListener { cont.resume(Unit) }
        }
    }
}
