/*  File Version: 4.0.3
 *	Copyright, David Book, buzztouch.com
 *  Updates made by Chris Ruddell, 1/31/2017
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are 
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the 
 *	following disclaimer. 
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list 
 *	of conditions and the following disclaimer in the documentation and/or other materials 
 *	provided with the distribution. 
 *
 *	Neither the name of David Book, or buzztouch.com nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without specific 
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *	OF SUCH DAMAGE. 
 */
package com.buzztouch.utils;
import java.io.BufferedReader;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;


import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;

import com.buzztouch.buzztouch_appDelegate;

public class BT_downloader {
	
	public static String objectName = "BT_downloader";
	private int connectionTimeoutSeconds = 30;
	private boolean downloadInProgress;
	private String downloadURL;
	private String saveAsFileName;
	
	// constructor
	public BT_downloader(String downloadURL){
		this.downloadURL = downloadURL;
		this.saveAsFileName = "";
	}
	
	
	// download image from URL, return as drawable...
    public Drawable downloadDrawable(){  
		BT_debugger.showIt(objectName + ":downloadDrawable from URL: " + this.downloadURL);
    	setDownloadInProgress(true);
        InputStream in = null;  
    	Drawable d = null;
        int response = -1;
        try{
        	
	        URL url = new URL(this.downloadURL); 
	        URLConnection conn = url.openConnection();
	                 
	        if (!(conn instanceof HttpURLConnection))                     
	            throw new IOException("Not an HTTP connection");
	        try{
	            HttpURLConnection httpConn = (HttpURLConnection) conn;
	            httpConn.setAllowUserInteraction(false);
	            httpConn.setInstanceFollowRedirects(true);
	            httpConn.setConnectTimeout(connectionTimeoutSeconds * 1000);
	            httpConn.setRequestMethod("GET");
	            httpConn.connect(); 
	            response = httpConn.getResponseCode();                 
	            if(response == HttpURLConnection.HTTP_OK) {
	                in = httpConn.getInputStream();  
	            }else{
		    		BT_debugger.showIt(objectName + ":downloadImage HTTP response NOT OK, URL: " + url);
	            }
	            
	            // BitmpFactory chokes while fetching giant images (not enough memory)...
	            Bitmap b = null;
	            try{
	            	b = BitmapFactory.decodeStream(in);
	            	d = new BitmapDrawable(buzztouch_appDelegate.getApplication().getResources(), b);
	            }catch(java.lang.OutOfMemoryError err){
	        		BT_debugger.showIt(objectName + ":downloadImage EXCEPTION downlaoding image: " + err.toString());
	            }
	            
	            // save to cache if we have a filename...
	            if(this.saveAsFileName.length() > 1 && b != null){
	            	BT_fileManager.saveImageToCache(b, this.saveAsFileName);
	            }
	            
	            
	        }catch (Exception ex){
	    		BT_debugger.showIt(objectName + ":downloadDrawable Exception in httpConnection: " + ex.toString() + " URL: " + url);
	            throw new IOException("Error connecting");            
	        }
	        
       
        }catch (Exception e) {
    		d = null;
        	BT_debugger.showIt(objectName + ":downloadImage from URL error: " + this.downloadURL);
    		BT_debugger.showIt(objectName + ":downloadImage Exception: " + e.toString());
        }
        setDownloadInProgress(false);
        return d;                
    } 
    
	// download image from URL, return as bitmap...
    public Bitmap downloadBitmap(){  
		BT_debugger.showIt(objectName + ":downloadBitmap from URL: " + this.downloadURL);
    	setDownloadInProgress(true);
        InputStream in = null;  
    	Bitmap b = null;
        int response = -1;
        try{
        	
	        URL url = new URL(this.downloadURL); 
	        URLConnection conn = url.openConnection();
	                 
	        if (!(conn instanceof HttpURLConnection))                     
	            throw new IOException("Not an HTTP connection");
	        try{
	            HttpURLConnection httpConn = (HttpURLConnection) conn;
	            httpConn.setAllowUserInteraction(false);
	            httpConn.setInstanceFollowRedirects(true);
	            httpConn.setConnectTimeout(connectionTimeoutSeconds * 1000);
	            httpConn.setRequestMethod("GET");
	            httpConn.connect(); 
	            response = httpConn.getResponseCode();                 
	            if(response == HttpURLConnection.HTTP_OK) {
	                in = httpConn.getInputStream();  
	            }else{
		    		BT_debugger.showIt(objectName + ":downloadImage HTTP response NOT OK, URL: " + url);
	            }
	            b = BitmapFactory.decodeStream(in);
	            
	            // save to cache if we have a filename...
	            if(this.saveAsFileName.length() > 1 && b != null){
	            	BT_fileManager.saveImageToCache(b, this.saveAsFileName);
	            }
	            
	            
	        }catch (Exception ex){
	    		BT_debugger.showIt(objectName + ":downloadBitmap Exception in httpConnection: " + ex.toString());
	            throw new IOException("Error connecting");            
	        }
	        
       
        }catch (Exception e) {
    		b = null;
        	BT_debugger.showIt(objectName + ":downloadBitmap from URL error: " + this.downloadURL);
    		BT_debugger.showIt(objectName + ":downloadBitmap Exception: " + e.toString());
        }
        setDownloadInProgress(false);
        return b;                
    } 


    
    // download text data from URL...
    public String downloadTextData(){  
		BT_debugger.showIt(objectName + ":downloadTextData from URL: " + this.downloadURL);
		setDownloadInProgress(true);
		String ret = "";
        
		try{
        	
			// create a URL for the desired data...
            URL url = new URL(this.downloadURL);

            //read all the text returned by the server
            BufferedReader in = new BufferedReader(new InputStreamReader(url.openStream()), 8192);
		    
		    StringBuilder total = new StringBuilder();
            String str;
            while ((str = in.readLine()) != null) {
                total.append(str);
            }
            in.close(); 

		    //return results...
		    ret = total.toString();
		    
            // save to cache if we have a filename...
            if(this.saveAsFileName.length() > 1 && ret.length() > 1){
            	BT_fileManager.saveTextFileToCache(ret, this.saveAsFileName);
            } 		    
		    
            
        }catch (Exception e) {
    		ret = "";
        	BT_debugger.showIt(objectName + ":downloadTextData from URL EXCEPTION: " + e.toString() + " URL: " + this.downloadURL);
        }
          
        // flag, return...
        setDownloadInProgress(false);
        return ret;                
    }    
    
    // download binary data from URL...

    /**
     * Downloads and saves a binary file
     * @deprecated use downloadAndSaveBinaryData(NetworkListener listener) instead
     */
	@Deprecated	// Deprecated -- should implement a thread safe method for downloading files!
    public String downloadAndSaveBinaryData(){  
		BT_debugger.showIt(objectName + ":downloadBinaryData from URL: " + this.downloadURL);
		return "deprecated_method";
    }

    /**
     * Downloads and saves a binary file
     * see {@link com.buzztouch.ui.bt_screens.BT_screen_htmlDoc.DownloadScreenDataWorker} for example
     * @param listener -- callback listener
     */
    public void downloadAndSaveBinaryData(NetworkListener listener) {
		setDownloadInProgress(true);

		InputStream in = null;
		// save file location after it's done donwloading...

		int response = -1;
		String message = "";
		try{
			FileOutputStream f = buzztouch_appDelegate.getApplication().openFileOutput(saveAsFileName, Context.MODE_PRIVATE);
			URL url = new URL(this.downloadURL);
			URLConnection conn = url.openConnection();

			if (!(conn instanceof HttpURLConnection))
				throw new IOException("Not an HTTP connection");
			try{
				HttpURLConnection httpConn = (HttpURLConnection) conn;
				httpConn.setAllowUserInteraction(false);
				httpConn.setInstanceFollowRedirects(true);
				httpConn.setConnectTimeout(connectionTimeoutSeconds * 1000);
				httpConn.setRequestMethod("GET");
				httpConn.connect();
				response = httpConn.getResponseCode();
				if(response == HttpURLConnection.HTTP_OK) {
					in = httpConn.getInputStream();

					byte[] buffer = new byte[1024];
					int len1 = 0;
					while ( (len1 = in.read(buffer)) > 0 ) {
						f.write(buffer,0, len1);
					}
					f.flush();
					f.close();

					setDownloadInProgress(false);
					listener.onNetworkReqeuestFinished(response, "");
				}else{
					BT_debugger.showIt(objectName + ":downloadImage HTTP response NOT OK, URL: " + url);
					in = httpConn.getErrorStream();
					//read all the text returned by the server
					BufferedReader errorIn = new BufferedReader(new InputStreamReader(in));

					StringBuilder total = new StringBuilder();
					String str;
					while ((str = errorIn.readLine()) != null) {
						total.append(str);
					}
					errorIn.close();

					//return results...
					message = total.toString();
				}


			}catch (Exception ex){
				BT_debugger.showIt(objectName + ":downloadBitmap Exception in httpConnection: " + ex.toString());
				throw new IOException("Error connecting");
			}


		}catch (Exception e) {
			BT_debugger.showIt(objectName + ":downloadBitmap from URL error: " + this.downloadURL);
			BT_debugger.showIt(objectName + ":downloadBitmap Exception: " + e.toString());
		}

		listener.onNetworkReqeuestFinished(response, message);
	}
    
    
    // getters / setters
    public void setDownloadInProgress(boolean downloadInProgress){
    	this.downloadInProgress = downloadInProgress;
    }
    public boolean getDownloadInProgress(){
    	return downloadInProgress;
    }
    public void setDownloadURL(String downloadURL){
    	this.downloadURL = downloadURL;
    }
    public String getDownloadURL(){
    	return downloadURL;
    }
    public void setSaveAsFileName(String saveAsFileName){
    	this.saveAsFileName = saveAsFileName;
    }
    public String getSaveAsFileName(){
    	return saveAsFileName;
    } 

	public interface NetworkListener {
		void onNetworkReqeuestFinished(int statusCode, String body);
	}



}





