/*  File Version: 3.0
 *	Copyright, David Book, buzztouch.com
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are 
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the 
 *	following disclaimer. 
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list 
 *	of conditions and the following disclaimer in the documentation and/or other materials 
 *	provided with the distribution. 
 *
 *	Neither the name of David Book, or buzztouch.com nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without specific 
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *	OF SUCH DAMAGE. 
 */
package com.buzztouch.utils;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.StringWriter;
import java.io.Writer;
import java.net.URL;
import java.util.ArrayList;

import android.content.Context;
import android.content.res.AssetManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Bitmap.CompressFormat;
import android.graphics.drawable.Drawable;

import com.buzztouch.buzztouch_appDelegate;
import com.google.common.base.Charsets;

public class BT_fileManager {

	
	public static String objectName = "BT_fileManager";
	
	// not all documents are cachable...
	public static boolean canCacheDocumentType(String theFileName){
		return BT_fileManagerKt.INSTANCE.canCacheDocumentType(theFileName);
	}	
	
	// does file exist in projects assets directory? assetsDirectory will be "BT_Images" or "BT_Audio", etc...
	public static boolean doesProjectAssetExist(String assetsDirectory, String fileName){
		return BT_fileManagerKt.INSTANCE.doesProjectAssetExist(assetsDirectory, fileName);
	}
	

	// copies a file from assets to the app's cache..
	public static boolean copyAssetToCache(String assetsDirectory, String fileName){
		return BT_fileManagerKt.INSTANCE.copyAssetToCache(assetsDirectory, fileName);
  	}	
		
	// copies a file from the app's cache to the SDCARD..
	public static boolean copyFileFromCacheToSDCard(String fileName){
		return BT_fileManagerKt.INSTANCE.copyFileFromCacheToSDCard(fileName);
  	}		
	
	
	// returns a list of all the assets in a folder. Much faster to search an array than the file system!
	public static ArrayList<String> getAssetListFromDirectory(String assetsDirectory){
		return BT_fileManagerKt.INSTANCE.getAssetListFromDirectory(assetsDirectory);
	}
	
	
	// does file exist in applications download cache?
	public static boolean doesCachedFileExist(String fileName){
		return BT_fileManagerKt.INSTANCE.doesCachedFileExist(fileName);
	}	
	
	// returns resource id from projects bundle
	public static int getResourceIdFromBundle(String resourcesDirectory, String fileName){
		return BT_fileManagerKt.INSTANCE.getResourceIdFromBundle(resourcesDirectory, fileName);
	}
	
	// returns drawable by name
	public static Drawable getDrawableByName(String fileName){
		return BT_fileManagerKt.INSTANCE.getDrawableByName(fileName);
	}

	// get image (Bitmap) from URL...
	public static Bitmap getImageFromURL(String theUrl){
		BT_debugger.showIt(objectName + ": getImageFromURL: " + theUrl);
		Bitmap ret = null;
		try{
    		// download the bitmap...
            ret = BitmapFactory.decodeStream((InputStream)new URL(theUrl).getContent());
		}catch (Exception je){
			BT_debugger.showIt(objectName + ": An exception occurred trying to get an image from a URL: " + je.toString());
		}
		return ret;
	}
	
		
	// get image (drawable) from cache...
	public static Drawable getDrawableFromCache(String fileName){
		return BT_fileManagerKt.INSTANCE.getDrawableFromCache(fileName);
	}
	
	// get image (Bitmap) from cache...
	public static Bitmap getBitmapFromCache(String fileName){
		return BT_fileManagerKt.INSTANCE.getBitmapFromCache(fileName);
	}	
	
	// save image to cache
	public static void saveImageToCache(Bitmap theImage, String saveAsFileName){
		BT_fileManagerKt.INSTANCE.saveImageToCache(theImage, saveAsFileName);
	}
	
	// save file to cache
	public static void saveTextFileToCache(String theData, String saveAsFileName){
		BT_fileManagerKt.INSTANCE.saveTextFileToCache(theData, saveAsFileName);
	}	

	
	// get text file contents from project assets...
	public static String readTextFileFromAssets(String assetsDirectory, String fileName){
		return BT_fileManagerKt.INSTANCE.readTextFileFromAssets(assetsDirectory, fileName);
	}	
	
	// get text file contents from cache...
	public static String readTextFileFromCache(String fileName){
		return BT_fileManagerKt.INSTANCE.readTextFileFromCache(fileName);
	}
	
	// get size of cached data directory..
	public static int getCachedDataSize(){
		return BT_fileManagerKt.INSTANCE.getCachedDataSize();
	}

	/**
	 * Loads a raw resource and returns its content as a String.
	 *
	 * @throws IOException If any error was encountered, such as an incorrect resource ID, or
	 *                     inaccessible file.
	 */
	public static String parseResource(Context context, int resource) throws IOException {
		return BT_fileManagerKt.INSTANCE.parseResource(context, resource);
	}
		
	// deletes one cached file...
	public static void deleteFile(String fileName){
		BT_fileManagerKt.INSTANCE.deleteFile(fileName);
	}
	
	// deletes all cached files...
	public static void deleteAllCachedData(String skipFile){
		BT_fileManagerKt.INSTANCE.deleteAllCachedData(skipFile);
	}
}
