/*  File Version: 4.0.3
 *	Copyright, Chris Ruddell, buzztouch.com
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the
 *	following disclaimer.
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list
 *	of conditions and the following disclaimer in the documentation and/or other materials
 *	provided with the distribution.
 *
 *	Neither the name of David Book, or buzztouch.com nor the names of its contributors
 *	may be used to endorse or promote products derived from this software without specific
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 *	OF SUCH DAMAGE.
 */
package com.buzztouch.utils;

import org.json.JSONArray;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.ProtocolException;
import java.net.URL;


/**
 * Performs HTTP requests.  Preferred class over BT_downloader.
 */
public class BT_networkManager {
    private static final String TAG = "BT_networkManager";

    public enum RequestType {
        GET,
        PUT,
        POST,
        DELETE
    }

    public static void downloadTextContent(final String url, final BT_downloader.NetworkListener networkListener) {
        new Thread(new Runnable() {
            @Override
            public void run() {
                HttpSendObj sendObj = new HttpSendObj();
                HttpResponseObj responseObj = performHttpRequest(url, RequestType.GET, sendObj);
                networkListener.onNetworkReqeuestFinished(responseObj.statusCode, responseObj.responseText);
            }
        }).start();



    }

    public static HttpResponseObj performHttpRequest(String url, RequestType requestType, HttpSendObj httpSendObj) {
        BT_debugger.logInfo(TAG, "performhttpRequest(" + url + ")");
        HttpResponseObj responseObject = new HttpResponseObj();
        if(url.length() < 1) {
            return responseObject;
        } else {
            BufferedReader in = null;

            try {
                URL e = new URL(url);
                HttpURLConnection urlc = (HttpURLConnection)e.openConnection();
                switch(requestType) {
                    case GET:
                        urlc.setRequestMethod("GET");
                        break;
                    case PUT:
                        urlc.setRequestMethod("PUT");
                        break;
                    case POST:
                        urlc.setRequestMethod("POST");
                        break;
                    case DELETE:
                        urlc.setRequestMethod("DELETE");
                        break;
                    default:
                        urlc.setRequestMethod("GET");
                }

                urlc.setDoOutput(requestType == RequestType.POST || requestType == RequestType.PUT);
                urlc.setDoInput(true);
                urlc.setUseCaches(false);
                urlc.setAllowUserInteraction(requestType != RequestType.GET);
                urlc.setRequestProperty("Content-Type", requestType == RequestType.GET?"text/json":"application/json");
                urlc.setRequestProperty("Accept", requestType == RequestType.GET?"text/json":"application/json");
                String dataAsString = httpSendObj.encodedString;
                if(httpSendObj.jsonObject != null) {
                    dataAsString = httpSendObj.jsonObject.toString();
                } else if(httpSendObj.jsonArray != null) {
                    dataAsString = httpSendObj.jsonArray.toString();
                }

                if(requestType == RequestType.PUT || requestType == RequestType.POST) {
                    DataOutputStream status = new DataOutputStream(urlc.getOutputStream());
                    status.writeBytes(dataAsString);
                    status.flush();
                    status.close();
                }

                int status1 = urlc.getResponseCode();
                responseObject.statusCode = status1;
                boolean isError = false;
                if(status1 >= 300) {
                    isError = true;
                }

                in = new BufferedReader(new InputStreamReader(isError?urlc.getErrorStream():urlc.getInputStream()), 8096);
                StringBuilder sb = new StringBuilder();

                String line;
                while((line = in.readLine()) != null) {
                    sb.append(line);
                    if(status1 >= 300) {
                    }
                }

                responseObject.responseText = sb.toString();
            } catch (ProtocolException var22) {
                var22.printStackTrace();
            } catch (IOException var23) {
                var23.printStackTrace();
            } finally {
                if(in != null) {
                    try {
                        in.close();
                    } catch (IOException var21) {
                        var21.printStackTrace();
                    }
                }

            }

            responseObject.responseText = responseObject.responseText.length() > 0 ? responseObject.responseText : url.length() < 1 ? "Error:missing url" : "Error:" + responseObject.statusCode;
            return responseObject;
        }
    }

    public static class HttpSendObj {
        public String encodedString = "";
        public JSONObject jsonObject;
        public JSONArray jsonArray;

        public HttpSendObj() {
        }
    }

    public static class HttpResponseObj {
        public String responseText = "";
        public int statusCode = -1;
    }

}
