package com.buzztouch.utils

import android.content.Context
import androidx.core.content.edit
import com.buzztouch.buzztouch_appDelegate

object BT_preferencesKt {
    private const val PREF_NAME = "BT_prefs"
    private const val PREF_DATA_BOOSTRAP = "data_bootstrap"
    private const val PREF_CONFIG_VERSION = "config_version"

    fun isDataBoostrapDone(): Boolean = sharedPreferences.getBoolean(PREF_DATA_BOOSTRAP, false)
    fun setDataBoostrapDone(isDone: Boolean) {
        sharedPreferences.edit { putBoolean(PREF_DATA_BOOSTRAP, isDone) }
    }

    fun getConfigVersion(): Float = sharedPreferences.getFloat(PREF_CONFIG_VERSION, 0.1f)
    fun setConfigVersion(version: Float) {
        sharedPreferences.edit { putFloat(PREF_CONFIG_VERSION, version) }
    }

    fun setString(key: String, value: String) {
        sharedPreferences.edit { putString(key, value) }
    }

    fun getString(key: String, default: String? = null): String? = sharedPreferences?.getString(key, default)

    fun setBoolean(key: String, value: Boolean) {
        sharedPreferences.edit { putBoolean(key, value) }
    }

    fun getBoolean(key: String, default: Boolean = false) = sharedPreferences.getBoolean(key, default)

    fun setInt(key: String, value: Int) {
        sharedPreferences.edit { putInt(key, value) }
    }

    // returns int preference, or null if none could be found.  Uses Int.MIN_VALUE to determine none could be found
    fun getInt(key: String, default: Int = Int.MIN_VALUE): Int? = sharedPreferences.getInt(key, default).let { if (it == Int.MIN_VALUE) null else it }

    fun deleteAllLocalPrefs() {
        sharedPreferences.edit { clear() }
    }

    private val sharedPreferences get() = buzztouch_appDelegate.getApplication().getSharedPreferences(PREF_NAME, Context.MODE_PRIVATE)
}
